@@ -1,5 +1,5 @@
 /* GDA library
- * Copyright (C) 1998 - 2005 The GNOME Foundation.
+ * Copyright (C) 1998 - 2006 The GNOME Foundation.
  *
  * AUTHORS:
  *      Michael Lausch <michael@lausch.at>
@@ -33,6 +33,7 @@
 
 struct _GdaColumnPrivate {
 	gint         defined_size;
+	gchar       *id;
 	gchar       *name;
 	gchar       *title;
 
@@ -41,7 +42,7 @@ struct _GdaColumnPrivate {
 	gint         scale;
 
 	gchar       *dbms_type;
-	GdaValueType gda_type;
+	GType        g_type;
 
 	gboolean     allow_null;
 	gboolean     primary_key;
@@ -52,20 +53,38 @@ struct _GdaColumnPrivate {
 	glong        auto_increment_start;
 	glong        auto_increment_step;
 	gint         position;
-	GdaValue    *default_value;
-};
-
-enum {
-	NAME_CHANGED,
-	GDA_TYPE_CHANGED,
-	LAST_SIGNAL
+	GValue      *default_value;
 };
 
 static void gda_column_class_init (GdaColumnClass *klass);
 static void gda_column_init       (GdaColumn *column, GdaColumnClass *klass);
 static void gda_column_finalize   (GObject *object);
 
+static void gda_column_set_property (GObject *object,
+				     guint param_id,
+				     const GValue *value,
+				     GParamSpec *pspec);
+static void gda_column_get_property (GObject *object,
+				     guint param_id,
+				     GValue *value,
+				     GParamSpec *pspec);
+
+/* signals */
+enum {
+	NAME_CHANGED,
+	GDA_TYPE_CHANGED,
+	LAST_SIGNAL
+};
+
 static guint gda_column_signals[LAST_SIGNAL] = {0 , 0};
+
+/* properties */
+enum
+{
+        PROP_0,
+        PROP_ID,
+};
+
 static GObjectClass *parent_class = NULL;
 
 static void
@@ -75,6 +94,7 @@ gda_column_class_init (GdaColumnClass *klass)
 	
 	parent_class = g_type_class_peek_parent (klass);
 	
+	/* signals */
 	gda_column_signals[NAME_CHANGED] =
 		g_signal_new ("name_changed",
 			      G_TYPE_FROM_CLASS (object_class),
@@ -85,15 +105,23 @@ gda_column_class_init (GdaColumnClass *klass)
 			      G_TYPE_NONE,
 			      1, G_TYPE_STRING);
 	gda_column_signals[GDA_TYPE_CHANGED] =
-		g_signal_new ("gda_type_changed",
+		g_signal_new ("g_type_changed",
 			      G_TYPE_FROM_CLASS (object_class),
 			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (GdaColumnClass, gda_type_changed),
+			      G_STRUCT_OFFSET (GdaColumnClass, g_type_changed),
 			      NULL, NULL,
 			      gda_marshal_VOID__INT_INT,
 			      G_TYPE_NONE,
 			      2, G_TYPE_INT, G_TYPE_INT);
 
+	/* properties */
+        object_class->set_property = gda_column_set_property;
+        object_class->get_property = gda_column_get_property;
+
+        g_object_class_install_property (object_class, PROP_ID,
+                                         g_param_spec_string ("id", NULL, NULL,
+                                                              NULL, G_PARAM_WRITABLE | G_PARAM_READABLE));
+
 	object_class->finalize = gda_column_finalize;
 }
 
@@ -104,12 +132,13 @@ gda_column_init (GdaColumn *column, GdaColumnClass *klass)
 	
 	column->priv = g_new0 (GdaColumnPrivate, 1);
 	column->priv->defined_size = 0;
+	column->priv->id = NULL;
 	column->priv->name = NULL;
 	column->priv->table = NULL;
 	column->priv->title = NULL;
 	column->priv->caption = NULL;
 	column->priv->scale = 0;
-	column->priv->gda_type = GDA_VALUE_TYPE_UNKNOWN;
+	column->priv->g_type = G_TYPE_INVALID;
 	column->priv->allow_null = TRUE;
 	column->priv->primary_key = FALSE;
 	column->priv->unique_key = FALSE;
@@ -162,6 +191,50 @@ gda_column_get_type (void)
 	return type;
 }
 
+static void
+gda_column_set_property (GObject *object,
+                                   guint param_id,
+                                   const GValue *value,
+                                   GParamSpec *pspec)
+{
+        GdaColumn *col;
+
+        col = GDA_COLUMN (object);
+	if (col->priv) {
+		switch (param_id) {
+                case PROP_ID:
+			g_free (col->priv->id);
+			if (g_value_get_string (value))
+				col->priv->id = g_strdup (g_value_get_string (value));
+			else
+				col->priv->id = NULL;
+			break;
+		default:
+			g_assert_not_reached ();
+		}
+	}
+}
+
+static void
+gda_column_get_property (GObject *object,
+			 guint param_id,
+			 GValue *value,
+			 GParamSpec *pspec)
+{
+	GdaColumn *col;
+
+        col = GDA_COLUMN (object);
+	if (col->priv) {
+		switch (param_id) {
+                case PROP_ID:
+			g_value_set_string (value, col->priv->id);
+			break;
+		default:
+			g_assert_not_reached ();
+		}
+	}
+}
+
 /**
  * gda_column_new
  *
@@ -194,6 +267,8 @@ gda_column_copy (GdaColumn *column)
   	
 	column_copy = gda_column_new (); 	 
 	column_copy->priv->defined_size = column->priv->defined_size;
+	if (column->priv->id)
+		column_copy->priv->id = g_strdup (column->priv->id);
 	if (column->priv->name)
 		column_copy->priv->name = g_strdup (column->priv->name);
 	if (column->priv->title)
@@ -203,7 +278,7 @@ gda_column_copy (GdaColumn *column)
 	if (column->priv->caption)
 		column_copy->priv->caption = g_strdup (column->priv->caption);
 	column_copy->priv->scale = column->priv->scale;
-	column_copy->priv->gda_type = column->priv->gda_type;
+	column_copy->priv->g_type = column->priv->g_type;
 	column_copy->priv->allow_null = column->priv->allow_null;
 	column_copy->priv->primary_key = column->priv->primary_key;
 	column_copy->priv->unique_key = column->priv->unique_key;
@@ -219,81 +294,6 @@ gda_column_copy (GdaColumn *column)
 	return column_copy; 	 
 }
 
-/** 	 
- * gda_column_equal: 	 
- * @lhs: a #GdaColumn 	 
- * @rhs: another #GdaColumn 	 
- * 	 
- * Tests whether two colums are equal. 	 
- * 	 
- * Return value: %TRUE if the columns contain the same information. 	 
- */
-gboolean
-gda_column_equal (const GdaColumn *lhs, const GdaColumn *rhs) 	 
-{ 	 
-	g_return_val_if_fail (GDA_IS_COLUMN (lhs), FALSE); 	 
-	g_return_val_if_fail (GDA_IS_COLUMN (rhs), FALSE); 	 
-  	 
-	/* Compare every struct field: */ 	 
-	if ((lhs->priv->defined_size != rhs->priv->defined_size) || 	 
-	    (lhs->priv->scale != rhs->priv->scale) || 	 
-	    (lhs->priv->gda_type != rhs->priv->gda_type) || 	 
-	    (lhs->priv->allow_null != rhs->priv->allow_null) || 	 
-	    (lhs->priv->primary_key != rhs->priv->primary_key) || 	 
-	    (lhs->priv->unique_key != rhs->priv->unique_key) || 	 
-	    (lhs->priv->auto_increment != rhs->priv->auto_increment) || 	 
-	    (lhs->priv->auto_increment_step != rhs->priv->auto_increment_step) || 	 
-	    (lhs->priv->position != rhs->priv->position)) 	 
-		return FALSE; 	 
-  	 
-	/* Check the strings if they have are not null. 	 
-	   Then check whether one is null while the other is not, because strcmp can not do that. */ 	 
-	if ((lhs->priv->name && rhs->priv->name) 	 
-	    && (strcmp (lhs->priv->name, rhs->priv->name) != 0)) 	 
-		return FALSE; 	 
-  	 
-	if ((lhs->priv->name == 0) != (rhs->priv->name == 0)) 	 
-		return FALSE; 	 
-
-	if ((lhs->priv->title && rhs->priv->title) 	 
-	    && (strcmp (lhs->priv->title, rhs->priv->title) != 0)) 	 
-		return FALSE; 	 
-  	 
-	if ((lhs->priv->title == 0) != (rhs->priv->title == 0)) 	 
-		return FALSE; 	 
-  	 
-	if ((lhs->priv->table && rhs->priv->table) 	 
-	    && (strcmp (lhs->priv->table, rhs->priv->table) != 0)) 	 
-		return FALSE; 	 
-  	 
-	if ((lhs->priv->table == 0) != (rhs->priv->table == 0)) 	 
-		return FALSE; 	 
-  	 
-	if ((lhs->priv->caption && rhs->priv->caption) 	 
-	    && (strcmp (lhs->priv->caption, rhs->priv->caption) != 0)) 	 
-		return FALSE; 	 
-  	 
-	if ((lhs->priv->caption == 0) != (rhs->priv->caption == 0)) 	 
-		return FALSE; 	 
-  	 
-	if ((lhs->priv->references && rhs->priv->references) 	 
-	    && (strcmp (lhs->priv->references, rhs->priv->references) != 0)) 	 
-		return FALSE; 	 
-  	 
-	if ((lhs->priv->references == 0) != (rhs->priv->references == 0)) 	 
-		return FALSE; 	 
-  	 
-	if (lhs->priv->default_value 	 
-	    && rhs->priv->default_value 	 
-	    && gda_value_compare (lhs->priv->default_value, rhs->priv->default_value) != 0) 	 
-		return FALSE; 	 
-  	 
-	if ((lhs->priv->default_value == 0) != (rhs->priv->default_value == 0)) 	 
-		return FALSE; 	 
-  	 
-	return TRUE; 	 
- }
-
 /**
  * gda_column_get_defined_size
  * @column: a @GdaColumn.
@@ -510,34 +510,34 @@ gda_column_set_dbms_type (GdaColumn *column, const gchar *dbms_type)
 }
 
 /**
- * gda_column_get_gda_type
+ * gda_column_get_g_type
  * @column: a #GdaColumn.
  *
  * Returns: the type of @column.
  */
-GdaValueType
-gda_column_get_gda_type (GdaColumn *column)
+GType
+gda_column_get_g_type (GdaColumn *column)
 {
-	g_return_val_if_fail (GDA_IS_COLUMN (column), GDA_VALUE_TYPE_NULL);
-	return column->priv->gda_type;
+	g_return_val_if_fail (GDA_IS_COLUMN (column), GDA_TYPE_NULL);
+	return column->priv->g_type;
 }
 
 /**
- * gda_column_set_gda_type
+ * gda_column_set_g_type
  * @column: a #GdaColumn.
  * @type: the new type of @column.
  *
  * Sets the type of @column to @type.
  */
 void
-gda_column_set_gda_type (GdaColumn *column, GdaValueType type)
+gda_column_set_g_type (GdaColumn *column, GType type)
 {
-	GdaValueType old_type;
+	GType old_type;
 
 	g_return_if_fail (GDA_IS_COLUMN (column));
 
-	old_type = column->priv->gda_type;
-	column->priv->gda_type = type;
+	old_type = column->priv->g_type;
+	column->priv->g_type = type;
 
 	g_signal_emit (G_OBJECT (column),
 		       gda_column_signals[GDA_TYPE_CHANGED],
@@ -726,29 +726,29 @@ gda_column_set_position (GdaColumn *column, gint position)
  * gda_column_get_default_value
  * @column: a #GdaColumn.
  *
- * Returns: @column's default value, as a #GdaValue object.
+ * Returns: @column's default value, as a #GValue object.
  */
-const GdaValue *
+const GValue *
 gda_column_get_default_value (GdaColumn *column)
 {
 	g_return_val_if_fail (GDA_IS_COLUMN (column), NULL);
-	return (const GdaValue *) column->priv->default_value;
+	return (const GValue *) column->priv->default_value;
 }
 
 /**
  * gda_column_set_default_value
  * @column: a #GdaColumn.
- * @default_value: default #GdaValue for the column
+ * @default_value: default #GValue for the column
  *
- * Sets @column's default #GdaValue.
+ * Sets @column's default #GValue.
  */
 void
-gda_column_set_default_value (GdaColumn *column, const GdaValue *default_value)
+gda_column_set_default_value (GdaColumn *column, const GValue *default_value)
 {
 	g_return_if_fail (GDA_IS_COLUMN (column));
 	g_return_if_fail (default_value != NULL);
 
 	if (column->priv->default_value)
 		g_free (column->priv->default_value);
-	column->priv->default_value = gda_value_copy ( (GdaValue*)default_value);
+	column->priv->default_value = gda_value_copy ( (GValue*)default_value);
 }
