@@ -27,6 +27,7 @@
 #include <libgda/gda-client.h>
 #include <libgda/gda-config.h>
 #include <libgda/gda-connection.h>
+#include <libgda/gda-connection-private.h>
 #include <libgda/gda-connection-event.h>
 #include <glib/gi18n-lib.h>
 #include <libgda/gda-dict.h>
@@ -34,6 +35,9 @@
 #include <libgda/gda-server-provider.h>
 #include <libgda/gda-parameter-list.h>
 #include "gda-marshal.h"
+#include <libgda/gda-transaction-status-private.h>
+#include <string.h>
+#include <libgda/sql-transaction/gda-sql-transaction-parser.h>
 
 #define PARENT_TYPE G_TYPE_OBJECT
 
@@ -49,10 +53,13 @@ struct _GdaConnectionPrivate {
 	gboolean              is_open;
 	GList                *events_list;
 	GList                *recset_list;
+
+	GdaTransactionStatus *trans_status;
 };
 
 static void gda_connection_class_init (GdaConnectionClass *klass);
 static void gda_connection_init       (GdaConnection *cnc, GdaConnectionClass *klass);
+static void gda_connection_dispose    (GObject *object);
 static void gda_connection_finalize   (GObject *object);
 static void gda_connection_set_property (GObject *object,
 					 guint param_id,
@@ -69,10 +76,11 @@ enum {
         CONN_TO_CLOSE,
         CONN_CLOSED,
 	DSN_CHANGED,
+	TRANSACTION_STATUS_CHANGED,
 	LAST_SIGNAL
 };
 
-static gint gda_connection_signals[LAST_SIGNAL] = { 0, 0, 0, 0, 0 };
+static gint gda_connection_signals[LAST_SIGNAL] = { 0, 0, 0, 0, 0, 0 };
 
 /* properties */
 enum
@@ -140,13 +148,22 @@ gda_connection_class_init (GdaConnectionClass *klass)
 			      NULL, NULL,
 			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE, 0);
+	gda_connection_signals[TRANSACTION_STATUS_CHANGED] =
+		g_signal_new ("transaction_status_changed",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (GdaConnectionClass, transaction_status_changed),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE, 0);
 
 	/* Properties */
         object_class->set_property = gda_connection_set_property;
         object_class->get_property = gda_connection_get_property;
 
 	g_object_class_install_property (object_class, PROP_CLIENT,
-                                         g_param_spec_pointer ("client", _("GdaClient to use"), NULL,
+                                         g_param_spec_object ("client", _("GdaClient to use"), NULL,
+                                                               GDA_TYPE_CLIENT,
 							       (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 	g_object_class_install_property (object_class, PROP_DSN,
                                          g_param_spec_string ("dsn", _("DSN to use"), NULL, NULL,
@@ -155,7 +172,8 @@ gda_connection_class_init (GdaConnectionClass *klass)
                                          g_param_spec_string ("cnc_string", _("Connection string to use"), NULL, NULL,
 							      (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 	g_object_class_install_property (object_class, PROP_PROVIDER_OBJ,
-                                         g_param_spec_pointer ("provider_obj", _("Provider to use"), NULL,
+                                         g_param_spec_object ("provider_obj", _("Provider to use"), NULL,
+                                                               GDA_TYPE_SERVER_PROVIDER,
 							       (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 
         g_object_class_install_property (object_class, PROP_USERNAME,
@@ -171,6 +189,7 @@ gda_connection_class_init (GdaConnectionClass *klass)
 							    NULL, 0, G_MAXUINT, 0,
 							    (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 	
+	object_class->dispose = gda_connection_dispose;
 	object_class->finalize = gda_connection_finalize;
 }
 
@@ -189,6 +208,37 @@ gda_connection_init (GdaConnection *cnc, GdaConnectionClass *klass)
 	cnc->priv->is_open = FALSE;
 	cnc->priv->events_list = NULL;
 	cnc->priv->recset_list = NULL;
+	cnc->priv->trans_status = NULL; /* no transaction yet */
+}
+
+static void
+gda_connection_dispose (GObject *object)
+{
+	GdaConnection *cnc = (GdaConnection *) object;
+
+	g_return_if_fail (GDA_IS_CONNECTION (cnc));
+
+	/* free memory */
+	gda_connection_close_no_warning (cnc);
+
+	if (cnc->priv->provider_obj) {
+		g_object_unref (G_OBJECT (cnc->priv->provider_obj));
+		cnc->priv->provider_obj = NULL;
+	}
+
+	if (cnc->priv->events_list)
+		gda_connection_event_list_free (cnc->priv->events_list);
+
+	if (cnc->priv->recset_list)
+		g_list_foreach (cnc->priv->recset_list, (GFunc) g_object_unref, NULL);
+	
+	if (cnc->priv->trans_status) {
+		g_object_unref (cnc->priv->trans_status);
+		cnc->priv->trans_status = NULL;
+	}
+
+	/* chain to parent class */
+	parent_class->dispose (object);
 }
 
 static void
@@ -199,20 +249,11 @@ gda_connection_finalize (GObject *object)
 	g_return_if_fail (GDA_IS_CONNECTION (cnc));
 
 	/* free memory */
-	gda_connection_close_no_warning (cnc);
-
-	g_object_unref (G_OBJECT (cnc->priv->provider_obj));
-	cnc->priv->provider_obj = NULL;
-
 	g_free (cnc->priv->dsn);
 	g_free (cnc->priv->cnc_string);
 	g_free (cnc->priv->username);
 	g_free (cnc->priv->password);
 
-	gda_connection_event_list_free (cnc->priv->events_list);
-
-	g_list_foreach (cnc->priv->recset_list, (GFunc) g_object_unref, NULL);
-
 	g_free (cnc->priv);
 	cnc->priv = NULL;
 
@@ -269,7 +310,12 @@ gda_connection_set_property (GObject *object,
         if (cnc->priv) {
                 switch (param_id) {
                 case PROP_CLIENT:
-			cnc->priv->client = g_value_get_pointer (value);
+                        if(cnc->priv->client)
+                          g_object_unref(cnc->priv->client);
+
+			cnc->priv->client = g_value_get_object (value);
+                        g_object_ref(cnc->priv->client);
+
 			break;
                 case PROP_DSN:
 			gda_connection_set_dsn (cnc, g_value_get_string (value));
@@ -281,7 +327,10 @@ gda_connection_set_property (GObject *object,
 				cnc->priv->cnc_string = g_strdup (g_value_get_string (value));
                         break;
                 case PROP_PROVIDER_OBJ:
-			cnc->priv->provider_obj = g_value_get_pointer (value);
+                        if (cnc->priv->provider_obj)
+                          g_object_unref(cnc->priv->provider_obj);
+
+			cnc->priv->provider_obj = g_value_get_object (value);
 			g_object_ref (G_OBJECT (cnc->priv->provider_obj));
                         break;
                 case PROP_USERNAME:
@@ -309,7 +358,7 @@ gda_connection_get_property (GObject *object,
         if (cnc->priv) {
                 switch (param_id) {
                 case PROP_CLIENT:
-			g_value_set_pointer (value, cnc->priv->client);
+			g_value_set_object (value, G_OBJECT (cnc->priv->client));
 			break;
                 case PROP_DSN:
 			g_value_set_string (value, cnc->priv->dsn);
@@ -318,7 +367,7 @@ gda_connection_get_property (GObject *object,
 			g_value_set_string (value, cnc->priv->cnc_string);
 			break;
                 case PROP_PROVIDER_OBJ:
-			g_value_set_pointer (value, cnc->priv->provider_obj);
+			g_value_set_object (value, G_OBJECT (cnc->priv->provider_obj));
                         break;
                 case PROP_USERNAME:
 			g_value_set_string (value, cnc->priv->username);
@@ -1246,30 +1295,32 @@ gda_connection_execute_non_select_command (GdaConnection *cnc, GdaCommand *cmd,
 /**
  * gda_connection_begin_transaction
  * @cnc: a #GdaConnection object.
- * @xaction: a #GdaTransaction object.
+ * @name: the name of the transation to start
+ * @level:
+ * @error: a place to store errors, or %NULL
  *
  * Starts a transaction on the data source, identified by the
  * @xaction parameter.
  *
  * Before starting a transaction, you can check whether the underlying
  * provider does support transactions or not by using the
- * #gda_connection_supports_feature function.
+ * #gda_connection_supports_feature() function.
  *
  * Returns: %TRUE if the transaction was started successfully, %FALSE
  * otherwise.
  */
 gboolean
-gda_connection_begin_transaction (GdaConnection *cnc, GdaTransaction *xaction)
+gda_connection_begin_transaction (GdaConnection *cnc, const gchar *name, GdaTransactionIsolation level,
+				  GError **error)
 {
 	gboolean retval;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
 	g_return_val_if_fail (cnc->priv, FALSE);
-	g_return_val_if_fail (GDA_IS_TRANSACTION (xaction), FALSE);
 
-	retval = gda_server_provider_begin_transaction (cnc->priv->provider_obj, cnc, xaction);
+	retval = gda_server_provider_begin_transaction (cnc->priv->provider_obj, cnc, name, level, error);
 	if (retval)
-		gda_client_notify_transaction_started_event (cnc->priv->client, cnc, xaction);
+		gda_client_notify_event (cnc->priv->client, cnc, GDA_CLIENT_EVENT_TRANSACTION_STARTED, NULL);
 
 	return retval;
 }
@@ -1277,26 +1328,26 @@ gda_connection_begin_transaction (GdaConnection *cnc, GdaTransaction *xaction)
 /**
  * gda_connection_commit_transaction
  * @cnc: a #GdaConnection object.
- * @xaction: a #GdaTransaction object.
+ * @name: the name of the transation to commit
+ * @error: a place to store errors, or %NULL
  *
- * Commits the given transaction to the backend database.  You need to do 
+ * Commits the given transaction to the backend database. You need to call
  * gda_connection_begin_transaction() first.
  *
  * Returns: %TRUE if the transaction was finished successfully,
  * %FALSE otherwise.
  */
 gboolean
-gda_connection_commit_transaction (GdaConnection *cnc, GdaTransaction *xaction)
+gda_connection_commit_transaction (GdaConnection *cnc, const gchar *name, GError **error)
 {
 	gboolean retval;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
 	g_return_val_if_fail (cnc->priv, FALSE);
-	g_return_val_if_fail (GDA_IS_TRANSACTION (xaction), FALSE);
 
-	retval = gda_server_provider_commit_transaction (cnc->priv->provider_obj, cnc, xaction);
+	retval = gda_server_provider_commit_transaction (cnc->priv->provider_obj, cnc, name, error);
 	if (retval)
-		gda_client_notify_transaction_committed_event (cnc->priv->client, cnc, xaction);
+		gda_client_notify_event (cnc->priv->client, cnc, GDA_CLIENT_EVENT_TRANSACTION_COMMITTED, NULL);
 
 	return retval;
 }
@@ -1304,31 +1355,108 @@ gda_connection_commit_transaction (GdaConnection *cnc, GdaTransaction *xaction)
 /**
  * gda_connection_rollback_transaction
  * @cnc: a #GdaConnection object.
- * @xaction: a #GdaTransaction object.
+ * @name: the name of the transation to commit
+ * @error: a place to store errors, or %NULL
  *
  * Rollbacks the given transaction. This means that all changes
  * made to the underlying data source since the last call to
- * #gda_connection_begin_transaction or #gda_connection_commit_transaction
+ * #gda_connection_begin_transaction() or #gda_connection_commit_transaction()
  * will be discarded.
  *
  * Returns: %TRUE if the operation was successful, %FALSE otherwise.
  */
 gboolean
-gda_connection_rollback_transaction (GdaConnection *cnc, GdaTransaction *xaction)
+gda_connection_rollback_transaction (GdaConnection *cnc, const gchar *name, GError **error)
 {
 	gboolean retval;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
 	g_return_val_if_fail (cnc->priv, FALSE);
-	g_return_val_if_fail (GDA_IS_TRANSACTION (xaction), FALSE);
 
-	retval = gda_server_provider_rollback_transaction (cnc->priv->provider_obj, cnc, xaction);
+	retval = gda_server_provider_rollback_transaction (cnc->priv->provider_obj, cnc, name, error);
 	if (retval)
-		gda_client_notify_transaction_cancelled_event (cnc->priv->client, cnc, xaction);
+		gda_client_notify_event (cnc->priv->client, cnc, GDA_CLIENT_EVENT_TRANSACTION_CANCELLED, NULL);
 
 	return retval;
 }
 
+/**
+ * gda_connection_add_savepoint
+ * @cnc: a #GdaConnection object
+ * @name: name of the savepoint to add
+ * @error: a place to store errors or %NULL
+ *
+ * Adds a SAVEPOINT named @name.
+ *
+ * Returns: TRUE if no error occurred
+ */
+gboolean
+gda_connection_add_savepoint (GdaConnection *cnc, const gchar *name, GError **error)
+{
+	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
+	g_return_val_if_fail (cnc->priv, FALSE);
+	
+	return gda_server_provider_add_savepoint (cnc->priv->provider_obj, cnc, name, error);
+}
+
+/**
+ * gda_connection_rollback_savepoint
+ * @cnc: a #GdaConnection object
+ * @name: name of the savepoint to rollback to
+ * @error: a place to store errors or %NULL
+ *
+ * Rollback all the modifications made after the SAVEPOINT named @name.
+ *
+ * Returns: TRUE if no error occurred
+ */
+gboolean
+gda_connection_rollback_savepoint (GdaConnection *cnc, const gchar *name, GError **error)
+{
+	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
+	g_return_val_if_fail (cnc->priv, FALSE);
+	
+	return gda_server_provider_rollback_savepoint (cnc->priv->provider_obj, cnc, name, error);
+}
+
+/**
+ * gda_connection_delete_savepoint
+ * @cnc: a #GdaConnection object
+ * @name: name of the savepoint to delete
+ * @error: a place to store errors or %NULL
+ *
+ * Delete the SAVEPOINT named @name when not used anymore.
+ *
+ * Returns: TRUE if no error occurred
+ */
+gboolean
+gda_connection_delete_savepoint (GdaConnection *cnc, const gchar *name, GError **error)
+{
+	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
+	g_return_val_if_fail (cnc->priv, FALSE);
+	
+	return gda_server_provider_delete_savepoint (cnc->priv->provider_obj, cnc, name, error);
+}
+
+/**
+ * gda_connection_get_transaction_status
+ * @cnc: a #GdaConnection object
+ *
+ * Get the status of @cnc regarding transactions. The returned object should not be modified
+ * or destroyed; however it may be modified or destroyed by the connection itself.
+ *
+ * If %NULL is returned, then no transaction has been associated with @cnc
+ *
+ * Returns: a #GdaTransactionStatus object, or %NULL
+ */
+GdaTransactionStatus *
+gda_connection_get_transaction_status (GdaConnection *cnc)
+{
+	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
+	g_return_val_if_fail (cnc->priv, NULL);
+
+	return cnc->priv->trans_status;
+}
+
 /**
  * gda_connection_supports_feature
  * @cnc: a #GdaConnection object.
@@ -1456,3 +1584,243 @@ gda_connection_value_to_sql_string (GdaConnection *cnc, GValue *from)
 	/* execute the command on the provider */
 	return gda_server_provider_value_to_sql_string (cnc->priv->provider_obj, cnc, from);
 }
+
+/*
+ * Internal functions to keep track
+ * of the transactional status of the connection
+ */
+void
+gda_connection_internal_transaction_started (GdaConnection *cnc, const gchar *parent_trans, const gchar *trans_name, 
+					     GdaTransactionIsolation isol_level)
+{
+	GdaTransactionStatus *parent, *st;
+
+	st = gda_transaction_status_new (trans_name);
+	st->isolation_level = isol_level;
+	parent = gda_transaction_status_find (cnc->priv->trans_status, parent_trans, NULL);
+	if (!parent)
+		cnc->priv->trans_status = st;
+	else {
+		gda_transaction_status_add_event_sub (parent, st);
+		g_object_unref (st);
+	}
+#ifdef GDA_DEBUG_signal
+        g_print (">> 'TRANSACTION_STATUS_CHANGED' from %s\n", __FUNCTION__);
+#endif
+        g_signal_emit (G_OBJECT (cnc), gda_connection_signals[TRANSACTION_STATUS_CHANGED], 0);
+#ifdef GDA_DEBUG_signal
+        g_print ("<< 'TRANSACTION_STATUS_CHANGED' from %s\n", __FUNCTION__);
+#endif
+
+#ifdef GDA_DEBUG_NO
+	if (cnc->priv->trans_status)
+		gda_transaction_status_dump (cnc->priv->trans_status, 5);
+#endif
+}
+
+void 
+gda_connection_internal_transaction_rolledback (GdaConnection *cnc, const gchar *trans_name)
+{
+	GdaTransactionStatus *st = NULL;
+	GdaTransactionStatusEvent *ev = NULL;
+
+	if (cnc->priv->trans_status)
+		st = gda_transaction_status_find (cnc->priv->trans_status, trans_name, &ev);
+	if (st) {
+		if (ev) {
+			/* there is a parent transaction */
+			gda_transaction_status_free_events (ev->trans, ev, TRUE);
+		}
+		else {
+			/* no parent transaction */
+			g_object_unref (cnc->priv->trans_status);
+			cnc->priv->trans_status = NULL;
+		}
+#ifdef GDA_DEBUG_signal
+		g_print (">> 'TRANSACTION_STATUS_CHANGED' from %s\n", __FUNCTION__);
+#endif
+		g_signal_emit (G_OBJECT (cnc), gda_connection_signals[TRANSACTION_STATUS_CHANGED], 0);
+#ifdef GDA_DEBUG_signal
+		g_print ("<< 'TRANSACTION_STATUS_CHANGED' from %s\n", __FUNCTION__);
+#endif
+	}
+	else
+		g_warning (_("Connection transaction status tracking: no transaction exists for ROLLBACK"));
+#ifdef GDA_DEBUG_NO
+	if (cnc->priv->trans_status)
+		gda_transaction_status_dump (cnc->priv->trans_status, 5);
+#endif
+}
+
+void 
+gda_connection_internal_transaction_committed (GdaConnection *cnc, const gchar *trans_name)
+{
+	GdaTransactionStatus *st = NULL;
+	GdaTransactionStatusEvent *ev = NULL;
+
+	if (cnc->priv->trans_status)
+		st = gda_transaction_status_find (cnc->priv->trans_status, trans_name, &ev);
+	if (st) {
+		if (ev) {
+			/* there is a parent transaction */
+			gda_transaction_status_free_events (ev->trans, ev, TRUE);
+		}
+		else {
+			/* no parent transaction */
+			g_object_unref (cnc->priv->trans_status);
+			cnc->priv->trans_status = NULL;
+		}
+#ifdef GDA_DEBUG_signal
+		g_print (">> 'TRANSACTION_STATUS_CHANGED' from %s\n", __FUNCTION__);
+#endif
+		g_signal_emit (G_OBJECT (cnc), gda_connection_signals[TRANSACTION_STATUS_CHANGED], 0);
+#ifdef GDA_DEBUG_signal
+		g_print ("<< 'TRANSACTION_STATUS_CHANGED' from %s\n", __FUNCTION__);
+#endif
+	}
+	else
+		g_warning (_("Connection transaction status tracking: no transaction exists for COMMIT"));
+#ifdef GDA_DEBUG_NO
+	if (cnc->priv->trans_status)
+		gda_transaction_status_dump (cnc->priv->trans_status, 5);
+#endif
+}
+
+void
+gda_connection_internal_sql_executed (GdaConnection *cnc, const gchar *sql, GdaConnectionEvent *error)
+{
+	GdaTransactionStatus *st = NULL;
+	
+	if (cnc->priv->trans_status)
+		st = gda_transaction_status_find_current (cnc->priv->trans_status, NULL, FALSE);
+	if (st)
+		gda_transaction_status_add_event_sql (st, sql, error);
+#ifdef GDA_DEBUG_signal
+		g_print (">> 'TRANSACTION_STATUS_CHANGED' from %s\n", __FUNCTION__);
+#endif
+		g_signal_emit (G_OBJECT (cnc), gda_connection_signals[TRANSACTION_STATUS_CHANGED], 0);
+#ifdef GDA_DEBUG_signal
+		g_print ("<< 'TRANSACTION_STATUS_CHANGED' from %s\n", __FUNCTION__);
+#endif
+#ifdef GDA_DEBUG_NO
+	if (cnc->priv->trans_status)
+		gda_transaction_status_dump (cnc->priv->trans_status, 5);
+#endif
+}
+
+void
+gda_connection_internal_savepoint_added (GdaConnection *cnc, const gchar *parent_trans, const gchar *svp_name)
+{
+	GdaTransactionStatus *st;
+
+	st = gda_transaction_status_find (cnc->priv->trans_status, parent_trans, NULL);
+	if (st) {
+		gda_transaction_status_add_event_svp (st, svp_name);
+#ifdef GDA_DEBUG_signal
+		g_print (">> 'TRANSACTION_STATUS_CHANGED' from %s\n", __FUNCTION__);
+#endif
+		g_signal_emit (G_OBJECT (cnc), gda_connection_signals[TRANSACTION_STATUS_CHANGED], 0);
+#ifdef GDA_DEBUG_signal
+		g_print ("<< 'TRANSACTION_STATUS_CHANGED' from %s\n", __FUNCTION__);
+#endif
+	}
+	else
+		g_warning (_("Connection transaction status tracking: no transaction exists for ADD SAVEPOINT"));
+#ifdef GDA_DEBUG_NO
+	if (cnc->priv->trans_status)
+		gda_transaction_status_dump (cnc->priv->trans_status, 5);
+#endif
+}
+
+void
+gda_connection_internal_savepoint_rolledback (GdaConnection *cnc, const gchar *svp_name)
+{
+	GdaTransactionStatus *st;
+	GdaTransactionStatusEvent *ev = NULL;
+
+	st = gda_transaction_status_find (cnc->priv->trans_status, svp_name, &ev);
+	if (st) {
+		gda_transaction_status_free_events (st, ev, TRUE);
+#ifdef GDA_DEBUG_signal
+		g_print (">> 'TRANSACTION_STATUS_CHANGED' from %s\n", __FUNCTION__);
+#endif
+		g_signal_emit (G_OBJECT (cnc), gda_connection_signals[TRANSACTION_STATUS_CHANGED], 0);
+#ifdef GDA_DEBUG_signal
+		g_print ("<< 'TRANSACTION_STATUS_CHANGED' from %s\n", __FUNCTION__);
+#endif
+	}
+	else
+		g_warning (_("Connection transaction status tracking: no transaction exists for ROLLBACK SAVEPOINT"));
+#ifdef GDA_DEBUG_NO
+	if (cnc->priv->trans_status)
+		gda_transaction_status_dump (cnc->priv->trans_status, 5);
+#endif	
+}
+
+void
+gda_connection_internal_savepoint_removed (GdaConnection *cnc, const gchar *svp_name)
+{
+	GdaTransactionStatus *st;
+	GdaTransactionStatusEvent *ev = NULL;
+
+	st = gda_transaction_status_find (cnc->priv->trans_status, svp_name, &ev);
+	if (st) {
+		gda_transaction_status_free_events (st, ev, FALSE);
+#ifdef GDA_DEBUG_signal
+		g_print (">> 'TRANSACTION_STATUS_CHANGED' from %s\n", __FUNCTION__);
+#endif
+		g_signal_emit (G_OBJECT (cnc), gda_connection_signals[TRANSACTION_STATUS_CHANGED], 0);
+#ifdef GDA_DEBUG_signal
+		g_print ("<< 'TRANSACTION_STATUS_CHANGED' from %s\n", __FUNCTION__);
+#endif
+	}
+	else
+		g_warning (_("Connection transaction status tracking: no transaction exists for REMOVE SAVEPOINT"));
+#ifdef GDA_DEBUG_NO
+	if (cnc->priv->trans_status)
+		gda_transaction_status_dump (cnc->priv->trans_status, 5);
+#endif		
+}
+
+
+void
+gda_connection_internal_treat_sql (GdaConnection *cnc, const gchar *sql, GdaConnectionEvent *error)
+{
+	gboolean done = FALSE;
+
+	if (!error || (error && (gda_connection_event_get_event_type (error) != GDA_CONNECTION_EVENT_ERROR))) {
+		GdaSqlTransaction *trans;
+
+		trans = gda_sql_transaction_parse_with_error (sql, NULL);
+		if (trans) {
+			switch (trans->trans_type) {
+			case GDA_SQL_TRANSACTION_BEGIN:
+				gda_connection_internal_transaction_started (cnc, NULL, trans->trans_name, 
+									     GDA_TRANSACTION_ISOLATION_UNKNOWN);
+				break;
+			case GDA_SQL_TRANSACTION_COMMIT:
+				gda_connection_internal_transaction_committed (cnc, trans->trans_name);
+				break;
+			case GDA_SQL_TRANSACTION_ROLLBACK:
+				gda_connection_internal_transaction_rolledback (cnc, trans->trans_name);
+				break;
+			case GDA_SQL_TRANSACTION_SAVEPOINT_ADD:
+				gda_connection_internal_savepoint_added (cnc, NULL, trans->trans_name);
+				break;
+			case GDA_SQL_TRANSACTION_SAVEPOINT_REMOVE:
+				gda_connection_internal_savepoint_removed (cnc, trans->trans_name);
+				break;
+			case GDA_SQL_TRANSACTION_SAVEPOINT_ROLLBACK:
+				gda_connection_internal_savepoint_rolledback (cnc, trans->trans_name);
+				break;
+			default:
+				g_assert_not_reached ();
+			}
+			gda_sql_transaction_destroy (trans);
+			done = TRUE;
+		}
+	}
+	
+	if (!done)
+		gda_connection_internal_sql_executed (cnc, sql, error);
+}
