@@ -225,7 +225,8 @@ gda_data_model_import_class_init (GdaDataModelImportClass *klass)
 							      G_PARAM_READABLE | G_PARAM_WRITABLE |
 							      G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_install_property (object_class, PROP_OPTIONS,
-                                         g_param_spec_pointer ("options", "Options to configure the import", NULL,
+                                         g_param_spec_object ("options", "Options to configure the import", NULL,
+                                                               GDA_TYPE_PARAMETER_LIST,
 							       G_PARAM_READABLE | G_PARAM_WRITABLE |
 							       G_PARAM_CONSTRUCT_ONLY));
 
@@ -454,7 +455,6 @@ gda_data_model_import_set_property (GObject *object,
 {
 	GdaDataModelImport *model;
 	const gchar *string;
-	gpointer data;
 
 	model = GDA_DATA_MODEL_IMPORT (object);
 	if (model->priv) {
@@ -511,15 +511,19 @@ gda_data_model_import_set_property (GObject *object,
 			model->priv->data_start = model->priv->src.string;
 			model->priv->data_length = strlen (model->priv->src.string);
 			break;
-		case PROP_XML_NODE:
-			data = g_value_get_pointer (value);
+		case PROP_XML_NODE: {
+			gpointer data = g_value_get_pointer (value);
 			if (!data)
 				return;
 			model->priv->format = FORMAT_XML_NODE;
-			model->priv->extract.node.node = g_value_get_pointer (value);
+			model->priv->extract.node.node = data;
 			break;
+                }
 		case PROP_OPTIONS:
-			model->priv->options = g_value_get_pointer (value);
+                        if(model->priv->options)
+                          g_object_unref(model->priv->options);
+
+			model->priv->options = g_value_get_object (value);
 			if (model->priv->options) {
 				if (!GDA_IS_PARAMETER_LIST (model->priv->options)) {
 					g_warning (_("\"options\" property is not a GdaParameterList object"));
@@ -738,7 +742,7 @@ gda_data_model_import_dump (GdaDataModelImport *model, guint offset)
 		for (i = 0; i < n_cols; i++) {
 			GdaColumn *col = gda_data_model_describe_column (GDA_DATA_MODEL (model), i);
                         g_print ("%sModel col %d has type %s\n", stroff, i, 
-				 g_type_to_string (gda_column_get_g_type (col)));
+				 gda_g_type_to_string (gda_column_get_g_type (col)));
 		}
 
 		for (i = 0; i < n_cols; i++) {
@@ -1129,7 +1133,7 @@ csv_compute_row_values (GdaDataModelImport *model)
 			gchar *str;
 
 			str = g_strdup_printf (_("Could not convert '%s' to a value of type %s"), 
-					       *arrvalue, g_type_to_string (gtype));
+					       *arrvalue, gda_g_type_to_string (gtype));
 			add_error (model, str);
 			g_free (str);
 			value = gda_value_new_null ();
@@ -1331,7 +1335,7 @@ init_xml_import (GdaDataModelImport *model)
 			spec->dbms_type = xmlTextReaderGetAttribute (reader, "dbms_type");
 			str = xmlTextReaderGetAttribute (reader, "gdatype");
 			if (str) {
-				spec->gdatype = g_type_from_string (str);
+				spec->gdatype = gda_g_type_from_string (str);
 				xmlFree (str);
 			}
 			else {
@@ -1536,13 +1540,13 @@ xml_fetch_next_row (GdaDataModelImport *model)
 
 					gtype = gda_column_get_g_type (column);
 					xmlstr = xmlTextReaderConstValue (reader);
-					/* g_print ("Convert #%s# to %s\n", (gchar *) xmlstr, g_type_to_string (gtype)); */
+					/* g_print ("Convert #%s# to %s\n", (gchar *) xmlstr, gda_g_type_to_string (gtype)); */
 					value = gda_value_new_from_string ((gchar *) xmlstr, gtype);
 					if (!value) {
 						gchar *str;
 						
 						str = g_strdup_printf (_("Could not convert '%s' to a value of type %s"), 
-								       (gchar *) xmlstr, g_type_to_string (gtype));
+								       (gchar *) xmlstr, gda_g_type_to_string (gtype));
 						add_error (model, str);
 						g_free (str);
 						value = gda_value_new_null ();
@@ -1622,7 +1626,7 @@ init_node_import (GdaDataModelImport *model)
 			spec->dbms_type = xmlGetProp (cur, "dbms_type");
 			str = xmlGetProp (cur, "gdatype");
 			if (str) {
-				spec->gdatype = g_type_from_string (str);
+				spec->gdatype = gda_g_type_from_string (str);
 				xmlFree (str);
 			}
 			else {
