@@ -503,11 +503,11 @@ gda_data_model_get_value_at_col_name (GdaDataModel *model,
  * @row: a valid row number, or -1
  *
  * Get the attributes of the value stored at (row, col) in @proxy, which
- * is an ORed value of #GValueAttribute flags. As a special case, if
+ * is an ORed value of #GdaValueAttribute flags. As a special case, if
  * @row is -1, then the attributes returned correspond to a "would be" value
  * if a row was added to @model.
  *
- * Returns: the attributes as an ORed value of #GValueAttribute
+ * Returns: the attributes as an ORed value of #GdaValueAttribute
  */
 guint
 gda_data_model_get_attributes_at (GdaDataModel *model, gint col, gint row)
@@ -610,6 +610,10 @@ gboolean
 gda_data_model_move_iter_at_row (GdaDataModel *model, GdaDataModelIter *iter, gint row)
 {
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
+
+	if ((gda_data_model_iter_get_row (iter) != row) && ! gda_data_model_iter_can_be_moved (iter))
+		return FALSE;
+
 	if (GDA_DATA_MODEL_GET_CLASS (model)->i_iter_at_row)
 		return (GDA_DATA_MODEL_GET_CLASS (model)->i_iter_at_row) (model, iter, row);
 	else 
@@ -617,6 +621,8 @@ gda_data_model_move_iter_at_row (GdaDataModel *model, GdaDataModelIter *iter, gi
 		return gda_data_model_move_iter_at_row_default (model, iter, row);
 }
 
+static void set_param_attributes (GdaParameter *param, guint flags);
+
 gboolean
 gda_data_model_move_iter_at_row_default (GdaDataModel *model, GdaDataModelIter *iter, gint row)
 {
@@ -625,7 +631,6 @@ gda_data_model_move_iter_at_row_default (GdaDataModel *model, GdaDataModelIter *
 	gint col;
 	GdaDataModel *test;
 	gboolean update_model;
-	guint flags;
 	
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
 
@@ -643,6 +648,7 @@ gda_data_model_move_iter_at_row_default (GdaDataModel *model, GdaDataModelIter *
 	
 	g_object_get (G_OBJECT (iter), "data_model", &test, NULL);
 	g_return_val_if_fail (test == model, FALSE);
+	g_object_unref (test);
 	
 	/* actual sync. */
 	g_object_get (G_OBJECT (iter), "update_model", &update_model, NULL);
@@ -653,11 +659,8 @@ gda_data_model_move_iter_at_row_default (GdaDataModel *model, GdaDataModelIter *
 		gda_parameter_set_value (GDA_PARAMETER (list->data), 
 					 gda_data_model_get_value_at (model, col, row));
 
-		flags = gda_data_model_get_attributes_at (model, col, row);
-		if (flags & GDA_VALUE_ATTR_IS_DEFAULT)
-			g_object_set (G_OBJECT (list->data), "use-default-value", TRUE, NULL);
-		gda_parameter_set_exists_default_value (GDA_PARAMETER (list->data), 
-							flags & GDA_VALUE_ATTR_CAN_BE_DEFAULT);
+		set_param_attributes (GDA_PARAMETER (list->data), 
+				      gda_data_model_get_attributes_at (model, col, row));
 		list = g_slist_next (list);
 		col ++;
 	}
@@ -665,6 +668,21 @@ gda_data_model_move_iter_at_row_default (GdaDataModel *model, GdaDataModelIter *
 	return TRUE;
 }
 
+static void
+set_param_attributes (GdaParameter *param, guint flags)
+{
+	if (!gda_parameter_get_default_value (param))
+		gda_parameter_set_exists_default_value (param, 
+							flags & GDA_VALUE_ATTR_CAN_BE_DEFAULT);
+	if (flags & GDA_VALUE_ATTR_IS_DEFAULT)
+		g_object_set (G_OBJECT (param), "use-default-value", TRUE, NULL);
+	gda_parameter_set_not_null (param, ! (flags & GDA_VALUE_ATTR_CAN_BE_NULL));
+	if (flags & GDA_VALUE_ATTR_IS_NULL)
+		gda_parameter_set_value (param, NULL);
+	if (flags & GDA_VALUE_ATTR_DATA_NON_VALID)
+		gda_parameter_declare_invalid (param);
+}
+
 /**
  * gda_data_model_move_iter_next
  * @model: a #GdaDataModel object.
@@ -679,6 +697,10 @@ gboolean
 gda_data_model_move_iter_next (GdaDataModel *model, GdaDataModelIter *iter)
 {
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
+
+	if (! gda_data_model_iter_can_be_moved (iter))
+		return FALSE;
+
 	if (GDA_DATA_MODEL_GET_CLASS (model)->i_iter_next)
 		return (GDA_DATA_MODEL_GET_CLASS (model)->i_iter_next) (model, iter);
 	else 
@@ -694,7 +716,6 @@ gda_data_model_move_iter_next_default (GdaDataModel *model, GdaDataModelIter *it
 	gint row;
 	GdaDataModel *test;
 	gboolean update_model;
-	guint flags;
 	
 	/* validity tests */
 	if (! (gda_data_model_get_access_flags (model) & GDA_DATA_MODEL_ACCESS_RANDOM))
@@ -704,7 +725,8 @@ gda_data_model_move_iter_next_default (GdaDataModel *model, GdaDataModelIter *it
 	
 	g_object_get (G_OBJECT (iter), "data_model", &test, NULL);
 	g_return_val_if_fail (test == model, FALSE);
-	
+	g_object_unref (test);
+
 	g_object_get (G_OBJECT (iter), "current_row", &row, NULL);
 	row++;
 	if (row >= gda_data_model_get_n_rows (model)) {
@@ -720,11 +742,8 @@ gda_data_model_move_iter_next_default (GdaDataModel *model, GdaDataModelIter *it
 	while (list) {
 		gda_parameter_set_value (GDA_PARAMETER (list->data), 
 					 gda_data_model_get_value_at (model, col, row));
-		flags = gda_data_model_get_attributes_at (model, col, row);
-		if (flags & GDA_VALUE_ATTR_IS_DEFAULT)
-			g_object_set (G_OBJECT (list->data), "use-default-value", TRUE, NULL);
-		gda_parameter_set_exists_default_value (GDA_PARAMETER (list->data), 
-							flags & GDA_VALUE_ATTR_CAN_BE_DEFAULT);
+		set_param_attributes (GDA_PARAMETER (list->data), 
+				      gda_data_model_get_attributes_at (model, col, row));
 		list = g_slist_next (list);
 		col ++;
 	}
@@ -747,6 +766,10 @@ gboolean
 gda_data_model_move_iter_prev (GdaDataModel *model, GdaDataModelIter *iter)
 {
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
+
+	if (! gda_data_model_iter_can_be_moved (iter))
+		return FALSE;
+
 	if (GDA_DATA_MODEL_GET_CLASS (model)->i_iter_prev)
 		return (GDA_DATA_MODEL_GET_CLASS (model)->i_iter_prev) (model, iter);
 	else 
@@ -762,7 +785,6 @@ gda_data_model_move_iter_prev_default (GdaDataModel *model, GdaDataModelIter *it
 	gint row;
 	GdaDataModel *test;
 	gboolean update_model;
-	guint flags;
 	
 	/* validity tests */
 	if (! (gda_data_model_get_access_flags (model) & GDA_DATA_MODEL_ACCESS_RANDOM))
@@ -772,7 +794,8 @@ gda_data_model_move_iter_prev_default (GdaDataModel *model, GdaDataModelIter *it
 	
 	g_object_get (G_OBJECT (iter), "data_model", &test, NULL);
 	g_return_val_if_fail (test == model, FALSE);
-	
+	g_object_unref (test);
+
 	g_object_get (G_OBJECT (iter), "current_row", &row, NULL);
 	row--;
 	if (row < 0)
@@ -786,11 +809,8 @@ gda_data_model_move_iter_prev_default (GdaDataModel *model, GdaDataModelIter *it
 	while (list) {
 		gda_parameter_set_value (GDA_PARAMETER (list->data), 
 					 gda_data_model_get_value_at (model, col, row));
-		flags = gda_data_model_get_attributes_at (model, col, row);
-		if (flags & GDA_VALUE_ATTR_IS_DEFAULT)
-			g_object_set (G_OBJECT (list->data), "use-default-value", TRUE, NULL);
-		gda_parameter_set_exists_default_value (GDA_PARAMETER (list->data), 
-							flags & GDA_VALUE_ATTR_CAN_BE_DEFAULT);
+		set_param_attributes (GDA_PARAMETER (list->data), 
+				      gda_data_model_get_attributes_at (model, col, row));
 		list = g_slist_next (list);
 		col ++;
 	}
@@ -1288,7 +1308,7 @@ gda_data_model_to_xml_node (GdaDataModel *model, const gint *cols, gint nb_cols,
 		cstr = gda_column_get_dbms_type (column);
 		if (cstr && *cstr)
 			xmlSetProp (field, "dbms_type", cstr);
-		xmlSetProp (field, "gdatype", g_type_to_string (gda_column_get_g_type (column)));
+		xmlSetProp (field, "gdatype", gda_g_type_to_string (gda_column_get_g_type (column)));
 		if (gda_column_get_defined_size (column) != 0)
 			xml_set_int (field, "size", gda_column_get_defined_size (column));
 		if (gda_column_get_scale (column) != 0)
@@ -1310,7 +1330,7 @@ gda_data_model_to_xml_node (GdaDataModel *model, const gint *cols, gint nb_cols,
 	}
 	
 	/* add the model data to the XML output */
-	utility_data_model_dump_data_to_xml (model, node, cols, nb_cols, rows, nb_rows, FALSE);
+	gda_utility_data_model_dump_data_to_xml (model, node, cols, nb_cols, rows, nb_rows, FALSE);
 
 	if (!cols)
 		g_free (rcols);
@@ -1427,7 +1447,7 @@ add_xml_row (GdaDataModel *model, xmlNodePtr xml_row, GError **error)
 			if (!gda_value_set_from_string (value, xmlNodeGetContent (xml_field), gdatype)) {
 				g_free (value);
 				g_set_error (error, 0, 0, _("Cannot interpret string as a valid %s value"), 
-					     g_type_to_string (gdatype));
+					     gda_g_type_to_string (gdatype));
 				retval = FALSE;
 				break;
 			}
@@ -1508,6 +1528,7 @@ gda_data_model_add_data_from_xml_node (GdaDataModel *model, xmlNodePtr node, GEr
  * gda_data_model_import_from_model
  * @to: the destination #GdaDataModel
  * @from: the source #GdaDataModel
+ * @overwrite: TRUE if @to is completely overwritten by @from's data, and FALSE if @from's data is appended to @to
  * @cols_trans: a #GHashTable for columns translating, or %NULL
  * @error: a place to store errors, or %NULL
  *
@@ -1521,11 +1542,11 @@ gda_data_model_add_data_from_xml_node (GdaDataModel *model, xmlNodePtr node, GEr
  * Returns: TRUE if no error occurred.
  */
 gboolean
-gda_data_model_import_from_model (GdaDataModel *to, GdaDataModel *from, GHashTable *cols_trans, GError **error)
+gda_data_model_import_from_model (GdaDataModel *to, GdaDataModel *from, gboolean overwrite, GHashTable *cols_trans, GError **error)
 {
 	GdaDataModelIter *from_iter;
 	gboolean retval = TRUE;
-	gint to_nb_cols;
+	gint to_nb_cols, to_nb_rows, to_row = -1;;
 	gint from_nb_cols;
 	GSList *copy_params = NULL;
 	gint i;
@@ -1588,9 +1609,9 @@ gda_data_model_import_from_model (GdaDataModel *to, GdaDataModel *from, GHashTab
 				g_set_error (error, 0, 0,
 					     _("Destination column %d has a gda type (%s) incompatible with "
 					       "source column %d type (%s)"), i,
-					     g_type_to_string (gda_column_get_g_type (column)),
+					     gda_g_type_to_string (gda_column_get_g_type (column)),
 					     col,
-					     g_type_to_string (gda_parameter_get_g_type (param)));
+					     gda_g_type_to_string (gda_parameter_get_g_type (param)));
 				return FALSE;
 			}
 		}
@@ -1633,7 +1654,12 @@ gda_data_model_import_from_model (GdaDataModel *to, GdaDataModel *from, GHashTab
 	/* actual data copy (no memory allocation is needed here) */
 	gda_data_model_send_hint (to, GDA_DATA_MODEL_HINT_START_BATCH_UPDATE, NULL);
 	
-	gda_data_model_move_iter_next (from, from_iter); /* move to first row */
+	if (overwrite) {
+		to_row = 0;
+		to_nb_rows = gda_data_model_get_n_rows (to);
+	}
+
+	gda_data_model_iter_move_next (from_iter); /* move to first row */
 	while (retval && gda_data_model_iter_is_valid (from_iter)) {
 		GList *values = NULL;
 		GList *avlist = append_values;
@@ -1656,8 +1682,8 @@ gda_data_model_import_from_model (GdaDataModel *to, GdaDataModel *from, GHashTab
 						g_set_error (error, 0, 0,
 							     _("Can't transform '%s' from GDA type %s to GDA type %s"),
 							     str, 
-							     g_type_to_string (G_VALUE_TYPE (value)),
-							     g_type_to_string (G_VALUE_TYPE ((GValue *) (avlist->data))));
+							     gda_g_type_to_string (G_VALUE_TYPE (value)),
+							     gda_g_type_to_string (G_VALUE_TYPE ((GValue *) (avlist->data))));
 						g_free (str);
 						retval = FALSE;
 					}
@@ -1674,8 +1700,20 @@ gda_data_model_import_from_model (GdaDataModel *to, GdaDataModel *from, GHashTab
 		}
 
 		if (retval) {
-			if (gda_data_model_append_values (to, values, error) < 0) 
-				retval = FALSE;
+			if (to_row >= 0) {
+				if (!gda_data_model_set_values (to, to_row, values, error))
+					retval = FALSE;
+				else {
+					to_row ++;
+					if (to_row >= to_nb_rows)
+						/* we have finished modifying the existng rows */
+						to_row = -1;
+				}
+			}
+			else {
+				if (gda_data_model_append_values (to, values, error) < 0) 
+					retval = FALSE;
+			}
 		}
 		
 		g_list_free (values);
@@ -1695,6 +1733,14 @@ gda_data_model_import_from_model (GdaDataModel *to, GdaDataModel *from, GHashTab
 		g_free (append_types);
 	}
 
+	if (retval && (to_row >= 0)) {
+		/* remove extra rows */
+		for (; retval && (to_row < to_nb_rows); to_row++) {
+			if (!gda_data_model_remove_row (to, to_row, error))
+				retval = FALSE;
+		}
+	}
+
 	gda_data_model_send_hint (to, GDA_DATA_MODEL_HINT_END_BATCH_UPDATE, NULL);
 	return retval;
 }
@@ -1725,7 +1771,7 @@ gda_data_model_import_from_string (GdaDataModel *model,
 		return TRUE;
 
 	import = gda_data_model_import_new_mem (string, FALSE, options);
-	retval = gda_data_model_import_from_model (model, import, cols_trans, error);
+	retval = gda_data_model_import_from_model (model, import, FALSE, cols_trans, error);
 	g_object_unref (import);
 
 	return retval;
@@ -1758,12 +1804,14 @@ gda_data_model_import_from_file (GdaDataModel *model,
 		return TRUE;
 
 	import = gda_data_model_import_new_file (file, FALSE, options);
-	retval = gda_data_model_import_from_model (model, import, cols_trans, error);
+	retval = gda_data_model_import_from_model (model, import, FALSE, cols_trans, error);
 	g_object_unref (import);
 
 	return retval;
 }
 
+static gchar *real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attributes);
+
 /**
  * gda_data_model_dump
  * @model: a #GdaDataModel.
@@ -1782,6 +1830,10 @@ gda_data_model_dump (GdaDataModel *model, FILE *to_stream)
 	str = gda_data_model_dump_as_string (model);
 	g_fprintf (to_stream, "%s", str);
 	g_free (str);
+
+	str= real_gda_data_model_dump_as_string (model, TRUE);
+	g_fprintf (to_stream, "%s", str);
+	g_free (str);
 }
 
 /**
@@ -1794,6 +1846,14 @@ gda_data_model_dump (GdaDataModel *model, FILE *to_stream)
  */
 gchar *
 gda_data_model_dump_as_string (GdaDataModel *model)
+{
+	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), NULL);
+
+	return real_gda_data_model_dump_as_string (model, FALSE);
+}
+
+static gchar *
+real_gda_data_model_dump_as_string (GdaDataModel *model, gboolean dump_attributes)
 {
 	GString *string;
 	gchar *offstr, *str;
@@ -1808,8 +1868,6 @@ gda_data_model_dump_as_string (GdaDataModel *model)
 
 	gint offset = 0;
 
-	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), NULL);
-
 	string = g_string_new ("");
 
         /* string for the offset */
@@ -1829,7 +1887,7 @@ gda_data_model_dump_as_string (GdaDataModel *model)
 #ifdef GDA_DEBUG_NO
 		{
 			GdaColumn *col = gda_data_model_describe_column (model, i);
-			g_print ("Model col %d has type %s\n", i, g_type_to_string (gda_column_get_g_type (col)));
+			g_print ("Model col %d has type %s\n", i, gda_g_type_to_string (gda_column_get_g_type (col)));
 		}
 #endif
 
@@ -1839,30 +1897,43 @@ gda_data_model_dump_as_string (GdaDataModel *model)
 		else
 			cols_size [i] = 6; /* for "<none>" */
 
-		gdacol = gda_data_model_describe_column (model, i);
-		coltype = gda_column_get_g_type (gdacol);
-		/*g_string_append_printf (string, "COL %d is a %s\n", i+1, g_type_to_string (coltype));*/
-		if ((coltype == G_TYPE_INT64) ||
-		    (coltype == G_TYPE_UINT64) ||
-		    (coltype == G_TYPE_INT) ||
-		    (coltype == GDA_TYPE_NUMERIC) ||
-		    (coltype == G_TYPE_FLOAT) ||
-		    (coltype == GDA_TYPE_SHORT) ||
-		    (coltype == GDA_TYPE_USHORT) ||
-		    (coltype == G_TYPE_CHAR) ||
-		    (coltype == G_TYPE_UCHAR) ||
-		    (coltype == G_TYPE_UINT))
-			cols_is_num [i] = TRUE;
+		if (! dump_attributes) {
+			gdacol = gda_data_model_describe_column (model, i);
+			coltype = gda_column_get_g_type (gdacol);
+			/*g_string_append_printf (string, "COL %d is a %s\n", i+1, gda_g_type_to_string (coltype));*/
+			if ((coltype == G_TYPE_INT64) ||
+			    (coltype == G_TYPE_UINT64) ||
+			    (coltype == G_TYPE_INT) ||
+			    (coltype == GDA_TYPE_NUMERIC) ||
+			    (coltype == G_TYPE_FLOAT) ||
+			    (coltype == GDA_TYPE_SHORT) ||
+			    (coltype == GDA_TYPE_USHORT) ||
+			    (coltype == G_TYPE_CHAR) ||
+			    (coltype == G_TYPE_UCHAR) ||
+			    (coltype == G_TYPE_UINT))
+				cols_is_num [i] = TRUE;
+			else
+				cols_is_num [i] = FALSE;
+		}
 		else
-			cols_is_num [i] = FALSE;
+			cols_is_num [i] = TRUE;
 	}
 
 	/* ... and using column data */
 	for (j = 0; j < n_rows; j++) {
 		for (i = 0; i < n_cols; i++) {
-			value = gda_data_model_get_value_at (model, i, j);
-			str = value ? gda_value_stringify ((GValue*)value) : g_strdup ("_null_");
-			if (str) {
+			if (! dump_attributes) {
+				value = gda_data_model_get_value_at (model, i, j);
+				str = value ? gda_value_stringify ((GValue*)value) : g_strdup ("_null_");
+				if (str) {
+					cols_size [i] = MAX (cols_size [i], g_utf8_strlen (str, -1));
+					g_free (str);
+				}
+			}
+			else {
+				guint attrs;
+				attrs = gda_data_model_get_attributes_at (model, i, j);
+				str = g_strdup_printf ("%u", attrs);
 				cols_size [i] = MAX (cols_size [i], g_utf8_strlen (str, -1));
 				g_free (str);
 			}
@@ -1902,8 +1973,15 @@ gda_data_model_dump_as_string (GdaDataModel *model)
 	if (gda_data_model_get_access_flags (model) & GDA_DATA_MODEL_ACCESS_RANDOM) {
 		for (j = 0; j < n_rows; j++) {
 			for (i = 0; i < n_cols; i++) {
-				value = gda_data_model_get_value_at (model, i, j);
-				str = value ? gda_value_stringify ((GValue *)value) : g_strdup ("_null_");
+				if (!dump_attributes) {
+					value = gda_data_model_get_value_at (model, i, j);
+					str = value ? gda_value_stringify ((GValue *)value) : g_strdup ("_null_");
+				}
+				else {
+					guint attrs;
+					attrs = gda_data_model_get_attributes_at (model, i, j);
+					str = g_strdup_printf ("%u", attrs);
+				}
 				if (i != 0)
 					g_string_append_printf (string, "%s", sep_col);
 				if (cols_is_num [i])
