@@ -188,7 +188,8 @@ gda_dict_field_class_init (GdaDictFieldClass * class)
 	object_class->set_property = gda_dict_field_set_property;
 	object_class->get_property = gda_dict_field_get_property;
 	g_object_class_install_property (object_class, PROP_DB_TABLE,
-					 g_param_spec_pointer ("db_table", NULL, NULL, 
+					 g_param_spec_object ("db_table", NULL, NULL, 
+                                                               GDA_TYPE_DICT_TABLE,
 							       (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 	g_object_class_install_property (object_class, PROP_PLUGIN,
                                          g_param_spec_string ("entry_plugin", NULL, NULL, NULL,
@@ -303,26 +304,26 @@ gda_dict_field_set_property (GObject *object,
 			     const GValue *value,
 			     GParamSpec *pspec)
 {
-	gpointer ptr;
 	GdaDictField *gda_dict_field;
 
 	gda_dict_field = GDA_DICT_FIELD (object);
 	if (gda_dict_field->priv) {
 		switch (param_id) {
-		case PROP_DB_TABLE:
+		case PROP_DB_TABLE: {
 			if (gda_dict_field->priv->table) {
 				g_signal_handlers_disconnect_by_func (G_OBJECT (gda_dict_field->priv->table),
 								      G_CALLBACK (destroyed_object_cb), gda_dict_field);
-				gda_dict_field->priv->table = NULL;
+				gda_dict_field->priv->table = NULL; /* TODO: unref it? */
 			}
 
-			ptr = g_value_get_pointer (value);
+			GdaDictTable* ptr = GDA_DICT_TABLE (g_value_get_object (value));
 			if (ptr && GDA_IS_DICT_TABLE (ptr)) {
-				gda_dict_field->priv->table = GDA_DICT_TABLE (ptr);
+				gda_dict_field->priv->table = GDA_DICT_TABLE (ptr); /* TODO: ref it? */
 				gda_object_connect_destroy (ptr,
 							 G_CALLBACK (destroyed_object_cb), gda_dict_field);
 			}
 			break;
+                }
 		case PROP_PLUGIN:
 			g_free (gda_dict_field->priv->plugin);
 			if (g_value_get_string (value))
@@ -346,7 +347,7 @@ gda_dict_field_get_property (GObject *object,
 	if (gda_dict_field->priv) {
 		switch (param_id) {
 		case PROP_DB_TABLE:
-			g_value_set_pointer (value, gda_dict_field->priv->table);
+			g_value_set_object (value, G_OBJECT (gda_dict_field->priv->table));
 			break;
 		case PROP_PLUGIN:
 			g_value_set_string (value, gda_dict_field->priv->plugin);
@@ -814,7 +815,7 @@ gda_dict_field_get_xml_id (GdaXmlStorage *iface)
 	g_return_val_if_fail (GDA_DICT_FIELD (iface)->priv, NULL);
 
 	t_xml_id = gda_xml_storage_get_xml_id (GDA_XML_STORAGE (GDA_DICT_FIELD (iface)->priv->table));
-	tmp = utility_build_encoded_id ("FI", gda_object_get_name (GDA_OBJECT (iface)));
+	tmp = gda_utility_build_encoded_id ("FI", gda_object_get_name (GDA_OBJECT (iface)));
 	xml_id = g_strconcat (t_xml_id, ":", tmp, NULL);
 	g_free (t_xml_id);
 	g_free (tmp);
@@ -858,7 +859,7 @@ gda_dict_field_save_to_xml (GdaXmlStorage *iface, GError **error)
 		GType vtype;
 		
 		vtype = G_VALUE_TYPE (field->priv->default_val);
-		xmlSetProp (node, "default_g_type", g_type_to_string (vtype));
+		xmlSetProp (node, "default_g_type", gda_g_type_to_string (vtype));
 
 		dh = gda_dict_get_default_handler (gda_object_get_dict (GDA_OBJECT (field)), vtype);
 		str = gda_data_handler_get_str_from_value (dh, field->priv->default_val);
@@ -866,7 +867,7 @@ gda_dict_field_save_to_xml (GdaXmlStorage *iface, GError **error)
 		g_free (str);
 	}
 
-	str = utility_table_field_attrs_stringify (field->priv->extra_attrs);
+	str = gda_utility_table_field_attrs_stringify (field->priv->extra_attrs);
 	if (str) {
 		xmlSetProp (node, "extra_attr", str);
 		g_free (str);
@@ -931,7 +932,7 @@ gda_dict_field_load_from_xml (GdaXmlStorage *iface, xmlNodePtr node, GError **er
 				xmlNodePtr tnode;
 				
 				dt = GDA_DICT_TYPE (gda_dict_type_new (dict));
-				tmp = utility_build_decoded_id (NULL, prop + 2);
+				tmp = gda_utility_build_decoded_id (NULL, prop + 2);
 				gda_dict_type_set_sqlname (dt, tmp);
 				g_free (tmp);
 				gda_dict_type_set_g_type (dt, GDA_TYPE_BLOB);
@@ -976,7 +977,7 @@ gda_dict_field_load_from_xml (GdaXmlStorage *iface, xmlNodePtr node, GError **er
 			GdaDataHandler *dh;
 			GValue *value;
 			
-			vtype = g_type_from_string (str2);
+			vtype = gda_g_type_from_string (str2);
 			if (vtype == G_TYPE_INVALID) {
 				g_set_error (error,
 					     GDA_DICT_FIELD_ERROR,
@@ -1006,7 +1007,7 @@ gda_dict_field_load_from_xml (GdaXmlStorage *iface, xmlNodePtr node, GError **er
 
 	prop = xmlGetProp (node, "extra_attr");
 	if (prop) {
-		gda_dict_field_set_attributes (field, utility_table_field_attrs_parse (prop));
+		gda_dict_field_set_attributes (field, gda_utility_table_field_attrs_parse (prop));
 		xmlFree (prop);
 	}
 
