@@ -1,6 +1,6 @@
 /* gda-dict-function.c
  *
- * Copyright (C) 2003 - 2005 Vivien Malerba
+ * Copyright (C) 2003 - 2006 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -136,6 +136,11 @@ gda_dict_function_class_init (GdaDictFunctionClass * class)
 	/* Properties */
 	object_class->set_property = gda_dict_function_set_property;
 	object_class->get_property = gda_dict_function_get_property;
+
+        /* TODO: What kind of object is this meant to be?
+           When we know, we should use g_param_spec_object() instead of g_param_spec_pointer().
+           murrayc.
+         */
 	g_object_class_install_property (object_class, PROP,
 					 g_param_spec_pointer ("prop", NULL, NULL, (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 
@@ -193,8 +198,8 @@ gda_dict_function_dispose (GObject *object)
 	if (gda_dict_function->priv) {
 		gda_object_destroy_check (GDA_OBJECT (object));
 
-		gda_dict_function_set_ret_type (gda_dict_function, NULL);
-		gda_dict_function_set_arg_types (gda_dict_function, NULL);
+		gda_dict_function_set_ret_dict_type (gda_dict_function, NULL);
+		gda_dict_function_set_arg_dict_types (gda_dict_function, NULL);
 	}
 
 	/* parent class */
@@ -425,7 +430,7 @@ gnome_db_function_load_from_xml (GdaXmlStorage *iface, xmlNodePtr node, GError *
 			GdaDictType *dt = NULL;
 			prop = xmlGetProp (subnode, "type");
 			if (prop) {
-				dt = gda_dict_get_data_type_by_xml_id (dict, prop);
+				dt = gda_dict_get_dict_type_by_xml_id (dict, prop);
 				if (!dt) {
 					/* Add a new custom data type (this can't happen at the moment because of the DTD) */
 					TO_IMPLEMENT;
@@ -444,7 +449,7 @@ gnome_db_function_load_from_xml (GdaXmlStorage *iface, xmlNodePtr node, GError *
 							     gda_object_get_name (GDA_OBJECT (func)));
 						return FALSE;
 					}
-					gda_dict_function_set_ret_type (func, dt);
+					gda_dict_function_set_ret_dict_type (func, dt);
 				}
 				else 
 					argtypes = g_slist_prepend (argtypes, dt);
@@ -455,7 +460,7 @@ gnome_db_function_load_from_xml (GdaXmlStorage *iface, xmlNodePtr node, GError *
 	}
 
 	argtypes = g_slist_reverse (argtypes);
-	gda_dict_function_set_arg_types (func, argtypes);
+	gda_dict_function_set_arg_dict_types (func, argtypes);
 	g_slist_free (argtypes);
 
 	if (pname && pid)
@@ -489,7 +494,7 @@ gda_dict_function_set_dbms_id (GdaDictFunction *func, const gchar *id)
 	if (func->priv->objectid)
 		g_free (func->priv->objectid);
 
-	func->priv->objectid = utility_build_encoded_id (NULL, id);
+	func->priv->objectid = gda_utility_build_encoded_id (NULL, id);
 }
 
 
@@ -507,7 +512,7 @@ gda_dict_function_get_dbms_id (GdaDictFunction *func)
 	g_return_val_if_fail (func && GDA_IS_DICT_FUNCTION (func), NULL);
 	g_return_val_if_fail (func->priv, NULL);
 
-	return utility_build_decoded_id (NULL, func->priv->objectid);
+	return gda_utility_build_decoded_id (NULL, func->priv->objectid);
 }
 
 
@@ -548,7 +553,7 @@ gda_dict_function_get_sqlname (GdaDictFunction *func)
 static void destroyed_data_type_cb (GdaDictType *dt, GdaDictFunction *func);
 
 /**
- * gda_dict_function_set_arg_types
+ * gda_dict_function_set_arg_dict_types
  * @func: a #GdaDictFunction object
  * @arg_types: a list of #GdaDictType objects or #NULL values ordered to represent the data types
  * of the function's arguments .
@@ -556,7 +561,7 @@ static void destroyed_data_type_cb (GdaDictType *dt, GdaDictFunction *func);
  * Set the arguments types of a function
  */
 void 
-gda_dict_function_set_arg_types (GdaDictFunction *func, const GSList *arg_types)
+gda_dict_function_set_arg_dict_types (GdaDictFunction *func, const GSList *arg_types)
 {
 	GSList *list;
 
@@ -577,7 +582,7 @@ gda_dict_function_set_arg_types (GdaDictFunction *func, const GSList *arg_types)
 		g_slist_free (func->priv->arg_types);
 	}
 
-	func->priv->arg_types = g_slist_copy (arg_types);
+	func->priv->arg_types = g_slist_copy ((GSList *) arg_types);
 	list = func->priv->arg_types;
 	while (list) {
 		if (list->data) {
@@ -590,7 +595,7 @@ gda_dict_function_set_arg_types (GdaDictFunction *func, const GSList *arg_types)
 }
 
 /**
- * gda_dict_function_get_arg_types
+ * gda_dict_function_get_arg_dict_types
  * @func: a #GdaDictFunction object
  * 
  * To consult the list of arguments types (and number) of a function.
@@ -598,7 +603,7 @@ gda_dict_function_set_arg_types (GdaDictFunction *func, const GSList *arg_types)
  * Returns: a list of #GdaDictType objects, the list MUST NOT be modified.
  */
 const GSList *
-gda_dict_function_get_arg_types (GdaDictFunction *func)
+gda_dict_function_get_arg_dict_types (GdaDictFunction *func)
 {
 	g_return_val_if_fail (func && GDA_IS_DICT_FUNCTION (func), NULL);
 	g_return_val_if_fail (func->priv, NULL);
@@ -607,14 +612,14 @@ gda_dict_function_get_arg_types (GdaDictFunction *func)
 }
 
 /**
- * gda_dict_function_set_ret_type
+ * gda_dict_function_set_ret_dict_type
  * @func: a #GdaDictFunction object
  * @dt: a #GdaDictType object or #NULL
  *
  * Set the return type of a function
  */
 void 
-gda_dict_function_set_ret_type  (GdaDictFunction *func, GdaDictType *dt)
+gda_dict_function_set_ret_dict_type  (GdaDictFunction *func, GdaDictType *dt)
 {
 	g_return_if_fail (func && GDA_IS_DICT_FUNCTION (func));
 	g_return_if_fail (func->priv);
@@ -642,7 +647,7 @@ destroyed_data_type_cb (GdaDictType *dt, GdaDictFunction *func)
 }
 
 /**
- * gda_dict_function_get_ret_type
+ * gda_dict_function_get_ret_dict_type
  * @func: a #GdaDictFunction object
  * 
  * To consult the return type of a function.
@@ -650,7 +655,7 @@ destroyed_data_type_cb (GdaDictType *dt, GdaDictFunction *func)
  * Returns: a #GdaDictType object.
  */
 GdaDictType *
-gda_dict_function_get_ret_type  (GdaDictFunction *func)
+gda_dict_function_get_ret_dict_type  (GdaDictFunction *func)
 {
 	g_return_val_if_fail (func && GDA_IS_DICT_FUNCTION (func), NULL);
 	g_return_val_if_fail (func->priv, NULL);
@@ -660,7 +665,7 @@ gda_dict_function_get_ret_type  (GdaDictFunction *func)
 
 
 /**
- * gda_dict_function_accepts_args
+ * gda_dict_function_accepts_arg_dict_types
  * @func: a #GdaDictFunction object
  * @arg_types: a list of #GdaDictType objects or #NULL values, ordered
  *
@@ -673,9 +678,9 @@ gda_dict_function_get_ret_type  (GdaDictFunction *func)
  * Returns: TRUE if accepted
  */
 gboolean
-gda_dict_function_accepts_args (GdaDictFunction *func, const GSList *arg_types)
+gda_dict_function_accepts_arg_dict_types (GdaDictFunction *func, const GSList *arg_types)
 {
-	GSList *arg = arg_types, *list;
+	GSList *arg = (GSList *) arg_types, *list;
 	gboolean args_ok = TRUE;
 	GdaServerProviderInfo *sinfo = NULL;
 	GdaDict *dict;
@@ -688,7 +693,7 @@ gda_dict_function_accepts_args (GdaDictFunction *func, const GSList *arg_types)
 	cnc = gda_dict_get_connection (dict);
 	if (cnc) 
 		sinfo = gda_connection_get_infos (cnc);
-	list = gda_dict_function_get_arg_types (func);
+	list = (GSList *) gda_dict_function_get_arg_dict_types (func);
 	
 	if (g_slist_length (arg) != g_slist_length (list))
 		return FALSE;
@@ -701,10 +706,10 @@ gda_dict_function_accepts_args (GdaDictFunction *func, const GSList *arg_types)
 				args_ok = FALSE;
 		}
 		else {
-			/* GdaValueType compatibility test */
+			/* GType compatibility test */
 			if (arg->data && list->data &&
-			    (gda_dict_type_get_gda_type (GDA_DICT_TYPE (arg->data)) !=
-			     gda_dict_type_get_gda_type (GDA_DICT_TYPE (list->data))))
+			    (gda_dict_type_get_g_type (GDA_DICT_TYPE (arg->data)) !=
+			     gda_dict_type_get_g_type (GDA_DICT_TYPE (list->data))))
 				args_ok = FALSE;
 		}
 		
