@@ -122,6 +122,11 @@ gda_graphviz_class_init (GdaGraphvizClass * class)
 	/* Properties */
 	object_class->set_property = gda_graphviz_set_property;
 	object_class->get_property = gda_graphviz_get_property;
+
+        /* TODO: What kind of object is this meant to be?
+           When we know, we should use g_param_spec_object() instead of g_param_spec_pointer().
+           murrayc.
+         */
 	g_object_class_install_property (object_class, PROP,
 					 g_param_spec_pointer ("prop", NULL, NULL, 
 							       (G_PARAM_READABLE | G_PARAM_WRITABLE)));
@@ -385,6 +390,7 @@ get_depend_query (GdaQuery *query, GSList **deplist, GSList **alldisplayed)
 				GdaQuery *dquery = GDA_QUERY (gda_entity_field_get_entity (GDA_ENTITY_FIELD (base)));
 				if (!g_slist_find (*alldisplayed, dquery) && (dquery != query))
 					*deplist = g_slist_append (*deplist, dquery);
+				g_object_unref (base);
 			}
 		}
 		list = g_slist_next (list);
@@ -587,6 +593,7 @@ do_graph_query (GdaGraphviz *graph, GString *string, GdaQuery *query, gint tabof
 					g_string_append_printf (string, "%s\t\"%s\" -> \"%s\" [style=dotted];\n", str, fid, bid);
 					g_free (fid);
 					g_free (bid);
+					g_object_unref (base);
 				}
 			}
 			
