@@ -56,6 +56,7 @@ static void paramlist_remove_source (GdaParameterList *paramlist, GdaParameterLi
 static void destroyed_param_cb (GdaParameter *param, GdaParameterList *dataset);
 static void changed_param_cb (GdaParameter *param, GdaParameterList *dataset);
 static void restrict_changed_param_cb (GdaParameter *param, GdaParameterList *dataset);
+static void notify_param_cb (GdaParameter *param, GParamSpec *pspec, GdaParameterList *dataset);
 
 #ifdef GDA_DEBUG
 static void gda_parameter_list_dump                     (GdaParameterList *paramlist, guint offset);
@@ -69,10 +70,12 @@ enum
 {
 	PARAM_CHANGED,
 	PUBLIC_DATA_CHANGED,
+	PARAM_PLUGIN_CHANGED,
+	PARAM_ATTR_CHANGED,
 	LAST_SIGNAL
 };
 
-static gint gda_parameter_list_signals[LAST_SIGNAL] = { 0, 0 };
+static gint gda_parameter_list_signals[LAST_SIGNAL] = { 0, 0, 0, 0 };
 
 
 /* private structure */
@@ -136,6 +139,22 @@ gda_parameter_list_class_init (GdaParameterListClass *class)
 			      NULL, NULL,
 			      gda_marshal_VOID__OBJECT, G_TYPE_NONE, 1,
 			      G_TYPE_OBJECT);
+	gda_parameter_list_signals[PARAM_PLUGIN_CHANGED] =
+		g_signal_new ("param_plugin_changed",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_FIRST,
+			      G_STRUCT_OFFSET (GdaParameterListClass, param_plugin_changed),
+			      NULL, NULL,
+			      gda_marshal_VOID__OBJECT, G_TYPE_NONE, 1,
+			      G_TYPE_OBJECT);
+	gda_parameter_list_signals[PARAM_ATTR_CHANGED] =
+		g_signal_new ("param_attr_changed",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_FIRST,
+			      G_STRUCT_OFFSET (GdaParameterListClass, param_attr_changed),
+			      NULL, NULL,
+			      gda_marshal_VOID__OBJECT, G_TYPE_NONE, 1,
+			      G_TYPE_OBJECT);
 	gda_parameter_list_signals[PUBLIC_DATA_CHANGED] =
 		g_signal_new ("public_data_changed",
 			      G_TYPE_FROM_CLASS (object_class),
@@ -145,6 +164,8 @@ gda_parameter_list_class_init (GdaParameterListClass *class)
 			      gda_marshal_VOID__VOID, G_TYPE_NONE, 0);
 
 	class->param_changed = NULL;
+	class->param_plugin_changed = NULL;
+	class->param_attr_changed = NULL;
 	class->public_data_changed = NULL;
 
 	object_class->dispose = gda_parameter_list_dispose;
@@ -546,7 +567,7 @@ gda_parameter_list_new_from_spec_node (GdaDict *dict, xmlNodePtr xml_spec, GErro
 			/* find data type and create GdaParameter */
 			dbmstype = xmlGetProp (cur, BAD_CAST "dbmstype");
 			gdatype = xmlGetProp (cur, BAD_CAST "gdatype");
-			dtype = utility_find_or_create_data_type (dict, prov, cnc,
+			dtype = gda_utility_find_or_create_data_type (dict, prov, cnc,
 								  dbmstype, gdatype, &dtype_created);
 			if (dbmstype) xmlFree (dbmstype);
 			if (gdatype) xmlFree (gdatype);
@@ -573,7 +594,7 @@ gda_parameter_list_new_from_spec_node (GdaDict *dict, xmlNodePtr xml_spec, GErro
 				g_object_unref (dtype);
 			
 			/* set parameter's attributes */
-			utility_parameter_load_attributes (param, cur, sources);
+			gda_utility_parameter_load_attributes (param, cur, sources);
 		}
 	}
 
@@ -692,9 +713,9 @@ gda_parameter_list_get_spec (GdaParameterList *paramlist)
 		/* dtype = gda_parameter_get_g_type (param); */
 /* 		if (dtype) { */
 /* 			xmlSetProp (node, "dbmstype", gda_dict_type_get_sqlname (dtype)); */
-/* 			xmlSetProp (node, "gdatype", g_type_to_string (gda_dict_type_get_g_type (dtype))); */
+/* 			xmlSetProp (node, "gdatype", gda_g_type_to_string (gda_dict_type_get_g_type (dtype))); */
 /* 		} */
-		xmlSetProp (node, "gdatype", g_type_to_string (gda_parameter_get_g_type (param)));
+		xmlSetProp (node, "gdatype", gda_g_type_to_string (gda_parameter_get_g_type (param)));
 
 		xmlSetProp (node, "nullok", gda_parameter_get_not_null (param) ? "FALSE" : "TRUE");
 		g_object_get (G_OBJECT (param), "entry_plugin", &str, NULL);
@@ -728,6 +749,8 @@ destroyed_param_cb (GdaParameter *param, GdaParameterList *paramlist)
 					      G_CALLBACK (changed_param_cb), paramlist);
 	g_signal_handlers_disconnect_by_func (G_OBJECT (param),
 					      G_CALLBACK (restrict_changed_param_cb), paramlist);
+	g_signal_handlers_disconnect_by_func (G_OBJECT (param),
+					      G_CALLBACK (notify_param_cb), paramlist);
 
 	/* now destroy the GdaParameterListNode and the GdaParameterListSource if necessary */
 	node = gda_parameter_list_find_node_for_param (paramlist, param);
@@ -753,6 +776,29 @@ restrict_changed_param_cb (GdaParameter *param, GdaParameterList *paramlist)
 	compute_public_data (paramlist);
 }
 
+static void
+notify_param_cb (GdaParameter *param, GParamSpec *pspec, GdaParameterList *paramlist)
+{
+	if (!strcmp (pspec->name, "entry-plugin")) {
+#ifdef GDA_DEBUG_signal
+		g_print (">> 'PARAM_PLUGIN_CHANGED' from %s\n", __FUNCTION__);
+#endif
+		g_signal_emit (G_OBJECT (paramlist), gda_parameter_list_signals[PARAM_PLUGIN_CHANGED], 0, param);
+#ifdef GDA_DEBUG_signal
+		g_print ("<< 'PARAM_PLUGIN_CHANGED' from %s\n", __FUNCTION__);
+#endif
+	}
+	if (!strcmp (pspec->name, "use-default-value")) {
+#ifdef GDA_DEBUG_signal
+		g_print (">> 'PARAM_ATTR_CHANGED' from %s\n", __FUNCTION__);
+#endif
+		g_signal_emit (G_OBJECT (paramlist), gda_parameter_list_signals[PARAM_ATTR_CHANGED], 0, param);
+#ifdef GDA_DEBUG_signal
+		g_print ("<< 'PARAM_ATTR_CHANGED' from %s\n", __FUNCTION__);
+#endif
+	}
+}
+
 static void
 changed_param_cb (GdaParameter *param, GdaParameterList *paramlist)
 {
@@ -1103,6 +1149,8 @@ gda_parameter_list_real_add_param (GdaParameterList *paramlist, GdaParameter *pa
 				  G_CALLBACK (changed_param_cb), paramlist);
 		g_signal_connect (G_OBJECT (param), "restrict_changed",
 				  G_CALLBACK (restrict_changed_param_cb), paramlist);
+		g_signal_connect (G_OBJECT (param), "notify",
+				  G_CALLBACK (notify_param_cb), paramlist);
 
 		g_object_ref (G_OBJECT (param));
 		gda_parameter_replace_param_users (param, paramlist->priv->param_repl);
@@ -1138,7 +1186,7 @@ gda_parameter_list_add_param_from_string (GdaParameterList *paramlist, const gch
 	if (! gda_parameter_set_value_str (param, str)) {
 		g_object_unref (param);
 		g_warning (_("Could not add parameter of type %s with value '%s'"), 
-			   g_type_to_string (type), str);
+			   gda_g_type_to_string (type), str);
 		return NULL;
 	}
 
@@ -1269,8 +1317,8 @@ gda_parameter_list_is_coherent (GdaParameterList *paramlist, GError **error)
 				g_set_error (error, GDA_PARAMETER_LIST_ERROR, 0,
 					     _("GdaParameter is restricted by a column of the wrong type: "
 					       "%s (%s expected)"),
-					     g_type_to_string (gda_parameter_get_g_type (node->param)),
-					     g_type_to_string (gda_column_get_g_type (col)));
+					     gda_g_type_to_string (gda_parameter_get_g_type (node->param)),
+					     gda_g_type_to_string (gda_column_get_g_type (col)));
 				return FALSE;
 			}
 		}
