@@ -93,6 +93,7 @@ enum
 	PROP_RESTRICT_MODEL,
 	PROP_RESTRICT_COLUMN,
 	PROP_GDA_TYPE,
+	PROP_NOT_NULL,
 	PROP_DICT_TYPE /* TO ADD */
 };
 
@@ -100,15 +101,15 @@ enum
 struct _GdaParameterPrivate
 {
 	GSList                *param_users; /* list of #GdaObject using that parameter */
-	GdaValueType           gda_type;
+	GType                  g_type;
 	GdaParameter          *alias_of;     /* FULL bind to param */
 	GdaParameter          *change_with;  /* SIMPLE bind to param */
 	
 	gboolean               invalid_forced;
 	gboolean               valid;
 
-	GdaValue              *value;
-	GdaValue              *default_value; /* CAN be either NULL or of any type */
+	GValue                *value;
+	GValue                *default_value; /* CAN be either NULL or of any type */
 	gboolean               has_default_value; /* TRUE if param has a default value (even if unknown) */
 	gboolean               default_forced;
 	gboolean               not_null;      /* TRUE if 'value' must not be NULL when passed to destination fields */
@@ -198,24 +199,30 @@ gda_parameter_class_init (GdaParameterClass *class)
 	object_class->set_property = gda_parameter_set_property;
 	object_class->get_property = gda_parameter_get_property;
         g_object_class_install_property (object_class, PROP_GDA_TYPE,
-                                         g_param_spec_int ("gda_type", NULL, NULL,
-							   0, G_MAXINT, GDA_VALUE_TYPE_UNKNOWN,
+                                         g_param_spec_ulong ("g_type", NULL, NULL,
+							   0, G_MAXULONG, GDA_TYPE_NULL,
 							   (G_PARAM_READABLE | 
 							    G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY)));
 	g_object_class_install_property (object_class, PROP_PLUGIN,
-					 g_param_spec_string ("handler_plugin", NULL, NULL, NULL, 
+					 g_param_spec_string ("entry_plugin", NULL, NULL, NULL, 
 							      (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 	g_object_class_install_property (object_class, PROP_USE_DEFAULT_VALUE,
 					 g_param_spec_boolean ("use_default_value", NULL, NULL, FALSE,
 							       (G_PARAM_READABLE | G_PARAM_WRITABLE)));
+	g_object_class_install_property (object_class, PROP_NOT_NULL,
+					 g_param_spec_boolean ("not_null", NULL, NULL, FALSE,
+							       (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 	g_object_class_install_property (object_class, PROP_SIMPLE_BIND,
-					 g_param_spec_pointer ("simple_bind", NULL, NULL, 
+					 g_param_spec_object ("simple_bind", NULL, NULL, 
+                                                               GDA_TYPE_PARAMETER,
 							       (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 	g_object_class_install_property (object_class, PROP_FULL_BIND,
-					 g_param_spec_pointer ("full_bind", NULL, NULL, 
+					 g_param_spec_object ("full_bind", NULL, NULL, 
+                                                               GDA_TYPE_PARAMETER,
 							       (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 	g_object_class_install_property (object_class, PROP_RESTRICT_MODEL,
-                                         g_param_spec_pointer ("restrict_model", NULL, NULL,
+                                         g_param_spec_object ("restrict_model", NULL, NULL,
+                                                               GDA_TYPE_DATA_MODEL,
                                                                (G_PARAM_READABLE | G_PARAM_WRITABLE)));
         g_object_class_install_property (object_class, PROP_RESTRICT_COLUMN,
                                          g_param_spec_int ("restrict_column", NULL, NULL,
@@ -231,7 +238,7 @@ gda_parameter_init (GdaParameter *parameter)
 {
 	parameter->priv = g_new0 (GdaParameterPrivate, 1);
 	parameter->priv->param_users = NULL;
-	parameter->priv->gda_type = GDA_VALUE_TYPE_UNKNOWN;
+	parameter->priv->g_type = G_TYPE_INVALID;
 	parameter->priv->alias_of = NULL;
 	parameter->priv->change_with = NULL;
 
@@ -250,24 +257,22 @@ gda_parameter_init (GdaParameter *parameter)
 
 /**
  * gda_parameter_new
- * @type: the #GdaValueType requested
+ * @type: the #GType requested
  *
  * Creates a new parameter of type @type
  *
  * Returns: a new #GdaParameter object
  */
 GdaParameter *
-gda_parameter_new (GdaValueType type)
+gda_parameter_new (GType type)
 {
 	GObject   *obj;
-	GdaParameter *param;
 
-	g_return_val_if_fail (type != GDA_VALUE_TYPE_UNKNOWN, NULL);
+	g_return_val_if_fail (type != G_TYPE_INVALID, NULL);
 
-        obj = g_object_new (GDA_TYPE_PARAMETER, "gda_type", type, NULL);
-	param = GDA_PARAMETER (obj);
+        obj = g_object_new (GDA_TYPE_PARAMETER, "g_type", type, NULL);
 
-        return param;
+        return (GdaParameter *) obj;
 }
 
 /**
@@ -290,7 +295,7 @@ gda_parameter_new_copy (GdaParameter *orig)
 	g_return_val_if_fail (orig->priv, NULL);
 
 	dict = gda_object_get_dict (GDA_OBJECT (orig));
-	obj = g_object_new (GDA_TYPE_PARAMETER, "dict", dict,"gda_type", orig->priv->gda_type, NULL);
+	obj = g_object_new (GDA_TYPE_PARAMETER, "dict", dict,"g_type", orig->priv->g_type, NULL);
 	param = GDA_PARAMETER (obj);
 
 	gda_object_set_name (GDA_OBJECT (param), gda_object_get_name (GDA_OBJECT (orig)));
@@ -333,7 +338,7 @@ gda_parameter_new_copy (GdaParameter *orig)
  * @name: the name of the parameter to create
  * @str: the contents of the parameter to create
  *
- * Creates a new #GdaParameter object of type GDA_VALUE_TYPE_STRING
+ * Creates a new #GdaParameter object of type G_TYPE_STRING
  *
  * Returns: a new #GdaParameter object
  */
@@ -342,7 +347,7 @@ gda_parameter_new_string (const gchar *name, const gchar *str)
 {
 	GdaParameter *param;
 
-	param = gda_parameter_new (GDA_VALUE_TYPE_STRING);
+	param = gda_parameter_new (G_TYPE_STRING);
 	gda_object_set_name (GDA_OBJECT (param), name);
 	gda_parameter_set_value_str (param, str);
 
@@ -354,7 +359,7 @@ gda_parameter_new_string (const gchar *name, const gchar *str)
  * @name: the name of the parameter to create
  * @value: the value to give to the new parameter
  *
- * Creates a new #GdaParameter object of type GDA_VALUE_TYPE_BOOLEAN
+ * Creates a new #GdaParameter object of type G_TYPE_BOOLEAN
  *
  * Returns: a new #GdaParameter object
  */
@@ -363,7 +368,7 @@ gda_parameter_new_boolean (const gchar *name, gboolean value)
 {
 	GdaParameter *param;
 
-	param = gda_parameter_new (GDA_VALUE_TYPE_BOOLEAN);
+	param = gda_parameter_new (G_TYPE_BOOLEAN);
 	gda_object_set_name (GDA_OBJECT (param), name);
 	gda_parameter_set_value_str (param, value ? "true" : "false");
 
@@ -418,7 +423,7 @@ gda_parameter_dispose (GObject *object)
 		while (parameter->priv->param_users)
 			gda_parameter_del_user (parameter, GDA_OBJECT (parameter->priv->param_users->data));
 
-		parameter->priv->gda_type = GDA_VALUE_TYPE_UNKNOWN;
+		parameter->priv->g_type = G_TYPE_INVALID;
 
 		if (parameter->priv->value) {
 			gda_value_free (parameter->priv->value);
@@ -470,7 +475,7 @@ gda_parameter_set_property    (GObject *object,
 	if (parameter->priv) {
 		switch (param_id) {
 		case PROP_GDA_TYPE:
-			parameter->priv->gda_type = g_value_get_int (value);
+			parameter->priv->g_type = g_value_get_ulong (value);
 			break;
 		case PROP_PLUGIN:
 			ptr = g_value_get_string (value);
@@ -482,6 +487,9 @@ gda_parameter_set_property    (GObject *object,
 				parameter->priv->plugin = g_strdup (ptr);
 			break;
 		case PROP_USE_DEFAULT_VALUE:
+			if (g_value_get_boolean (value) == parameter->priv->default_forced)
+				break;
+
 			if (g_value_get_boolean (value)) {
 				if (!parameter->priv->has_default_value)
 					g_warning ("Can't force parameter to use default value if there "
@@ -493,19 +501,38 @@ gda_parameter_set_property    (GObject *object,
 			}
 			else
 				parameter->priv->default_forced = FALSE;
-
+			gda_object_signal_emit_changed (GDA_OBJECT (parameter));
 			break;
+		case PROP_NOT_NULL: {
+			gboolean not_null = g_value_get_boolean (value);
+			if (not_null != parameter->priv->not_null) {
+				parameter->priv->not_null = not_null;
+				
+				/* updating the parameter's validity regarding the NULL value */
+				if (!not_null && 
+				    (!parameter->priv->value || gda_value_is_null (parameter->priv->value)))
+					parameter->priv->valid = TRUE;
+				
+				if (not_null && 
+				    (!parameter->priv->value || gda_value_is_null (parameter->priv->value)))
+					parameter->priv->valid = FALSE;
+				
+				gda_object_signal_emit_changed (GDA_OBJECT (parameter));
+			}
+			break;
+		}
 		case PROP_SIMPLE_BIND:
-			gda_parameter_bind_to_param (parameter, g_value_get_pointer (value));
+			gda_parameter_bind_to_param (parameter, GDA_PARAMETER (g_value_get_object (value)));
 			break;
 		case PROP_FULL_BIND:
-			gda_parameter_set_full_bind_param (parameter, g_value_get_pointer (value));
+			gda_parameter_set_full_bind_param (parameter, GDA_PARAMETER (g_value_get_object (value)));
 			break;
-		case PROP_RESTRICT_MODEL:
-			ptr = g_value_get_pointer (value);
+		case PROP_RESTRICT_MODEL: {
+			GdaDataModel* ptr = g_value_get_object (value);
 			g_return_if_fail (gda_parameter_restrict_values (parameter, 
 									 (GdaDataModel *)ptr, -1, NULL));
 			break;
+                }
 		case PROP_RESTRICT_COLUMN:
 			parameter->priv->restrict_col = g_value_get_int (value);
 			break;
@@ -525,21 +552,24 @@ gda_parameter_get_property    (GObject *object,
 	if (parameter->priv) {
 		switch (param_id) {
 		case PROP_GDA_TYPE:
-			g_value_set_int (value, parameter->priv->gda_type);
+			g_value_set_ulong (value, parameter->priv->g_type);
 		case PROP_PLUGIN:
 			g_value_set_string (value, parameter->priv->plugin);
 			break;
 		case PROP_USE_DEFAULT_VALUE:
 			g_value_set_boolean (value, parameter->priv->default_forced);
 			break;
+		case PROP_NOT_NULL:
+			g_value_set_boolean (value, gda_parameter_get_not_null (parameter));
+			break;
 		case PROP_SIMPLE_BIND:
-			g_value_set_pointer (value, parameter->priv->change_with);
+			g_value_set_object (value, G_OBJECT (parameter->priv->change_with));
 			break;
 		case PROP_FULL_BIND:
-			g_value_set_pointer (value, parameter->priv->alias_of);
+			g_value_set_object (value, G_OBJECT (parameter->priv->alias_of));
 			break;
 		case PROP_RESTRICT_MODEL:
-			g_value_set_pointer (value, parameter->priv->restrict_model);
+			g_value_set_object (value, G_OBJECT (parameter->priv->restrict_model));
 			break;
 		case PROP_RESTRICT_COLUMN:
 			g_value_set_int (value, parameter->priv->restrict_col);
@@ -612,20 +642,20 @@ gda_parameter_replace_param_users (GdaParameter *param, GHashTable *replacements
 }
 
 /**
- * gda_parameter_get_gda_type
+ * gda_parameter_get_g_type
  * @param: a #GdaParameter object
  * 
  * Get the requested data type for @param.
  *
  * Returns: the data type
  */
-GdaValueType
-gda_parameter_get_gda_type (GdaParameter *param)
+GType
+gda_parameter_get_g_type (GdaParameter *param)
 {
-	g_return_val_if_fail (GDA_IS_PARAMETER (param), GDA_VALUE_TYPE_UNKNOWN);
-	g_return_val_if_fail (param->priv, GDA_VALUE_TYPE_UNKNOWN);
+	g_return_val_if_fail (GDA_IS_PARAMETER (param), G_TYPE_INVALID);
+	g_return_val_if_fail (param->priv, G_TYPE_INVALID);
 
-	return param->priv->gda_type;
+	return param->priv->g_type;
 }
 
 
@@ -635,9 +665,9 @@ gda_parameter_get_gda_type (GdaParameter *param)
  *
  * Get the value held into the parameter
  *
- * Returns: the value (a NULL value returns a GDA_VALUE_TYPE_NULL GdaValue)
+ * Returns: the value (a NULL value returns a GDA_TYPE_NULL GValue)
  */
-const GdaValue *
+const GValue *
 gda_parameter_get_value (GdaParameter *param)
 {
 	g_return_val_if_fail (GDA_IS_PARAMETER (param), NULL);
@@ -652,6 +682,39 @@ gda_parameter_get_value (GdaParameter *param)
 		return gda_parameter_get_value (param->priv->alias_of);
 }
 
+/**
+ * gda_parameter_get_value_str
+ * @param: a #GdaParameter object
+ * 
+ * Get a string representation of the value stored in @param. Calling
+ * gda_parameter_set_value_str () with this value will restore @param's current
+ * state.
+ *
+ * Returns: a new string, or %NULL if @param's value is NULL
+ */
+gchar *
+gda_parameter_get_value_str (GdaParameter *param)
+{
+	const GValue *current_val;
+
+	g_return_val_if_fail (GDA_IS_PARAMETER (param), NULL);
+	g_return_val_if_fail (param->priv, NULL);
+
+	current_val = gda_parameter_get_value (param);
+	if (!current_val)
+		return NULL;
+	else {
+		GdaDict *dict;
+		GdaDataHandler *dh;
+
+		dict = gda_object_get_dict (GDA_OBJECT (param));
+		dh = gda_dict_get_handler (dict, param->priv->g_type);
+		if (dh)
+			return gda_data_handler_get_str_from_value (dh, current_val);
+		else
+			return NULL;
+	}
+}
 
 /*
  * gda_parameter_set_value
@@ -668,11 +731,14 @@ gda_parameter_get_value (GdaParameter *param)
  * then @param is not chaged and no signal is emitted.
  */
 void
-gda_parameter_set_value (GdaParameter *param, const GdaValue *value)
+gda_parameter_set_value (GdaParameter *param, const GValue *value)
 {
 	gboolean changed = TRUE;
-	const GdaValue *current_val;
-#ifdef GDA_DEBUG_NO
+	const GValue *current_val;
+#define DEBUG_PARAM
+#undef DEBUG_PARAM
+
+#ifdef DEBUG_PARAM
 	gboolean was_valid = gda_parameter_is_valid (param);
 #endif
 
@@ -686,32 +752,33 @@ gda_parameter_set_value (GdaParameter *param, const GdaValue *value)
 	if (current_val == value)
 		changed = FALSE;
 
-	if (changed && gda_value_is_null ((GdaValue *)current_val) && 
-	    ((value && gda_value_is_null ((GdaValue *)value)) || !value)) 
+	if (changed && gda_value_is_null ((GValue *)current_val) && 
+	    ((value && gda_value_is_null ((GValue *)value)) || !value)) 
 		changed = FALSE;
 	
-	if (changed && value && (gda_value_get_type ((GdaValue *)value) == 
-				 gda_value_get_type ((GdaValue *)current_val)))
-		changed = gda_value_compare ((GdaValue *)value, (GdaValue *)current_val);
+	if (changed && value && (G_VALUE_TYPE ((GValue *)value) == 
+				 G_VALUE_TYPE ((GValue *)current_val)))
+		changed = gda_value_compare ((GValue *)value, (GValue *)current_val);
 		
 	/* param's validity */
 	param->priv->valid = TRUE;
-	if (!value || gda_value_is_null ((GdaValue *)value))
+	if (!value || gda_value_is_null ((GValue *)value))
 		if (param->priv->not_null)
 			param->priv->valid = FALSE;
 
 	if (value &&
-	    (gda_value_get_type ((GdaValue *)value) != GDA_VALUE_TYPE_NULL) &&
-	    (gda_value_get_type ((GdaValue *)value) != param->priv->gda_type))
+	    (G_VALUE_TYPE ((GValue *)value) != GDA_TYPE_NULL) &&
+	    (G_VALUE_TYPE ((GValue *)value) != param->priv->g_type)) {
 		param->priv->valid = FALSE;
+	}
 
-#ifdef GDA_DEBUG_NO
+#ifdef DEBUG_PARAM
 	g_print ("Changed param %p (%s): value %s --> %s \t(type %d -> %d) VALID: %d->%d CHANGED: %d\n", 
 		 param, gda_object_get_name (param),
-		 current_val ? gda_value_stringify ((GdaValue *)current_val) : "_NULL_",
-		 value ? gda_value_stringify ((GdaValue *)(value)) : "_NULL_",
-		 current_val ? gda_value_get_type ((GdaValue *)current_val) : 0,
-		 value ? gda_value_get_type ((GdaValue *)value) : 0, 
+		 current_val ? gda_value_stringify ((GValue *)current_val) : "_NULL_",
+		 value ? gda_value_stringify ((GValue *)(value)) : "_NULL_",
+		 current_val ? G_VALUE_TYPE ((GValue *)current_val) : 0,
+		 value ? G_VALUE_TYPE ((GValue *)value) : 0, 
 		 was_valid, gda_parameter_is_valid (param), changed);
 #endif
 
@@ -722,7 +789,7 @@ gda_parameter_set_value (GdaParameter *param, const GdaValue *value)
 			g_object_get (G_OBJECT (param), "changed_blocked", &changed_blocked, NULL);
 			if (!changed_blocked) {
 				g_object_set_data (G_OBJECT (param), "changed_pending", NULL);
-				gda_object_changed (GDA_OBJECT (param));
+				gda_object_signal_emit_changed (GDA_OBJECT (param));
 			}
 		}
 		return;
@@ -733,7 +800,7 @@ gda_parameter_set_value (GdaParameter *param, const GdaValue *value)
 
 	/* real setting of the value */
 	if (param->priv->alias_of) {
-#ifdef GDA_DEBUG_NO
+#ifdef DEBUG_PARAM
 		g_print ("Param %p is alias of param %p => propagating changes to param %p\n",
 			 param, param->priv->alias_of, param->priv->alias_of);
 #endif
@@ -747,7 +814,7 @@ gda_parameter_set_value (GdaParameter *param, const GdaValue *value)
 		}
 
 		if (value)
-			param->priv->value = gda_value_copy ((GdaValue *)value);
+			param->priv->value = gda_value_copy ((GValue *)value);
 
 		/* if the GdaObject has "changed_blocked" property TRUE, then store that the next time we need
 		 * to emit the "changed" signal even if the stored value has not changed "
@@ -756,7 +823,7 @@ gda_parameter_set_value (GdaParameter *param, const GdaValue *value)
 		if (changed_blocked)
 			g_object_set_data (G_OBJECT (param), "changed_pending", GINT_TO_POINTER (TRUE));
 		else
-			gda_object_changed (GDA_OBJECT (param));
+			gda_object_signal_emit_changed (GDA_OBJECT (param));
 	}
 }
 
@@ -769,7 +836,7 @@ gda_parameter_set_value (GdaParameter *param, const GdaValue *value)
  * is provided as a string, and may return FALSE if the string did not
  * represent a correct value for the data type of the parameter.
  *
- * Returns: TRUE if no error occured
+ * Returns: TRUE if no error occurred
  */
 gboolean
 gda_parameter_set_value_str (GdaParameter *param, const gchar *value)
@@ -777,39 +844,20 @@ gda_parameter_set_value_str (GdaParameter *param, const gchar *value)
 	g_return_val_if_fail (GDA_IS_PARAMETER (param), FALSE);
 	g_return_val_if_fail (param->priv, FALSE);
 
-	if (!value) {
+	if (!value || !g_ascii_strcasecmp (value, "NULL")) {
 		gda_parameter_set_value (param, NULL);	
 		return TRUE;
 	}
 	else {
-		GdaValue *gdaval = NULL;
+		GValue *gdaval = NULL;
 		GdaDict *dict;
-		GdaConnection *cnc = NULL;
+		GdaDataHandler *dh;
 
 		dict = gda_object_get_dict (GDA_OBJECT (param));
-		if (dict)
-			cnc = gda_dict_get_connection (dict);
+		dh = gda_dict_get_handler (dict, param->priv->g_type);
+		if (dh)
+			gdaval = gda_data_handler_get_value_from_str (dh, value, param->priv->g_type);
 		
-		if (cnc) {
-			GdaServerProvider *prov;
-			prov = gda_connection_get_provider_obj (cnc);
-			
-			if (prov) {
-				GdaDataHandler *dh;
-
-				dh = gda_server_provider_get_data_handler_gda (prov, cnc, param->priv->gda_type);
-				if (dh)
-					gdaval = gda_data_handler_get_value_from_str (dh, value, param->priv->gda_type);
-			}
-		}
-		else {
-			/* use dict's default data handlers */
-			GdaDataHandler *dh;
-
-			dh = gda_dict_get_default_handler (ASSERT_DICT (dict), param->priv->gda_type);
-			gdaval = gda_data_handler_get_value_from_str (dh, value, param->priv->gda_type);
-		}
-
 		if (gdaval) {
 			gda_parameter_set_value (param, gdaval);
 			gda_value_free (gdaval);
@@ -852,7 +900,7 @@ gda_parameter_declare_invalid (GdaParameter *param)
 	if (param->priv->alias_of) 
 		gda_parameter_declare_invalid (param->priv->alias_of);
 	else 
-		gda_object_changed (GDA_OBJECT (param));
+		gda_object_signal_emit_changed (GDA_OBJECT (param));
 }
 
 
@@ -879,7 +927,7 @@ gda_parameter_is_valid (GdaParameter *param)
 		}
 
 		if (param->priv->default_forced) {
-			return param->priv->default_value ? TRUE : FALSE;
+			return (param->priv->default_value || param->priv->has_default_value) ? TRUE : FALSE;
 		}
 		else {
 			return param->priv->valid;
@@ -897,7 +945,7 @@ gda_parameter_is_valid (GdaParameter *param)
  *
  * Returns: the default value
  */
-const GdaValue *
+const GValue *
 gda_parameter_get_default_value (GdaParameter *param)
 {
 	g_return_val_if_fail (GDA_IS_PARAMETER (param), NULL);
@@ -919,7 +967,7 @@ gda_parameter_get_default_value (GdaParameter *param)
  * the same type as the one required by @param.
  */
 void
-gda_parameter_set_default_value (GdaParameter *param, const GdaValue *value)
+gda_parameter_set_default_value (GdaParameter *param, const GValue *value)
 {
 	g_return_if_fail (GDA_IS_PARAMETER (param));
 	g_return_if_fail (param->priv);
@@ -931,7 +979,7 @@ gda_parameter_set_default_value (GdaParameter *param, const GdaValue *value)
 
 	if (value) {
 		param->priv->has_default_value = TRUE;
-		param->priv->default_value = gda_value_copy ((GdaValue *)value);
+		param->priv->default_value = gda_value_copy ((GValue *)value);
 	}
 	
 	/* don't emit the "changed" signal */
@@ -987,20 +1035,7 @@ gda_parameter_set_not_null (GdaParameter *param, gboolean not_null)
 	g_return_if_fail (GDA_IS_PARAMETER (param));
 	g_return_if_fail (param->priv);
 
-	if (not_null != param->priv->not_null) {
-		param->priv->not_null = not_null;
-
-		/* updating the parameter's validity regarding the NULL value */
-		if (!not_null && 
-		    (!param->priv->value || gda_value_is_null (param->priv->value)))
-			param->priv->valid = TRUE;
-
-		if (not_null && 
-		    (!param->priv->value || gda_value_is_null (param->priv->value)))
-			param->priv->valid = FALSE;
-
-		gda_object_changed (GDA_OBJECT (param));
-	}
+	g_object_set (G_OBJECT (param), "not_null", not_null, NULL);
 }
 
 /**
@@ -1115,8 +1150,8 @@ gda_parameter_has_restrict_values (GdaParameter *param, GdaDataModel **model, gi
 void
 gda_parameter_bind_to_param (GdaParameter *param, GdaParameter *bind_to)
 {
-	const GdaValue *cvalue;
-	GdaValue *value1 = NULL, *value2 = NULL;
+	const GValue *cvalue;
+	GValue *value1 = NULL, *value2 = NULL;
 
 	g_return_if_fail (GDA_IS_PARAMETER (param));
 	g_return_if_fail (param->priv);
@@ -1129,15 +1164,15 @@ gda_parameter_bind_to_param (GdaParameter *param, GdaParameter *bind_to)
 	if (bind_to) {
 		g_return_if_fail (bind_to && GDA_IS_PARAMETER (bind_to));
 		g_return_if_fail (bind_to->priv);
-		g_return_if_fail (param->priv->gda_type == bind_to->priv->gda_type);
+		g_return_if_fail (param->priv->g_type == bind_to->priv->g_type);
 		cvalue = gda_parameter_get_value (bind_to);
-		if (cvalue && !gda_value_is_null ((GdaValue*)cvalue))
-			value2 = gda_value_copy ((GdaValue*)cvalue);
+		if (cvalue && !gda_value_is_null ((GValue*)cvalue))
+			value2 = gda_value_copy ((GValue*)cvalue);
 	}
 
 	cvalue = gda_parameter_get_value (param);
-	if (cvalue && !gda_value_is_null ((GdaValue*)cvalue))
-		value1 = gda_value_copy ((GdaValue*)cvalue);
+	if (cvalue && !gda_value_is_null ((GValue*)cvalue))
+		value1 = gda_value_copy ((GValue*)cvalue);
 
 	/* get rid of the old alias */
 	if (param->priv->change_with) {
@@ -1160,7 +1195,7 @@ gda_parameter_bind_to_param (GdaParameter *param, GdaParameter *bind_to)
 
 		/* if alias_of has a different value than param, then we set param to the new value */
 		if (value1 && value2 &&
-		    (gda_value_get_type (value1) == gda_value_get_type (value2)))
+		    (G_VALUE_TYPE (value1) == G_VALUE_TYPE (value2)))
 			equal = !gda_value_compare (value1, value2);
 		else {
 			if (!value1 && !value2)
@@ -1186,8 +1221,8 @@ gda_parameter_bind_to_param (GdaParameter *param, GdaParameter *bind_to)
 static void
 gda_parameter_set_full_bind_param (GdaParameter *param, GdaParameter *alias_of)
 {
-	const GdaValue *cvalue;
-	GdaValue *value1 = NULL, *value2 = NULL;
+	const GValue *cvalue;
+	GValue *value1 = NULL, *value2 = NULL;
 
 	g_return_if_fail (GDA_IS_PARAMETER (param));
 	g_return_if_fail (param->priv);
@@ -1199,15 +1234,15 @@ gda_parameter_set_full_bind_param (GdaParameter *param, GdaParameter *alias_of)
 	if (alias_of) {
 		g_return_if_fail (alias_of && GDA_IS_PARAMETER (alias_of));
 		g_return_if_fail (alias_of->priv);
-		g_return_if_fail (param->priv->gda_type == alias_of->priv->gda_type);
+		g_return_if_fail (param->priv->g_type == alias_of->priv->g_type);
 		cvalue = gda_parameter_get_value (alias_of);
-		if (cvalue && !gda_value_is_null ((GdaValue*)cvalue))
-			value2 = gda_value_copy ((GdaValue*)cvalue);
+		if (cvalue && !gda_value_is_null ((GValue*)cvalue))
+			value2 = gda_value_copy ((GValue*)cvalue);
 	}
 
 	cvalue = gda_parameter_get_value (param);
-	if (cvalue && !gda_value_is_null ((GdaValue*)cvalue))
-		value1 = gda_value_copy ((GdaValue*)cvalue);
+	if (cvalue && !gda_value_is_null ((GValue*)cvalue))
+		value1 = gda_value_copy ((GValue*)cvalue);
 		
 	
 	/* get rid of the old alias */
@@ -1237,7 +1272,7 @@ gda_parameter_set_full_bind_param (GdaParameter *param, GdaParameter *alias_of)
 
 		/* if alias_of has a different value than param, then we emit a CHANGED signal */
 		if (value1 && value2 &&
-		    (gda_value_get_type (value1) == gda_value_get_type (value2)))
+		    (G_VALUE_TYPE (value1) == G_VALUE_TYPE (value2)))
 			equal = !gda_value_compare (value1, value2);
 		else {
 			if (!value1 && !value2)
@@ -1245,7 +1280,7 @@ gda_parameter_set_full_bind_param (GdaParameter *param, GdaParameter *alias_of)
 		}
 
 		if (!equal)
-			gda_object_changed (GDA_OBJECT (param));
+			gda_object_signal_emit_changed (GDA_OBJECT (param));
 	}
 	else {
 		/* restore the value that was in the previous alias parameter, 
@@ -1274,7 +1309,7 @@ alias_of_changed_cb (GdaParameter *alias_of, GdaParameter *param)
 {
 	if ((gpointer) alias_of == (gpointer) param->priv->alias_of) {
 		/* callback used as full bind */
-		gda_object_changed (GDA_OBJECT (param));
+		gda_object_signal_emit_changed (GDA_OBJECT (param));
 	}
 	else {
 		/* callback used as simple bind */
@@ -1317,29 +1352,26 @@ static void
 gda_parameter_dump (GdaParameter *parameter, guint offset)
 {
 	gchar *str;
-	guint i;
 
 	g_return_if_fail (parameter);
 	g_return_if_fail (GDA_IS_PARAMETER (parameter));
 
 	/* string for the offset */
 	str = g_new0 (gchar, offset+1);
-        for (i=0; i<offset; i++)
-                str[i] = ' ';
-        str[offset] = 0;
+	memset (str, ' ', offset);
 	
 	/* dump */
 	if (parameter->priv) {
 		GSList *list;
-		gchar *str;
+		gchar *strval;
 
-		str = gda_value_stringify (gda_parameter_get_value (parameter));
+		strval = gda_value_stringify ((GValue *) gda_parameter_get_value (parameter));
 		g_print ("%s" D_COL_H1 "GdaParameter %p (%s), type=%s, %s, value=%s\n" D_COL_NOR, str, parameter,
 			 gda_object_get_name (GDA_OBJECT (parameter)), 
-			 gda_type_to_string (parameter->priv->gda_type),
+			 gda_g_type_to_string (parameter->priv->g_type),
 			 gda_parameter_is_valid (parameter) ? "VALID" : "INVALID",
-			 str);
-		g_free (str);
+			 strval);
+		g_free (strval);
 		
 		list = parameter->priv->param_users;
 		while (list) {
@@ -1365,8 +1397,9 @@ gda_parameter_dump (GdaParameter *parameter, guint offset)
 			g_print ("\n");
 
 		if (parameter->priv->restrict_model) {
-				g_print ("%sValues restrictions: COLUMN %d\n", str, 
-					 parameter->priv->restrict_col);
+				g_print ("%sParameter restricted by column %d of data model %p\n", str,
+					 parameter->priv->restrict_col,
+					 parameter->priv->restrict_model);
 				gda_object_dump (GDA_OBJECT (parameter->priv->restrict_model), offset+5);
 		}
 
