@@ -1,6 +1,6 @@
 /* gda-query-field-value.c
  *
- * Copyright (C) 2003 - 2005 Vivien Malerba
+ * Copyright (C) 2003 - 2006 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -60,7 +60,7 @@ static gboolean    gda_query_field_value_load_from_xml (GdaXmlStorage *iface, xm
 /* Field interface */
 static void               gda_query_field_value_field_init      (GdaEntityFieldIface *iface);
 static GdaEntity         *gda_query_field_value_get_entity      (GdaEntityField *iface);
-static GdaValueType       gda_query_field_value_get_gda_type    (GdaEntityField *iface);
+static GType              gda_query_field_value_get_ig_type    (GdaEntityField *iface);
 static GdaDictType       *gda_query_field_value_get_data_type   (GdaEntityField *iface);
 
 /* Renderer interface */
@@ -85,7 +85,7 @@ static void destroyed_restrict_cb (GdaObject *obj, GdaQueryFieldValue *field);
 static GSList   *gda_query_field_value_get_params (GdaQueryField *qfield);
 
 static gboolean gda_query_field_value_render_find_value (GdaQueryFieldValue *field, GdaParameterList *context,
-					       const GdaValue **value_found, GdaParameter **param_source);
+					       const GValue **value_found, GdaParameter **param_source);
 
 #ifdef GDA_DEBUG
 static void gda_query_field_value_dump (GdaQueryFieldValue *field, guint offset);
@@ -102,7 +102,7 @@ enum
 	PROP_GDA_TYPE,
 	PROP_RESTRICT_MODEL,
         PROP_RESTRICT_COLUMN,
-	PROP_HANDLER_PLUGIN
+	PROP_ENTRY_PLUGIN
 };
 
 
@@ -110,11 +110,11 @@ enum
 struct _GdaQueryFieldValuePrivate
 {
 	GdaQuery              *query;
-	GdaValueType           gda_type;
+	GType                  g_type;
 	GdaDictType           *dict_type;
 
-	GdaValue              *value;        /* MUST either be NULL, or of type GDA_VALUE_NULL or 'type' */
-	GdaValue              *default_value;/* CAN either be NULL, or of any type */
+	GValue                *value;        /* MUST either be NULL, or of type GDA_VALUE_NULL or 'type' */
+	GValue                *default_value;/* CAN either be NULL, or of any type */
 	gboolean               is_parameter;
 	gboolean               is_null_allowed;
 
@@ -199,7 +199,7 @@ static void
 gda_query_field_value_field_init (GdaEntityFieldIface *iface)
 {
 	iface->get_entity = gda_query_field_value_get_entity;
-	iface->get_gda_type = gda_query_field_value_get_gda_type;
+	iface->get_g_type = gda_query_field_value_get_ig_type;
 	iface->get_data_type = gda_query_field_value_get_data_type;
 }
 
@@ -235,22 +235,24 @@ gda_query_field_value_class_init (GdaQueryFieldValueClass * class)
 	object_class->set_property = gda_query_field_value_set_property;
 	object_class->get_property = gda_query_field_value_get_property;
 	g_object_class_install_property (object_class, PROP_QUERY,
-					 g_param_spec_pointer ("query", "Query to which the field belongs", NULL, 
+					 g_param_spec_object ("query", "Query to which the field belongs", NULL, 
+                                                               GDA_TYPE_QUERY,
 							       (G_PARAM_READABLE | G_PARAM_WRITABLE |
 								G_PARAM_CONSTRUCT_ONLY)));
         g_object_class_install_property (object_class, PROP_GDA_TYPE,
-                                         g_param_spec_int ("gda_type", "Gda data type", NULL,
-							   GDA_VALUE_TYPE_NULL, GDA_VALUE_TYPE_UNKNOWN, GDA_VALUE_TYPE_UNKNOWN,
+                                         g_param_spec_ulong ("g_type", "Gda data type", NULL,
+							   0, G_MAXULONG, GDA_TYPE_NULL,
 							   (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 	g_object_class_install_property (object_class, PROP_RESTRICT_MODEL,
-                                         g_param_spec_pointer ("restrict_model", NULL, NULL,
+                                         g_param_spec_object ("restrict_model", NULL, NULL,
+                                                               GDA_TYPE_DATA_MODEL,
                                                                (G_PARAM_READABLE | G_PARAM_WRITABLE)));
         g_object_class_install_property (object_class, PROP_RESTRICT_COLUMN,
                                          g_param_spec_int ("restrict_column", NULL, NULL,
 							   0, G_MAXINT, 0,
 							   (G_PARAM_READABLE | G_PARAM_WRITABLE)));
-	g_object_class_install_property (object_class, PROP_HANDLER_PLUGIN,
-                                         g_param_spec_string ("handler_plugin", NULL, NULL, NULL,
+	g_object_class_install_property (object_class, PROP_ENTRY_PLUGIN,
+                                         g_param_spec_string ("entry_plugin", NULL, NULL, NULL,
                                                               (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 	/* virtual functions */
 #ifdef GDA_DEBUG
@@ -267,7 +269,7 @@ gda_query_field_value_init (GdaQueryFieldValue *gda_query_field_value)
 {
 	gda_query_field_value->priv = g_new0 (GdaQueryFieldValuePrivate, 1);
 	gda_query_field_value->priv->query = NULL;
-	gda_query_field_value->priv->gda_type = GDA_VALUE_TYPE_UNKNOWN;
+	gda_query_field_value->priv->g_type = G_TYPE_INVALID;
 	gda_query_field_value->priv->dict_type = NULL;
 	gda_query_field_value->priv->value = NULL;
 	gda_query_field_value->priv->default_value = NULL;
@@ -289,17 +291,17 @@ gda_query_field_value_init (GdaQueryFieldValue *gda_query_field_value)
  *
  * Returns: the new object
  */
-GObject*
-gda_query_field_value_new (GdaQuery *query, GdaValueType type)
+GdaQueryField*
+gda_query_field_value_new (GdaQuery *query, GType type)
 {
 	GObject *obj;
 
 	g_return_val_if_fail (GDA_IS_QUERY (query), NULL);
 
 	obj = g_object_new (GDA_TYPE_QUERY_FIELD_VALUE, "dict", gda_object_get_dict (GDA_OBJECT (query)), 
-			    "query", query, "gda_type", type, NULL);
+			    "query", query, "g_type", type, NULL);
 
-	return obj;
+	return (GdaQueryField*) obj;
 }
 
 
@@ -384,15 +386,14 @@ gda_query_field_value_set_property (GObject *object,
 				    GParamSpec *pspec)
 {
 	GdaQueryFieldValue *field;
-	gpointer ptr;
 	const gchar *str;
 	guint id;
 
 	field = GDA_QUERY_FIELD_VALUE (object);
 	if (field->priv) {
 		switch (param_id) {
-		case PROP_QUERY:
-			ptr = g_value_get_pointer (value);
+		case PROP_QUERY: {
+			GdaQuery* ptr = GDA_QUERY (g_value_get_object (value));
 			g_return_if_fail (ptr && GDA_IS_QUERY (ptr));
 
 			if (field->priv->query) {
@@ -409,19 +410,21 @@ gda_query_field_value_set_property (GObject *object,
 			g_object_get (G_OBJECT (ptr), "field_serial", &id, NULL);
 			gda_query_object_set_int_id (GDA_QUERY_OBJECT (field), id);
 			break;
+                }
 		case PROP_GDA_TYPE:
-			field->priv->gda_type = g_value_get_int (value);
+			field->priv->g_type = g_value_get_ulong (value);
 			break;
-		case PROP_RESTRICT_MODEL:
-			ptr = g_value_get_pointer (value);
+		case PROP_RESTRICT_MODEL: {
+			GdaDataModel* ptr = GDA_DATA_MODEL (g_value_get_object (value));
 			g_return_if_fail (gda_query_field_value_restrict (field, 
-									  (GdaDataModel *)ptr, -1,
+									  ptr, -1,
 									  NULL));
 			break;
+                }
 		case PROP_RESTRICT_COLUMN:
 			field->priv->restrict_col = g_value_get_int (value);
 			break;
-		case PROP_HANDLER_PLUGIN:
+		case PROP_ENTRY_PLUGIN:
 			str =  g_value_get_string (value);
 			if (field->priv->plugin) {
 				g_free (field->priv->plugin);
@@ -445,18 +448,18 @@ gda_query_field_value_get_property (GObject *object,
 	if (field->priv) {
 		switch (param_id) {
 		case PROP_QUERY:
-			g_value_set_pointer (value, field->priv->query);
+			g_value_set_object (value, G_OBJECT (field->priv->query));
 			break;
 		case PROP_GDA_TYPE:
-			g_value_set_int (value, field->priv->gda_type);
+			g_value_set_ulong (value, field->priv->g_type);
 			break;
 		case PROP_RESTRICT_MODEL:
-			g_value_set_pointer (value, field->priv->restrict_model);
+			g_value_set_object (value, G_OBJECT (field->priv->restrict_model));
 			break;
 		case PROP_RESTRICT_COLUMN:
 			g_value_set_int (value, field->priv->restrict_col);
 			break;
-		case PROP_HANDLER_PLUGIN:
+		case PROP_ENTRY_PLUGIN:
 			g_value_set_string (value, field->priv->plugin);
 			break;
 		}	
@@ -471,8 +474,9 @@ gda_query_field_value_copy (GdaQueryField *orig)
 	g_assert (GDA_IS_QUERY_FIELD_VALUE (orig));
 	qf = GDA_QUERY_FIELD_VALUE (orig);
 
-	obj = gda_query_field_value_new (qf->priv->query, qf->priv->gda_type);
-	nqf = GDA_QUERY_FIELD_VALUE (obj);
+	nqf = (GdaQueryFieldValue *) gda_query_field_value_new (qf->priv->query, qf->priv->g_type);
+	obj = (GObject *) nqf;
+
 	if (qf->priv->dict_type)
 		gda_query_field_value_set_dict_type (nqf, qf->priv->dict_type);
 
@@ -503,8 +507,8 @@ gda_query_field_value_is_equal (GdaQueryField *qfield1, GdaQueryField *qfield2)
 {
 	gboolean retval;
 	GdaQueryFieldValue *qf1, *qf2;
-	GdaValue *val1, *val2;
-	GdaValueType t1 = GDA_VALUE_TYPE_NULL, t2 = GDA_VALUE_TYPE_NULL;
+	GValue *val1, *val2;
+	GType t1 = GDA_TYPE_NULL, t2 = GDA_TYPE_NULL;
 
 	/* it is here assumed that qfield1 and qfield2 are of the same type and refer to the same
 	   query */
@@ -517,16 +521,16 @@ gda_query_field_value_is_equal (GdaQueryField *qfield1, GdaQueryField *qfield2)
 	val1 = qf1->priv->value;
 	val2 = qf2->priv->value;
 	if (val1)
-		t1 = gda_value_get_type (val1);
+		t1 = G_VALUE_TYPE (val1);
 	if (val2)
-		t2 = gda_value_get_type (val2);
+		t2 = G_VALUE_TYPE (val2);
 
 	retval = qf1->priv->dict_type == qf2->priv->dict_type ? TRUE : FALSE;
 
 	if (retval) 
 		retval = (t1 == t2) ? TRUE : FALSE;
 
-	if (retval && (t1 != GDA_VALUE_TYPE_NULL)) 
+	if (retval && (t1 != GDA_TYPE_NULL)) 
 		retval = gda_value_compare (val1, val2) ? FALSE : TRUE;
 
 	return retval;
@@ -540,13 +544,13 @@ gda_query_field_value_is_equal (GdaQueryField *qfield1, GdaQueryField *qfield2)
  * Sets the value of @field, or removes it (if @val is %NULL)
  */
 void
-gda_query_field_value_set_value (GdaQueryFieldValue *field, const GdaValue *val)
+gda_query_field_value_set_value (GdaQueryFieldValue *field, const GValue *val)
 {
 	g_return_if_fail (GDA_IS_QUERY_FIELD_VALUE (field));
 	g_return_if_fail (field->priv);	
 
 	if (val)
-		g_return_if_fail (gda_value_get_type ((GdaValue *)val) == field->priv->gda_type);
+		g_return_if_fail (G_VALUE_TYPE ((GValue *)val) == field->priv->g_type);
 
 	if (field->priv->value) {
 		gda_value_free (field->priv->value);
@@ -554,8 +558,8 @@ gda_query_field_value_set_value (GdaQueryFieldValue *field, const GdaValue *val)
 	}
 
 	if (val)
-		field->priv->value = gda_value_copy ((GdaValue *)val);
-	gda_object_changed (GDA_OBJECT (field));
+		field->priv->value = gda_value_copy ((GValue *)val);
+	gda_object_signal_emit_changed (GDA_OBJECT (field));
 }
 
 /**
@@ -568,7 +572,7 @@ gda_query_field_value_set_value (GdaQueryFieldValue *field, const GdaValue *val)
  *
  * Returns: the value or NULL
  */
-const GdaValue *
+const GValue *
 gda_query_field_value_get_value (GdaQueryFieldValue *field)
 {
 	g_return_val_if_fail (GDA_IS_QUERY_FIELD_VALUE (field), NULL);
@@ -587,7 +591,7 @@ gda_query_field_value_get_value (GdaQueryFieldValue *field)
  * Sets the default value of @field, or removes it (if @default_val is %NULL)
  */
 void
-gda_query_field_value_set_default_value (GdaQueryFieldValue *field, const GdaValue *default_val)
+gda_query_field_value_set_default_value (GdaQueryFieldValue *field, const GValue *default_val)
 {
 	g_return_if_fail (GDA_IS_QUERY_FIELD_VALUE (field));
 	g_return_if_fail (field->priv);
@@ -598,7 +602,7 @@ gda_query_field_value_set_default_value (GdaQueryFieldValue *field, const GdaVal
 	}
 
 	if (default_val) 
-		field->priv->default_value = gda_value_copy ((GdaValue *)default_val);
+		field->priv->default_value = gda_value_copy ((GValue *)default_val);
 }
 
 /**
@@ -609,7 +613,7 @@ gda_query_field_value_set_default_value (GdaQueryFieldValue *field, const GdaVal
  *
  * Returns: the value or NULL
  */
-const GdaValue *
+const GValue *
 gda_query_field_value_get_default_value (GdaQueryFieldValue *field)
 {
 	g_return_val_if_fail (GDA_IS_QUERY_FIELD_VALUE (field), NULL);
@@ -619,20 +623,20 @@ gda_query_field_value_get_default_value (GdaQueryFieldValue *field)
 }
 
 /**
- * gda_query_field_value_get_value_type
+ * gda_query_field_value_get_g_type
  * @field: a #GdaQueryFieldValue object
  *
  * Get the GDA type of value stored within @field
  *
  * Returns: the type
  */
-GdaValueType
-gda_query_field_value_get_value_type (GdaQueryFieldValue *field)
+GType
+gda_query_field_value_get_g_type (GdaQueryFieldValue *field)
 {
-	g_return_val_if_fail (GDA_IS_QUERY_FIELD_VALUE (field), GDA_VALUE_TYPE_UNKNOWN);
-	g_return_val_if_fail (field->priv, GDA_VALUE_TYPE_UNKNOWN);
+	g_return_val_if_fail (GDA_IS_QUERY_FIELD_VALUE (field), G_TYPE_INVALID);
+	g_return_val_if_fail (field->priv, G_TYPE_INVALID);
 
-	return field->priv->gda_type;
+	return field->priv->g_type;
 }
 
 /**
@@ -680,7 +684,7 @@ gda_query_field_value_get_params (GdaQueryField *qfield)
 
 		param = GDA_PARAMETER (g_object_new (GDA_TYPE_PARAMETER, 
 						     "dict", gda_object_get_dict (GDA_OBJECT (qfield)),
-						     "gda_type", field->priv->gda_type, NULL));
+						     "g_type", field->priv->g_type, NULL));
 		gda_parameter_declare_param_user (param, GDA_OBJECT (qfield));
 		
 		/* parameter's attributes */
@@ -694,12 +698,25 @@ gda_query_field_value_get_params (GdaQueryField *qfield)
 
 		/* specified plugin */
 		if (field->priv->plugin)
-			g_object_set (G_OBJECT (param), "handler_plugin", field->priv->plugin, NULL);
+			g_object_set (G_OBJECT (param), "entry_plugin", field->priv->plugin, NULL);
 
-		/* possible values in a sub query */
+		/* possible values in a data model */
 		if (field->priv->restrict_model && (field->priv->restrict_col >= 0)) {
 			gda_parameter_restrict_values (param, field->priv->restrict_model,
 						       field->priv->restrict_col, NULL);
+			if (GDA_IS_DATA_MODEL_QUERY (field->priv->restrict_model)) {
+				GdaParameterList *params;
+
+				params = gda_data_model_query_get_parameter_list 
+					(GDA_DATA_MODEL_QUERY (field->priv->restrict_model));
+				if (params) {
+					GSList *tmp;
+
+					tmp = g_slist_copy (params->parameters);
+					g_slist_foreach (tmp, (GFunc) g_object_ref, NULL);
+					list = g_slist_concat (list, tmp);
+				}
+			}
 		}
 
 		list = g_slist_append (list, param);
@@ -756,7 +773,7 @@ gboolean
 gda_query_field_value_is_value_null (GdaQueryFieldValue *field, GdaParameterList *context)
 {
 	gboolean value_found;
-	const GdaValue *value;
+	const GValue *value;
 
 	g_return_val_if_fail (GDA_IS_QUERY_FIELD_VALUE (field), FALSE);
 	g_return_val_if_fail (field->priv, FALSE);
@@ -765,7 +782,7 @@ gda_query_field_value_is_value_null (GdaQueryFieldValue *field, GdaParameterList
 	if (!value_found) 
 		value = field->priv->value;
 
-	if (!value || gda_value_is_null ((GdaValue *)value))
+	if (!value || gda_value_is_null ((GValue *)value))
 		return TRUE;
 	else
 		return FALSE;
@@ -782,7 +799,7 @@ gda_query_field_value_is_value_null (GdaQueryFieldValue *field, GdaParameterList
  * Restricts the possible values which @field can have among the calues stored in
  * @model at column @col.
  *
- * Returns: TRUE if no error occured
+ * Returns: TRUE if no error occurred
  */
 gboolean
 gda_query_field_value_restrict (GdaQueryFieldValue *field, GdaDataModel *model, gint col, GError **error)
@@ -856,14 +873,14 @@ gda_query_field_value_set_dict_type (GdaQueryFieldValue *field, GdaDictType *typ
 		gda_object_connect_destroy (type,
 					    G_CALLBACK (destroyed_type_cb), field);
 		
-		if (field->priv->gda_type != gda_dict_type_get_gda_type (type)) {
+		if (field->priv->g_type != gda_dict_type_get_g_type (type)) {
 			g_warning ("GdaQueryFieldValue: setting to GDA type incompatible dict type");
-			field->priv->gda_type = gda_dict_type_get_gda_type (type);
+			field->priv->g_type = gda_dict_type_get_g_type (type);
 		}
 	}
 
 	/* signal a change */
-	gda_object_changed (GDA_OBJECT (field));
+	gda_object_signal_emit_changed (GDA_OBJECT (field));
 }
 
 /**
@@ -888,7 +905,6 @@ static void
 gda_query_field_value_dump (GdaQueryFieldValue *field, guint offset)
 {
 	gchar *str;
-	gint i;
 	GdaDataHandler *dh;
 	GdaDict *dict;
 
@@ -896,15 +912,14 @@ gda_query_field_value_dump (GdaQueryFieldValue *field, guint offset)
 	dict = gda_object_get_dict (GDA_OBJECT (field));
 	
         /* string for the offset */
-        str = g_new0 (gchar, offset+1);
-        for (i=0; i<offset; i++)
-                str[i] = ' ';
+        str = g_new (gchar, offset+1);
+	memset (str, ' ', offset);
         str[offset] = 0;
 
         /* dump */
         if (field->priv) {
 		gchar *val;
-		dh = gda_dict_get_default_handler (dict, field->priv->gda_type);
+		dh = gda_dict_get_default_handler (dict, field->priv->g_type);
                 g_print ("%s" D_COL_H1 "GdaQueryFieldValue" D_COL_NOR " \"%s\" (%p, id=%s) ",
                          str, gda_object_get_name (GDA_OBJECT (field)), field, 
 			 gda_object_get_id (GDA_OBJECT (field)));
@@ -926,7 +941,7 @@ gda_query_field_value_dump (GdaQueryFieldValue *field, guint offset)
 
 		if (field->priv->default_value) {
 			GdaDataHandler *dhd;
-			dhd = gda_dict_get_default_handler (dict, gda_value_get_type (field->priv->default_value));
+			dhd = gda_dict_get_default_handler (dict, G_VALUE_TYPE (field->priv->default_value));
 			val = gda_data_handler_get_sql_from_value (dhd, field->priv->default_value);
 			g_print ("Default: %s ", val);
 			g_free (val);
@@ -934,7 +949,9 @@ gda_query_field_value_dump (GdaQueryFieldValue *field, guint offset)
 		g_print ("\n");
 
 		if (field->priv->restrict_model) {
-			g_print ("%sValues restrictions: COLUMN %d\n", str, field->priv->restrict_col);
+			g_print ("%sValue restricted by column %d of data model %p:\n", str, 
+				 field->priv->restrict_col,
+				 field->priv->restrict_model);
 			gda_object_dump (GDA_OBJECT (field->priv->restrict_model), offset+5);
 		}
 	}
@@ -956,13 +973,13 @@ gda_query_field_value_get_entity (GdaEntityField *iface)
 	return GDA_ENTITY (GDA_QUERY_FIELD_VALUE (iface)->priv->query);
 }
 
-static GdaValueType
-gda_query_field_value_get_gda_type (GdaEntityField *iface)
+static GType
+gda_query_field_value_get_ig_type (GdaEntityField *iface)
 {
-	g_return_val_if_fail (iface && GDA_IS_QUERY_FIELD_VALUE (iface), GDA_VALUE_TYPE_UNKNOWN);
-	g_return_val_if_fail (GDA_QUERY_FIELD_VALUE (iface)->priv, GDA_VALUE_TYPE_UNKNOWN);
+	g_return_val_if_fail (iface && GDA_IS_QUERY_FIELD_VALUE (iface), G_TYPE_INVALID);
+	g_return_val_if_fail (GDA_QUERY_FIELD_VALUE (iface)->priv, G_TYPE_INVALID);
 
-	return GDA_QUERY_FIELD_VALUE (iface)->priv->gda_type;
+	return GDA_QUERY_FIELD_VALUE (iface)->priv->g_type;
 }
 
 static GdaDictType *
@@ -1008,37 +1025,54 @@ gda_query_field_value_save_to_xml (GdaXmlStorage *iface, GError **error)
 		xmlSetProp (node, "is_internal", "t");
 
 	xmlSetProp (node, "is_param", field->priv->is_parameter ? "t" : "f");
-	xmlSetProp (node, "gda_type",  gda_type_to_string (field->priv->gda_type));
+	xmlSetProp (node, "g_type",  gda_g_type_to_string (field->priv->g_type));
 	if (field->priv->dict_type) {
 		str = gda_xml_storage_get_xml_id (GDA_XML_STORAGE (field->priv->dict_type));
 		xmlSetProp (node, "dict_type",  str);
 		g_free (str);
 	}
 
-	dh = gda_dict_get_default_handler (dict, field->priv->gda_type);
-	if (field->priv->value && (field->priv->gda_type != GDA_VALUE_TYPE_NULL)) {
+	dh = gda_dict_get_default_handler (dict, field->priv->g_type);
+	if (field->priv->value && (field->priv->g_type != GDA_TYPE_NULL)) {
 		str = gda_data_handler_get_str_from_value (dh, field->priv->value);
 		xmlSetProp (node, "value", str);
 		g_free (str);
 	}
 	if (field->priv->default_value) {
-		GdaDataHandler *dhd = gda_dict_get_default_handler (dict, gda_value_get_type (field->priv->default_value));
-		GdaValueType vtype;
+		GdaDataHandler *dhd = gda_dict_get_default_handler (dict, G_VALUE_TYPE (field->priv->default_value));
+		GType vtype;
 		
 		str = gda_data_handler_get_str_from_value (dhd, field->priv->default_value);
 		xmlSetProp (node, "default", str);
 		g_free (str);
-		vtype = gda_value_get_type (field->priv->default_value);
-		xmlSetProp (node, "default_gda_type", gda_type_to_string (vtype));
+		vtype = G_VALUE_TYPE (field->priv->default_value);
+		xmlSetProp (node, "default_g_type", gda_g_type_to_string (vtype));
 	}
 
-	xmlSetProp (node, "null_ok", field->priv->is_null_allowed ? "t" : "f");
+	xmlSetProp (node, "nullok", field->priv->is_null_allowed ? "t" : "f");
+
 	if (field->priv->restrict_model) {
-		str = g_strdup_printf ("DA%s:%d", 
-				       gda_object_get_name (GDA_OBJECT (field->priv->restrict_model)),
-				       field->priv->restrict_col);
-		xmlSetProp (node, "restrict", str);
-		g_free (str);
+		GSList *psources = (GSList *) gda_query_get_param_sources (field->priv->query);
+		str = NULL;
+		if (g_slist_find (psources, field->priv->restrict_model))
+			/* restricting model is in query's params sources */
+			str = g_strdup_printf ("_%d:%d", g_slist_index (psources, field->priv->restrict_model), 
+					       field->priv->restrict_col);
+		else {
+			if (gda_dict_object_is_assumed (gda_object_get_dict ((GdaObject*) field), 
+							GDA_OBJECT (field->priv->restrict_model)))
+				str = g_strdup_printf ("DA%s:%d", 
+						       gda_object_get_name (GDA_OBJECT (field->priv->restrict_model)),
+						       field->priv->restrict_col);
+			else {
+				g_warning (_("GdaDataModelQuery data model restricting GdaQueryFieldValue "
+					     "is not saved in the dictionary"));
+			}
+		}
+		if (str) {
+			xmlSetProp (node, "restrict", str);
+			g_free (str);
+		}
 	}
 
 	cstr = gda_query_field_get_alias (GDA_QUERY_FIELD (field));
@@ -1087,101 +1121,101 @@ gda_query_field_value_load_from_xml (GdaXmlStorage *iface, xmlNodePtr node, GErr
 			return FALSE;
 		}
 		gda_query_object_set_int_id (GDA_QUERY_OBJECT (field), atoi (ptr+2));
-		g_free (prop);
+		xmlFree (prop);
 	}
 
 	prop = xmlGetProp (node, "name");
 	if (prop) {
 		gda_object_set_name (GDA_OBJECT (field), prop);
-		g_free (prop);
+		xmlFree (prop);
 	}
 
 	prop = xmlGetProp (node, "descr");
 	if (prop) {
 		gda_object_set_description (GDA_OBJECT (field), prop);
-		g_free (prop);
+		xmlFree (prop);
 	}
 
 	prop = xmlGetProp (node, "is_visible");
 	if (prop) {
 		gda_query_field_set_visible (GDA_QUERY_FIELD (field), (*prop == 't') ? TRUE : FALSE);
-		g_free (prop);
+		xmlFree (prop);
 	}
 
 	prop = xmlGetProp (node, "is_internal");
 	if (prop) {
 		gda_query_field_set_internal (GDA_QUERY_FIELD (field), (*prop == 't') ? TRUE : FALSE);
-		g_free (prop);
+		xmlFree (prop);
 	}
 
-	prop = xmlGetProp (node, "gda_type");
+	prop = xmlGetProp (node, "g_type");
 	if (prop) {
-		field->priv->gda_type = gda_type_from_string (prop);
-		dh = gda_dict_get_default_handler (dict, field->priv->gda_type);
-		g_free (prop);
+		field->priv->g_type = gda_g_type_from_string (prop);
+		dh = gda_dict_get_default_handler (dict, field->priv->g_type);
+		xmlFree (prop);
 
-		if (field->priv->gda_type == GDA_VALUE_TYPE_NULL)
+		if (field->priv->g_type == GDA_TYPE_NULL)
 			field->priv->value = gda_value_new_null ();
 	}
 
 	prop = xmlGetProp (node, "dict_type");
 	if (prop) {
 		GdaDictType *dict_type;
-		dict_type = gda_dict_get_data_type_by_xml_id (dict, prop);
+		dict_type = gda_dict_get_dict_type_by_xml_id (dict, prop);
 		if (dict_type) {
 			gda_query_field_value_set_dict_type (field, dict_type);
-			dh = gda_dict_get_default_handler (dict, gda_dict_type_get_gda_type (field->priv->dict_type));
+			dh = gda_dict_get_default_handler (dict, gda_dict_type_get_g_type (field->priv->dict_type));
 		}
-		g_free (prop);
+		xmlFree (prop);
 	}
 
 	prop = xmlGetProp (node, "value");
 	if (prop) {
 		if (dh) {
-			field->priv->value = gda_data_handler_get_value_from_str (dh, prop, field->priv->gda_type);
+			field->priv->value = gda_data_handler_get_value_from_str (dh, prop, field->priv->g_type);
 			if (!field->priv->value) {
 				g_set_error (error,
 					     GDA_QUERY_FIELD_VALUE_ERROR,
 					     GDA_QUERY_FIELD_VALUE_XML_LOAD_ERROR,
 					     _("Can't interpret '%s' as a value"), prop);
-				g_free (prop);
+				xmlFree (prop);
 				return FALSE;
 			}
 		}
 
-		g_free (prop);
+		xmlFree (prop);
 	}
 
 	prop = xmlGetProp (node, "default");
 	if (prop) {
 		gchar *str2;
 
-		str2 = xmlGetProp (node, "default_gda_type");
+		str2 = xmlGetProp (node, "default_g_type");
 		if (str2) {
 			GdaDataHandler *dh2;
-			GdaValueType vtype;
-			GdaValue *value;
+			GType vtype;
+			GValue *value;
 			
-			vtype = gda_type_from_string (str2);			
+			vtype = gda_g_type_from_string (str2);			
 			dh2 = gda_dict_get_default_handler (dict, vtype);
 			value = gda_data_handler_get_value_from_str (dh2, prop, vtype);
 			field->priv->default_value = value;
 
-			g_free (str2);
+			xmlFree (str2);
 		}
-		g_free (prop);
+		xmlFree (prop);
 	}
 
 	prop = xmlGetProp (node, "is_param");
 	if (prop) {
 		field->priv->is_parameter = (*prop == 't') ? TRUE : FALSE;
-		g_free (prop);
+		xmlFree (prop);
 	}
 
-	prop = xmlGetProp (node, "null_ok");
+	prop = xmlGetProp (node, "nullok");
 	if (prop) {
 		field->priv->is_null_allowed = (*prop == 't') ? TRUE : FALSE;
-		g_free (prop);
+		xmlFree (prop);
 	}
 
 	prop = xmlGetProp (node, "plugin");
@@ -1190,26 +1224,69 @@ gda_query_field_value_load_from_xml (GdaXmlStorage *iface, xmlNodePtr node, GErr
 
 	prop = xmlGetProp (node, "restrict");
 	if (prop) {
-		GdaDataModel *model;
-		gint col;
-		TO_IMPLEMENT; /* find data model from name */
-		if (!gda_query_field_value_restrict (field, model, col, error))
+		GdaDataModel *model = NULL;
+		gint col = -1;
+
+		if (*prop == '_') {
+			/* restriction is done by a query's param source */
+			gchar *ptr;
+			gint pos;
+
+			for (ptr = prop + 1; *ptr && (*ptr != ':'); ptr++);
+			if (*ptr == ':') {
+				*ptr = 0;
+				pos = atoi (prop + 1);
+				col = atoi (ptr+1);
+
+				model = g_slist_nth_data ((GSList *) gda_query_get_param_sources (field->priv->query), pos);
+				if (!model) {
+					g_set_error (error, GDA_QUERY_FIELD_VALUE_ERROR,
+						     GDA_QUERY_FIELD_VALUE_XML_LOAD_ERROR,
+						     _("Query's param sources has no data model at position %d"), pos);
+					err = TRUE;	
+				}
+			}
+			else {
+				g_set_error (error, GDA_QUERY_FIELD_VALUE_ERROR,
+					     GDA_QUERY_FIELD_VALUE_XML_LOAD_ERROR,
+					     _("'restrict' attribute has a wrong format"));
+				err = TRUE;
+			}
+		}
+		else {
+			if (strlen (prop) <= 2)
+				err = TRUE;
+			else {
+				TO_IMPLEMENT; /* find data model from name */
+				g_set_error (error, GDA_QUERY_FIELD_VALUE_ERROR,
+					     GDA_QUERY_FIELD_VALUE_XML_LOAD_ERROR,
+					     _("Feature not yet implemented, see %s(), line %d"), 
+					     __FUNCTION__, __LINE__);
+			}
+		}
+			
+		if (model) {
+			if (!gda_query_field_value_restrict (field, model, col, error))
+				err = TRUE;
+		}
+		else
 			err = TRUE;
-		g_free (prop);
+
+		xmlFree (prop);
 	}
 
 	prop = xmlGetProp (node, "alias");
 	if (prop) {
 		gda_query_field_set_alias (GDA_QUERY_FIELD (field), prop);
-		g_free (prop);
+		xmlFree (prop);
 	}
 
-	if (!dh && (field->priv->gda_type != GDA_VALUE_TYPE_NULL)) {
+	if (!dh && (field->priv->g_type != GDA_TYPE_NULL)) {
 		err = TRUE;
 		g_set_error (error,
 			     GDA_QUERY_FIELD_VALUE_ERROR,
 			     GDA_QUERY_FIELD_VALUE_XML_LOAD_ERROR,
-			     _("Missing required gda_type for <gda_query_fval>"));
+			     _("Missing required g_type for <gda_query_fval>"));
 	}
 
 	if (!err) {
@@ -1235,9 +1312,9 @@ gda_query_field_value_load_from_xml (GdaXmlStorage *iface, xmlNodePtr node, GErr
 
 static gboolean
 gda_query_field_value_render_find_value (GdaQueryFieldValue *field, GdaParameterList *context, 
-			       const GdaValue **value_found, GdaParameter **param_source)
+			       const GValue **value_found, GdaParameter **param_source)
 {
-	const GdaValue *cvalue = NULL;
+	const GValue *cvalue = NULL;
 	gboolean found = FALSE;
 
 	if (param_source)
@@ -1279,20 +1356,15 @@ gda_query_field_value_render_as_sql (GdaRenderer *iface, GdaParameterList *conte
 {
 	gchar *str = NULL;
 	GdaQueryFieldValue *field;
-	const GdaValue *value = NULL;
+	const GValue *value = NULL;
 	gboolean sqlext = options & GDA_RENDERER_EXTRA_VAL_ATTRS;
-	GdaServerProvider *prov = NULL;
 	GdaDict *dict;
-	GdaConnection *cnc;
 
 	g_return_val_if_fail (iface && GDA_IS_QUERY_FIELD_VALUE (iface), NULL);
 	g_return_val_if_fail (GDA_QUERY_FIELD_VALUE (iface)->priv, NULL);
 	field = GDA_QUERY_FIELD_VALUE (iface);
 
 	dict = gda_object_get_dict (GDA_OBJECT (field));
-	cnc = gda_dict_get_connection (dict);
-	if (cnc)
-		prov = gda_connection_get_provider_obj (cnc);
 
 	if (field->priv->is_parameter) {
 		gboolean value_found;
@@ -1305,7 +1377,7 @@ gda_query_field_value_render_as_sql (GdaRenderer *iface, GdaParameterList *conte
 			if (param_source && ! gda_parameter_is_valid (param_source)) {
 				gchar *str, *str2;
 				
-				str2 = value ? gda_value_stringify ((GdaValue *)value) : g_strdup ("NULL");
+				str2 = value ? gda_value_stringify ((GValue *)value) : g_strdup ("NULL");
 				str = g_strdup_printf (_("Invalid parameter '%s' (value: %s)"),
 						       gda_object_get_name (GDA_OBJECT (param_source)), str2);
 				g_free (str2);
@@ -1338,16 +1410,14 @@ gda_query_field_value_render_as_sql (GdaRenderer *iface, GdaParameterList *conte
 				}
 			}
 			if (!str) {
-				if (value && (gda_value_get_type ((GdaValue *)value) != GDA_VALUE_TYPE_NULL)) {
-					GdaDataHandler *dh = NULL;
-					if (prov)
-						dh = gda_server_provider_get_data_handler_gda (prov, cnc, 
-											       field->priv->gda_type);
-					if (!dh)
-						dh = gda_dict_get_default_handler (dict, field->priv->gda_type);
-					str = gda_data_handler_get_sql_from_value (dh, value);
+				if (value && (G_VALUE_TYPE ((GValue *)value) != GDA_TYPE_NULL)) {
+					GdaDataHandler *dh;
+					
+					dh = gda_dict_get_handler (dict, field->priv->g_type);
+					if (dh)
+						str = gda_data_handler_get_sql_from_value (dh, value);
 				}
-				else
+				if (!str)
 					str = g_strdup ("NULL");	
 			}
 		}
@@ -1369,16 +1439,14 @@ gda_query_field_value_render_as_sql (GdaRenderer *iface, GdaParameterList *conte
 	}
 	else {
 		value = field->priv->value;
-		if (value && (gda_value_get_type ((GdaValue *)value) != GDA_VALUE_TYPE_NULL)) {
-			GdaDataHandler *dh = NULL;
+		if (value && (G_VALUE_TYPE ((GValue *)value) != GDA_TYPE_NULL)) {
+			GdaDataHandler *dh;
 			
-			if (prov)
-				dh = gda_server_provider_get_data_handler_gda (prov, cnc, 
-									       field->priv->gda_type);
-			if (!dh)
-				dh = gda_dict_get_default_handler (dict, field->priv->gda_type);
-			g_assert (dh);
-			str = gda_data_handler_get_sql_from_value (dh, value);
+			dh = gda_dict_get_handler (dict, field->priv->g_type);
+			if (dh)
+				str = gda_data_handler_get_sql_from_value (dh, value);
+			else
+				str = g_strdup ("NULL");
 		}
 		else
 			str = g_strdup ("NULL");	
@@ -1404,8 +1472,8 @@ gda_query_field_value_render_as_sql (GdaRenderer *iface, GdaParameterList *conte
 			g_string_append_printf (extra, ":type=\"%s\"", 
 						gda_object_get_name (GDA_OBJECT (field->priv->dict_type)));
 		else
-			/* print gda_type instead */
-			g_string_append_printf (extra, ":type=\"%s\"", gda_type_to_string (field->priv->gda_type));
+			/* print g_type instead */
+			g_string_append_printf (extra, ":type=\"%s\"", gda_g_type_to_string (field->priv->g_type));
 
 		tmpstr = gda_object_get_name (GDA_OBJECT (field));
 		if (tmpstr && *tmpstr) {
