@@ -699,63 +699,125 @@ gda_server_provider_get_last_insert_id (GdaServerProvider *provider, GdaConnecti
  * gda_server_provider_begin_transaction
  * @provider:
  * @cnc:
- * @xaction:
+ * @name: the name of the transation to start
+ * @level:
+ * @error: a place to store errors, or %NULL
  *
  * Returns:
- * 
  */
 gboolean
 gda_server_provider_begin_transaction (GdaServerProvider *provider,
 				       GdaConnection *cnc,
-				       GdaTransaction *xaction)
+				       const gchar *name, GdaTransactionIsolation level,
+				       GError **error)
 {
 	g_return_val_if_fail (GDA_IS_SERVER_PROVIDER (provider), FALSE);
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
 	g_return_val_if_fail (CLASS (provider)->begin_transaction != NULL, FALSE);
 
-	return CLASS (provider)->begin_transaction (provider, cnc, xaction);
+	return CLASS (provider)->begin_transaction (provider, cnc, name, level, error);
 }
 
 /**
  * gda_server_provider_commit_transaction
  * @provider:
  * @cnc:
- * @xaction:
+ * @name: the name of the transation to commit
+ * @error: a place to store errors, or %NULL
  *
  * Returns:
- * 
  */
 gboolean
 gda_server_provider_commit_transaction (GdaServerProvider *provider,
 					GdaConnection *cnc,
-					GdaTransaction *xaction)
+					const gchar *name, GError **error)
 {
 	g_return_val_if_fail (GDA_IS_SERVER_PROVIDER (provider), FALSE);
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
 	g_return_val_if_fail (CLASS (provider)->commit_transaction != NULL, FALSE);
 
-	return CLASS (provider)->commit_transaction (provider, cnc, xaction);
+	return CLASS (provider)->commit_transaction (provider, cnc, name, error);
 }
 
 /**
  * gda_server_provider_rollback_transaction
  * @provider:
  * @cnc:
- * @xaction:
+ * @name: the name of the transation to commit
+ * @error: a place to store errors, or %NULL
  *
  * Returns:
- * 
  */
 gboolean
 gda_server_provider_rollback_transaction (GdaServerProvider *provider,
 					  GdaConnection *cnc,
-					  GdaTransaction *xaction)
+					  const gchar *name, GError **error)
 {
 	g_return_val_if_fail (GDA_IS_SERVER_PROVIDER (provider), FALSE);
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
 	g_return_val_if_fail (CLASS (provider)->rollback_transaction != NULL, FALSE);
 
-	return CLASS (provider)->rollback_transaction (provider, cnc, xaction);
+	return CLASS (provider)->rollback_transaction (provider, cnc, name, error);
+}
+
+/**
+ * gda_server_provider_add_savepoint
+ * @provider: a #GdaServerProvider object
+ * @cnc: a #GdaConnection object
+ * @name: name of the savepoint to add
+ * @error: a place to store errors or %NULL
+ *
+ * Returns: TRUE if no error occurred
+ */
+gboolean
+gda_server_provider_add_savepoint (GdaServerProvider *provider, GdaConnection *cnc, 
+				   const gchar *name, GError **error)
+{
+	g_return_val_if_fail (GDA_IS_SERVER_PROVIDER (provider), FALSE);
+	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
+	g_return_val_if_fail (CLASS (provider)->add_savepoint != NULL, FALSE);
+
+	return CLASS (provider)->add_savepoint (provider, cnc, name, error);
+}
+
+/**
+ * gda_server_provider_rollback_savepoint
+ * @provider: a #GdaServerProvider object
+ * @cnc: a #GdaConnection object
+ * @name: name of the savepoint to rollback to
+ * @error: a place to store errors or %NULL
+ *
+ * Returns: TRUE if no error occurred
+ */
+gboolean
+gda_server_provider_rollback_savepoint (GdaServerProvider *provider, GdaConnection *cnc, 
+					const gchar *name, GError **error)
+{
+	g_return_val_if_fail (GDA_IS_SERVER_PROVIDER (provider), FALSE);
+	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
+	g_return_val_if_fail (CLASS (provider)->rollback_savepoint != NULL, FALSE);
+
+	return CLASS (provider)->rollback_savepoint (provider, cnc, name, error);
+}
+
+/**
+ * gda_server_provider_delete_savepoint
+ * @provider: a #GdaServerProvider object
+ * @cnc: a #GdaConnection object
+ * @name: name of the savepoint to delete
+ * @error: a place to store errors or %NULL
+ *
+ * Returns: TRUE if no error occurred
+ */
+gboolean
+gda_server_provider_delete_savepoint (GdaServerProvider *provider, GdaConnection *cnc, 
+				      const gchar *name, GError **error)
+{
+	g_return_val_if_fail (GDA_IS_SERVER_PROVIDER (provider), FALSE);
+	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
+	g_return_val_if_fail (CLASS (provider)->delete_savepoint != NULL, FALSE);
+
+	return CLASS (provider)->delete_savepoint (provider, cnc, name, error);
 }
 
 /**
@@ -771,12 +833,44 @@ gda_server_provider_supports_feature (GdaServerProvider *provider,
 			      GdaConnection *cnc,
 			      GdaConnectionFeature feature)
 {
+	gboolean retval;
+
 	g_return_val_if_fail (GDA_IS_SERVER_PROVIDER (provider), FALSE);
 	if (cnc)
 		g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
 	g_return_val_if_fail (CLASS (provider)->supports_feature != NULL, FALSE);
 
-	return CLASS (provider)->supports_feature (provider, cnc, feature);
+	retval = CLASS (provider)->supports_feature (provider, cnc, feature);
+
+	if (retval) {
+		switch (feature) {
+		case GDA_CONNECTION_FEATURE_TRANSACTIONS:
+			if (!CLASS (provider)->begin_transaction ||
+			    !CLASS (provider)->commit_transaction ||
+			    !CLASS (provider)->rollback_transaction)
+				retval = FALSE;
+			break;
+		case GDA_CONNECTION_FEATURE_SAVEPOINTS:
+			if (!CLASS (provider)->add_savepoint ||
+			    !CLASS (provider)->rollback_savepoint)
+				retval = FALSE;
+			break;
+		case GDA_CONNECTION_FEATURE_SAVEPOINTS_REMOVE:
+			if (!CLASS (provider)->delete_savepoint)
+				retval = FALSE;
+			break;
+		case GDA_CONNECTION_FEATURE_BLOBS:
+			if (!CLASS (provider)->create_blob ||
+			    !CLASS (provider)->fetch_blob)
+				retval = FALSE;
+			break;
+		default:
+			break;
+		}
+	}
+
+	return retval;
+
 }
 
 /**
