@@ -42,13 +42,13 @@
 #endif
 
 /**
- * gda_type_to_string
+ * gda_g_type_to_string
  * @type: Type to convert from.
  *
  * Returns: the string representing the given #GType.
  */
 const gchar *
-gda_type_to_string (GType type)
+gda_g_type_to_string (GType type)
 {
 	if (type == GDA_TYPE_NULL)
 		return "null";
@@ -57,13 +57,13 @@ gda_type_to_string (GType type)
 }
 
 /**
- * gda_type_from_string
- * @str: the name of a #GType, as returned by gda_type_to_string().
+ * gda_g_type_from_string
+ * @str: the name of a #GType, as returned by gda_g_type_to_string().
  *
  * Returns: the #GType represented by the given @str.
  */
 GType
-gda_type_from_string (const gchar *str)
+gda_g_type_from_string (const gchar *str)
 {
 	GType type;
 	g_return_val_if_fail (str != NULL, G_TYPE_INVALID);
@@ -217,38 +217,6 @@ gda_string_hash_to_list (GHashTable *hash_table)
         return list;
 }
 
-/**
- * gda_sql_replace_placeholders
- * @sql: a SQL command containing placeholders for values.
- * @params: a list of values for the placeholders.
- *
- * Replaces the placeholders (:name) in the given SQL command with
- * the values from the #GdaParameterList specified as the @params
- * argument.
- *
- * Returns: the SQL string with all placeholders replaced, or %NULL
- * on error. On success, the returned string must be freed by the caller
- * when no longer needed.
- */
-gchar *
-gda_sql_replace_placeholders (const gchar *sql, GdaParameterList *params)
-{
-	sql_statement *sql_stmt;
-
-	g_return_val_if_fail (sql != NULL, NULL);
-
-	/* parse the string */
-	sql_stmt = sql_parse (sql);
-	if (!sql_stmt) {
-		gda_log_error (_("Could not parse SQL command '%s'"), sql);
-		return NULL;
-	}
-
-	/* FIXME */
-
-	return NULL;
-}
-
 /**
  * gda_file_load
  * @filename: path for the file to be loaded.
@@ -310,10 +278,10 @@ gda_file_save (const gchar *filename, const gchar *buffer, gint len)
 }
 
 /**
- * utility_table_field_attrs_stringify
+ * gda_utility_table_field_attrs_stringify
  */
 gchar *
-utility_table_field_attrs_stringify (guint attributes)
+gda_utility_table_field_attrs_stringify (guint attributes)
 {
 	if (attributes & FIELD_AUTO_INCREMENT)
 		return g_strdup ("AUTO_INCREMENT");
@@ -322,10 +290,10 @@ utility_table_field_attrs_stringify (guint attributes)
 }
 
 /**
- * utility_table_field_attrs_parse
+ * gda_utility_table_field_attrs_parse
  */
 guint
-utility_table_field_attrs_parse (const gchar *str)
+gda_utility_table_field_attrs_parse (const gchar *str)
 {
 	if (!str)
 		return 0;
@@ -338,7 +306,7 @@ utility_table_field_attrs_parse (const gchar *str)
 
 
 /**
- * utility_build_encoded_id
+ * gda_utility_build_encoded_id
  *
  * Creates a BASE64 kind encoded string. It's not really a BASE64 because:
  * - the characters + and / of BASE64 are replaced with - and _
@@ -353,7 +321,7 @@ to_uchar (gchar ch)
 }
 
 gchar *
-utility_build_encoded_id (const gchar *prefix, const gchar *id)
+gda_utility_build_encoded_id (const gchar *prefix, const gchar *id)
 {
 	const gchar conv[64] = 
 		"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-_";
@@ -552,12 +520,12 @@ static const signed char b64[0x100] = {
 
 
 /**
- * utility_build_decoded_id
+ * gda_utility_build_decoded_id
  *
- * Reverse of utility_build_encoded_id()
+ * Reverse of gda_utility_build_encoded_id()
  */
 gchar *
-utility_build_decoded_id (const gchar *prefix, const gchar *id)
+gda_utility_build_decoded_id (const gchar *prefix, const gchar *id)
 {
 	gchar *str;
 	gchar *out;
@@ -609,7 +577,7 @@ utility_build_decoded_id (const gchar *prefix, const gchar *id)
 
 
 /**
- * utility_check_data_model
+ * gda_utility_check_data_model
  * @model: a #GdaDataModel object
  * @nbcols: the minimum requested number of columns
  * @Varargs: @nbcols arguments of type GType or -1 (if any data type is accepted)
@@ -619,7 +587,7 @@ utility_build_decoded_id (const gchar *prefix, const gchar *id)
  * Returns: TRUE if the data model's columns match the provided data types and number
  */
 gboolean
-utility_check_data_model (GdaDataModel *model, gint nbcols, ...)
+gda_utility_check_data_model (GdaDataModel *model, gint nbcols, ...)
 {
 	gboolean retval = TRUE;
 	gint i;
@@ -641,7 +609,7 @@ utility_check_data_model (GdaDataModel *model, gint nbcols, ...)
 		i = 0;
 		while ((i<nbcols) && retval) {
 			att = gda_data_model_describe_column (model, i);
-			mtype = gda_column_get_gda_type (att);
+			mtype = gda_column_get_g_type (att);
 			
 			argtype = va_arg (ap, GType);
 			if (argtype >= 0) {
@@ -650,7 +618,7 @@ utility_check_data_model (GdaDataModel *model, gint nbcols, ...)
 					retval = FALSE;
 #ifdef GDA_DEBUG
 					g_print ("Column %d: Expected %s, got %s\n",
-						 i, gda_type_to_string (rtype), gda_type_to_string (mtype));
+						 i, gda_g_type_to_string (rtype), gda_g_type_to_string (mtype));
 #endif
 				}
 			}
@@ -665,19 +633,23 @@ utility_check_data_model (GdaDataModel *model, gint nbcols, ...)
 }
 
 /**
- * utility_data_model_dump_data_to_xml
+ * gda_utility_data_model_dump_data_to_xml
  * @model: 
  * @cols: an array containing which columns of @model will be exported, or %NULL for all columns
  * @nb_cols: the number of columns in @cols
+ * @rows: an array containing which rows of @model will be exported, or %NULL for all rows
+ * @nb_rows: the number of rows in @rows
  * @name: name to use for the XML resulting table.
  *
  * Dump the data in a #GdaDataModel into a xmlNodePtr (as used in libxml).
  */
 void
-utility_data_model_dump_data_to_xml (GdaDataModel *model, xmlNodePtr parent, 
-				     const gint *cols, gint nb_cols, gboolean use_col_ids)
+gda_utility_data_model_dump_data_to_xml (GdaDataModel *model, xmlNodePtr parent, 
+				     const gint *cols, gint nb_cols, 
+				     const gint *rows, gint nb_rows,
+				     gboolean use_col_ids)
 {
-	gint rows, i;
+	gint nrows, i;
 	gint *rcols, rnb_cols;
 	gchar **col_ids = NULL;
 
@@ -710,19 +682,22 @@ utility_data_model_dump_data_to_xml (GdaDataModel *model, xmlNodePtr parent,
 	}
 
 	/* add the model data to the XML output */
-	rows = gda_data_model_get_n_rows (model);
-	if (rows > 0) {
+	if (!rows)
+		nrows = gda_data_model_get_n_rows (model);
+	else
+		nrows = nb_rows;
+	if (nrows > 0) {
 		xmlNodePtr row, data, field;
 		gint r, c;
 
 		data = xmlNewChild (parent, NULL, "gda_array_data", NULL);
-		for (r = 0; r < rows; r++) {
+		for (r = 0; r < nrows; r++) {
 			row = xmlNewChild (data, NULL, "gda_array_row", NULL);
 			for (c = 0; c < rnb_cols; c++) {
 				GValue *value;
 				gchar *str;
 
-				value = (GValue *) gda_data_model_get_value_at (model, rcols [c], r);
+				value = (GValue *) gda_data_model_get_value_at (model, rcols [c], rows ? rows [r] : r);
 				if (!value || gda_value_is_null ((GValue *) value))
 					str = NULL;
 				else {
@@ -754,7 +729,7 @@ utility_data_model_dump_data_to_xml (GdaDataModel *model, xmlNodePtr parent,
 }
 
 /**
- * utility_parameter_load_attributes
+ * gda_utility_parameter_load_attributes
  * @param:
  * @node: an xmlNodePtr with a &lt;parameter&gt; tag
  * @sources: a list of #GdaDataModel
@@ -762,7 +737,7 @@ utility_data_model_dump_data_to_xml (GdaDataModel *model, xmlNodePtr parent,
  * WARNING: may set the "source" custom string property 
  */
 void
-utility_parameter_load_attributes (GdaParameter *param, xmlNodePtr node, GSList *sources)
+gda_utility_parameter_load_attributes (GdaParameter *param, xmlNodePtr node, GSList *sources)
 {
 	xmlChar *str;
 	xmlNodePtr vnode;
@@ -849,7 +824,7 @@ utility_parameter_load_attributes (GdaParameter *param, xmlNodePtr node, GSList
 		const gchar *lang;
 		GType gdatype;
 
-		gdatype = gda_parameter_get_gda_type (param);
+		gdatype = gda_parameter_get_g_type (param);
 #ifdef HAVE_LC_MESSAGES
 		lang = setlocale (LC_MESSAGES, NULL);
 #else
@@ -910,7 +885,7 @@ utility_parameter_load_attributes (GdaParameter *param, xmlNodePtr node, GSList
  * @prov:
  * @cnc:
  * @dbms_type:
- * @gda_type:
+ * @g_type:
  * @created: a place to return if the data type has been created and is thus considered as a custom
  *           data type. Can't be %NULL!
  *
@@ -920,8 +895,8 @@ utility_parameter_load_attributes (GdaParameter *param, xmlNodePtr node, GSList
  * Returns: a #GdaDictType, or %NULL if it was not possible to find and create one
  */
 GdaDictType *
-utility_find_or_create_data_type (GdaDict *dict, GdaServerProvider *prov, GdaConnection *cnc, 
-				  const gchar *dbms_type, const gchar *gda_type, gboolean *created)
+gda_utility_find_or_create_data_type (GdaDict *dict, GdaServerProvider *prov, GdaConnection *cnc, 
+				  const gchar *dbms_type, const gchar *g_type, gboolean *created)
 {
 	GdaDictType *dtype = NULL;
 
@@ -935,10 +910,10 @@ utility_find_or_create_data_type (GdaDict *dict, GdaServerProvider *prov, GdaCon
 		dtype = gda_dict_get_dict_type_by_name (ASSERT_DICT (dict), dbms_type);
 
 	if (!dtype) {
-		if (gda_type) {
+		if (g_type) {
 			GType gtype;
 			
-			gtype = gda_type_from_string (gda_type);
+			gtype = gda_g_type_from_string (g_type);
 			if (prov) {
 				const gchar *deftype;
 				
@@ -950,8 +925,8 @@ utility_find_or_create_data_type (GdaDict *dict, GdaServerProvider *prov, GdaCon
 			if (!dtype) {
 				/* create a GdaDictType for that 'gda-type' */
 				dtype = GDA_DICT_TYPE (gda_dict_type_new (ASSERT_DICT (dict)));
-				gda_dict_type_set_sqlname (dtype, gda_type);
-				gda_dict_type_set_gda_type (dtype, gtype);
+				gda_dict_type_set_sqlname (dtype, g_type);
+				gda_dict_type_set_g_type (dtype, gtype);
 				gda_dict_declare_object (ASSERT_DICT (dict), (GdaObject *) dtype);
 				*created = TRUE;
 			}
