@@ -223,7 +223,7 @@ set_from_string (GValue *value, const gchar *as_string)
 	else if (type == GDA_TYPE_NULL)
 		gda_value_set_null (value);
 	else if (type == G_TYPE_ULONG)
-		g_value_set_ulong (value, g_type_from_string (as_string));
+		g_value_set_ulong (value, gda_g_type_from_string (as_string));
 
 	return retval;
 }
@@ -270,7 +270,7 @@ gda_binary_copy (gpointer boxed)
 	copy = g_new0 (GdaBinary, 1);
 	copy->data = g_memdup (src->data, src->binary_length);
 	copy->binary_length = src->binary_length;
-
+	
 	return copy;
 }
 
@@ -290,9 +290,6 @@ gda_binary_free (gpointer boxed)
 	g_free (binary->data);
 }
 
-
-
-
 /* 
  * Register the GdaGeometricPoint type in the GType system 
  */
@@ -759,11 +756,14 @@ string_to_timestamp (const GValue *src, GValue *dest)
 	timestamp->minute = atoi (as_string);
 	as_string += 3;
 	timestamp->second = atoi (as_string);
-	as_string += 3;
-	timestamp->fraction = atol (as_string) * 10; /* I have only hundredths of second */
-	as_string += 3;
-	timestamp->timezone = atol (as_string) * 60 * 60;
-
+	if (strlen(as_string)>=3) {
+		as_string += 3;
+		timestamp->fraction = atol (as_string) * 10; /* I have only hundredths of second */
+		if (strlen(as_string)>=3) {
+			as_string += 3;
+			timestamp->timezone = atol (as_string) * 60 * 60;
+		}
+	}
 	gda_value_set_timestamp (dest, timestamp);
 	g_free (timestamp);
 }
@@ -1465,7 +1465,8 @@ gda_value_set_from_value (GValue *value, const GValue *from)
  *
  * Converts a GValue to its string representation which is a human readable value. Note that the
  * returned string does not take into account the current locale of the user (on the contrary to the
- * #GdaDataHandler objects).
+ * #GdaDataHandler objects). Using this function should be limited to debugging and values serialization
+ * purposes.
  *
  * Dates are converted in a YYYY-MM-DD format.
  *
@@ -1552,8 +1553,24 @@ gda_value_compare (const GValue *value1, const GValue *value2)
 		/* FIXME: Does compare() make sense with BLOBs? Yes, use the same way of Binary comparison */
 		return 0;
 
-	else if (type == G_TYPE_DATE)
-		return g_date_compare ((GDate *) g_value_get_boxed (value1), (GDate *) g_value_get_boxed (value2));
+	else if (type == G_TYPE_DATE) {
+		GDate *d1, *d2;
+
+		d1 = (GDate *) g_value_get_boxed (value1);
+		d2 = (GDate *) g_value_get_boxed (value2);
+		if (d1 && d2)
+			return g_date_compare (d1, d2);
+		else {
+			if (d1)
+				return 1;
+			else {
+				if (d2)
+					return -1;
+				else
+					return 0;
+			}
+		}
+	}
 
 	else if (type == G_TYPE_DOUBLE) {
 		gdouble v1, v2;
@@ -1766,7 +1783,7 @@ string_to_short(const GValue *src, GValue *dest)
 	
 	const gchar *as_string;
 	long int lvalue;
-	gchar *endptr[1];
+	gchar *endptr;
 
 	g_return_if_fail (G_VALUE_HOLDS_STRING (src) &&
 			  (GDA_VALUE_HOLDS_SHORT (dest) || GDA_VALUE_HOLDS_USHORT (dest)));
@@ -1775,7 +1792,7 @@ string_to_short(const GValue *src, GValue *dest)
 	
 	lvalue = strtol (as_string, &endptr, 10);
 	
-	if (*as_string != '\0' && **endptr == '\0') {
+	if (*as_string != '\0' && *endptr == '\0') {
 		if (GDA_VALUE_HOLDS_SHORT (dest))
 			gda_value_set_short (dest, (gshort) lvalue);
 		else
