@@ -1,5 +1,5 @@
 /* GDA FireBird Provider
- * Copyright (C) 1998 - 2005 The GNOME Foundation
+ * Copyright (C) 1998 - 2006 The GNOME Foundation
  *
  * AUTHORS:
  *         Albi Jeronimo <jeronimoalbi@yahoo.com.ar>
@@ -78,7 +78,7 @@ static GObjectClass *parent_class = NULL;
 /*******************************/
 
 static GType
-fb_sql_type_to_gda_type (const gshort field_type, gboolean has_decimals)
+fb_sql_type_to_g_type (const gshort field_type, gboolean has_decimals)
 {
 	switch (field_type && ~1) {
 		case SQL_BLOB:
@@ -432,7 +432,11 @@ fb_gda_value_fill (GValue *gda_value,
 	GdaBlob *blob = NULL;
 	struct tm *a_date_time;
 	gpointer tmp_big, field_data;
+#if FB_API_VER < 20
 	struct vary *var_text;
+#else
+	PARAMVARY *var_text;
+#endif
 	gdouble i64_n;
 	XSQLDA *sql_result;
 	
@@ -509,7 +513,11 @@ fb_gda_value_fill (GValue *gda_value,
 			break;
 		case SQL_VARYING:
 		case SQL_VARYING+1:
+#if FB_API_VER < 20
 			var_text = (struct vary *) field_data;
+#else
+			var_text = (PARAMVARY *) field_data;
+#endif
 			var_text->vary_string[var_text->vary_length] = '\0';
 
 			g_value_set_string (gda_value, (const gchar *) var_text->vary_string);
@@ -853,7 +861,7 @@ gda_firebird_recordset_describe_column (GdaDataModel *model, gint col)
 	gda_column_set_defined_size (fa, (glong) recset->priv->sql_result->sqlvar[col].sqllen);
 	gda_column_set_table (fa, (const gchar *) recset->priv->sql_result->sqlvar[col].relname);
 	gda_column_set_scale (fa, (glong) (recset->priv->sql_result->sqlvar[col].sqlscale * -1));
-	gda_column_set_gda_type (fa, fb_sql_type_to_gda_type (
+	gda_column_set_g_type (fa, fb_sql_type_to_g_type (
 							recset->priv->sql_result->sqlvar[col].sqltype,
 							(recset->priv->sql_result->sqlvar[col].sqlscale < 0)));
 	gda_column_set_position (fa, col);
