@@ -1,9 +1,10 @@
 /* GDA IBMDB2 Provider
- * Copyright (C) 2002 The GNOME Foundation
+ * Copyright (C) 2002 - 2006 The GNOME Foundation
  *
  * AUTHORS:
  *         Holger Thon <holger.thon@gnome-db-org>
  *	   Sergey N. Belinsky <sergey_be@mail.ru>
+ *         Vivien Malerba <malerba@gnome-db.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -21,12 +22,15 @@
  */
 
 #include <libgda/gda-data-model-array.h>
-#include <libgda/gda-intl.h>
+#include <libgda/gda-data-model-private.h>
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
 #include <sqlcli1.h>
 #include "gda-ibmdb2.h"
 #include "gda-ibmdb2-recordset.h"
 
+#include <libgda/sql-delimiter/gda-sql-delimiter.h>
+
 #define PARENT_TYPE GDA_TYPE_SERVER_PROVIDER
 
 #define OBJECT_DATA_IBMDB2_HANDLE "GDA_IBMDB2_IBMDB2Handle"
@@ -40,7 +44,7 @@ static GObjectClass *parent_class = NULL;
 static void gda_ibmdb2_provider_class_init (GdaIBMDB2ProviderClass *klass);
 static void gda_ibmdb2_provider_init       (GdaIBMDB2Provider      *provider,
                                             GdaIBMDB2ProviderClass *klass);
-static void gda_ibmdb2_provider_finalize   (GObject                 *object);
+static void gda_ibmdb2_provider_finalize   (GObject *object);
 
 static gboolean gda_ibmdb2_provider_open_connection (GdaServerProvider *provider,
                                                      GdaConnection *cnc,
@@ -54,25 +58,20 @@ static const gchar *gda_ibmdb2_provider_get_database (GdaServerProvider *provide
 static gboolean gda_ibmdb2_provider_change_database (GdaServerProvider *provider,
                                                      GdaConnection *cnc,
                                                      const gchar *name);
-static gboolean gda_ibmdb2_provider_create_database_cnc (GdaServerProvider *provider,
-                                                         GdaConnection *cnc,
-                                                         const gchar *name);
-static gboolean gda_ibmdb2_provider_drop_database_cnc (GdaServerProvider *provider,
-						       GdaConnection *cnc,
-						       const gchar *name);
 static GList *gda_ibmdb2_provider_execute_command (GdaServerProvider *provider,
                                                    GdaConnection *cnc,
                                                    GdaCommand *cmd,
                                                    GdaParameterList *params);
 static gboolean gda_ibmdb2_provider_begin_transaction (GdaServerProvider *provider,
                                                        GdaConnection *cnc,
-                                                       GdaTransaction *xaction);
+                                                       const gchar *name, GdaTransactionIsolation level,
+							GError **error);
 static gboolean gda_ibmdb2_provider_commit_transaction (GdaServerProvider *provider,
                                                         GdaConnection *cnc,
-                                                        GdaTransaction *xaction);
+                                                        const gchar *name, GError **error);
 static gboolean gda_ibmdb2_provider_rollback_transaction (GdaServerProvider *provider,
                                                           GdaConnection *cnc,
-                                                          GdaTransaction *xaction);
+                                                          const gchar *name, GError **error);
 static gboolean gda_ibmdb2_provider_supports (GdaServerProvider *provider,
                                               GdaConnection *cnc,
                                               GdaConnectionFeature feature);
@@ -81,7 +80,7 @@ static GdaDataModel *gda_ibmdb2_provider_get_schema (GdaServerProvider *provider
                                                      GdaConnectionSchema schema,
                                                      GdaParameterList *params);
 
-static const gboolean gda_ibmdb2_provider_update_database (GdaIBMDB2ConnectionData *conn_data);
+static gboolean gda_ibmdb2_provider_update_database (GdaIBMDB2ConnectionData *conn_data);
 
 static const gchar *gda_ibmdb2_provider_get_version (GdaServerProvider *provider);
 static const gchar *gda_ibmdb2_provider_get_server_version (GdaServerProvider *provider,
@@ -89,8 +88,97 @@ static const gchar *gda_ibmdb2_provider_get_server_version (GdaServerProvider *p
 static gboolean gda_ibmdb2_provider_single_command (GdaIBMDB2ConnectionData *conn_data,
 						    GdaConnection *cnc,
 						    const gchar *command);
-									    
-							   
+		
+/**********************************************************************/
+#define CHECK_HANDLE( htype, hndl, rc ) if ( rc != SQL_SUCCESS )	\
+		{check_error(htype,hndl,rc,__LINE__,__FILE__);}
+
+
+/******************************************************************/
+/* print_error                                                    */
+/* calls SQLGetDiagRec() displays SQLSTATE and message            */
+/******************************************************************/
+void print_error( SQLSMALLINT htype, /* A handle type */
+		  SQLHANDLE   hndl,  /* A handle */
+		  SQLRETURN   frc,   /* Return code */
+		  int         line,  /* error from line */
+		  char *      file   /* error from file */)
+{
+	SQLCHAR     buffer[SQL_MAX_MESSAGE_LENGTH + 1] ;
+	SQLCHAR     sqlstate[SQL_SQLSTATE_SIZE + 1] ;
+	SQLINTEGER  sqlcode ;
+	SQLSMALLINT length, i ;
+	SQLRETURN   prc;
+
+	printf("rc=%d reported from file:%s, line:%d\n",
+	       frc,
+	       file,
+	       line
+	       ) ;
+	i = 1 ;
+	while ( SQLGetDiagRec( htype,
+			       hndl,
+			       i,
+			       sqlstate,
+			       &sqlcode,
+			       buffer,
+			       SQL_MAX_MESSAGE_LENGTH + 1,
+			       &length
+			       ) == SQL_SUCCESS ) {
+		printf( "SQLSTATE: %s\n", sqlstate ) ;
+		printf( "Native Error Code: %ld\n", sqlcode ) ;
+		printf( "buffer: %s \n", buffer ) ;
+		i++ ;
+	}
+}
+
+/******************************************************************/
+/* check_error                                                    */
+/******************************************************************/
+/*   RETCODE values   from sqlcli.h                               */
+/*#define  SQL_SUCCESS             0                              */
+/*#define  SQL_SUCCESS_WITH_INFO   1                              */
+/*#define  SQL_NO_DATA_FOUND       100                            */
+/*#define  SQL_NEED_DATA           99                             */
+/*#define  SQL_NO_DATA             SQL_NO_DATA_FOUND              */
+/*#define  SQL_STILL_EXECUTING     2       not currently returned */
+/*#define  SQL_ERROR               -1                             */
+/*#define  SQL_INVALID_HANDLE      -2                             */
+/******************************************************************/
+void check_error( SQLSMALLINT htype, /* A handle type */
+		  SQLHANDLE   hndl,  /* A handle */
+		  SQLRETURN   frc,   /* Return code */
+		  int         line,  /* Line error issued */
+		  char *      file   /* file error issued */)
+ {
+	SQLCHAR         cli_sqlstate[SQL_SQLSTATE_SIZE + 1];
+	SQLINTEGER      cli_sqlcode;
+	SQLSMALLINT     length;
+
+	switch (frc) {
+	case SQL_SUCCESS:
+		break;
+	case SQL_INVALID_HANDLE:
+		printf("check_error> SQL_INVALID HANDLE \n");
+		break;
+	case SQL_ERROR:
+		printf("check_error> SQL_ERROR\n");
+		break;
+	case SQL_SUCCESS_WITH_INFO:
+		printf("check_error>  SQL_SUCCESS_WITH_INFO\n");
+		break;
+	case SQL_NO_DATA_FOUND:
+		printf("check_error> SQL_NO_DATA_FOUND\n");
+		break;
+	default:
+		printf("check_error> Received rc from api rc=%i\n",frc);
+		break;
+	}
+
+	print_error(htype,hndl,frc,line,file);
+}	    
+/********************************************************************************/
+
 static gboolean
 gda_ibmdb2_provider_open_connection (GdaServerProvider *provider,
                                      GdaConnection *cnc,
@@ -134,7 +222,7 @@ gda_ibmdb2_provider_open_connection (GdaServerProvider *provider,
 		gda_connection_add_event_string (cnc, _("You must at least provide host, user and password to connect."));
 		return FALSE;
 	}*/
-	
+
 	conn_data = g_new0 (GdaIBMDB2ConnectionData, 1);
 	g_return_val_if_fail (conn_data != NULL, FALSE);
 	conn_data->henv = SQL_NULL_HANDLE;
@@ -143,9 +231,9 @@ gda_ibmdb2_provider_open_connection (GdaServerProvider *provider,
 	
 	conn_data->rc = SQLAllocHandle (SQL_HANDLE_ENV, SQL_NULL_HANDLE, &conn_data->henv);
 	if (conn_data->rc != SQL_SUCCESS) {
-		if (conn_data->henv != SQL_NULL_HANDLE) {
+		if (conn_data->henv != SQL_NULL_HANDLE) 
 			SQLFreeHandle (SQL_HANDLE_ENV, conn_data->henv);
-		}
+
 		g_free (conn_data);
 		conn_data = NULL;
 		
@@ -153,18 +241,22 @@ gda_ibmdb2_provider_open_connection (GdaServerProvider *provider,
 		return FALSE;
 	}
 	conn_data->rc = SQLAllocHandle (SQL_HANDLE_DBC, conn_data->henv, &conn_data->hdbc);
-	if (conn_data->rc != SQL_SUCCESS) {
-		if (conn_data->hdbc != SQL_NULL_HANDLE) {
+	CHECK_HANDLE( SQL_HANDLE_DBC, conn_data->hdbc, conn_data->rc );
+	if ((conn_data->rc != SQL_SUCCESS) && (conn_data->rc != SQL_SUCCESS_WITH_INFO)){
+		if (conn_data->hdbc != SQL_NULL_HANDLE) 
 			conn_data->rc = SQLFreeHandle (SQL_HANDLE_DBC, conn_data->hdbc);
-		}
+
 		conn_data->rc = SQLFreeHandle (SQL_HANDLE_ENV, conn_data->henv);
+		gda_ibmdb2_emit_error (cnc, conn_data->henv, SQL_NULL_HANDLE, SQL_NULL_HANDLE);
+
 		g_free (conn_data);
 		conn_data = NULL;
 		
-		gda_ibmdb2_emit_error (cnc, conn_data->henv, SQL_NULL_HANDLE, SQL_NULL_HANDLE);
-		
 		return FALSE;
 	}
+	if (conn_data->rc != SQL_SUCCESS_WITH_INFO) {
+		g_print ("With info...\n");
+	}
 	/* IBM DB2 use autocommit for all DML commands by default.
 	   See gda_ibmdb2_provider_begin_transaction and gda_ibmdb2_provider_begin_transaction
 	 
@@ -291,8 +383,8 @@ gda_ibmdb2_provider_close_connection (GdaServerProvider *provider,
 	return retval;
 }
 
-static const gchar
-*gda_ibmdb2_provider_get_database (GdaServerProvider *provider,
+static const gchar *
+gda_ibmdb2_provider_get_database (GdaServerProvider *provider,
                                    GdaConnection *cnc)
 {
 	GdaIBMDB2Provider *db2_prov = (GdaIBMDB2Provider *) provider;
@@ -362,56 +454,7 @@ gda_ibmdb2_provider_single_command (GdaIBMDB2ConnectionData *conn_data,
 	return TRUE;
 }
 
-
-static gboolean
-gda_ibmdb2_provider_create_database_cnc (GdaServerProvider *provider,                                                    
-				         GdaConnection *cnc,
-                                         const gchar *name)
-{
-	GdaIBMDB2Provider *db2_prov = (GdaIBMDB2Provider *) provider;
-	GdaIBMDB2ConnectionData *conn_data = NULL;
-	gchar *sql;
-	gboolean retval;
-
-	g_return_val_if_fail (GDA_IS_IBMDB2_PROVIDER (db2_prov), FALSE);
-	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	
-        conn_data = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_IBMDB2_HANDLE);
-        g_return_val_if_fail (conn_data != NULL, FALSE);
-	
-	sql = g_strdup_printf("CREATE DATABASE %s", name);
-	retval = gda_ibmdb2_provider_single_command (conn_data, cnc, sql);	
-	g_free(sql);
-	
-	return retval;
-}
-
-
-
-static gboolean
-gda_ibmdb2_provider_drop_database_cnc (GdaServerProvider *provider,
-				       GdaConnection *cnc,
-				       const gchar *name)
-{
-	GdaIBMDB2Provider *db2_prov = (GdaIBMDB2Provider *) provider;
-	GdaIBMDB2ConnectionData *conn_data = NULL;
-	gchar *sql;
-	gboolean retval;
-
-	g_return_val_if_fail (GDA_IS_IBMDB2_PROVIDER (db2_prov), FALSE);
-	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	
-        conn_data = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_IBMDB2_HANDLE);
-        g_return_val_if_fail (conn_data != NULL, FALSE);
-	
-	sql = g_strdup_printf("DROP DATABASE %s", name);
-	retval = gda_ibmdb2_provider_single_command (conn_data, cnc, sql);	
-	g_free(sql);
-	
-	return retval;
-}
-
-static const gboolean 
+static gboolean 
 gda_ibmdb2_provider_update_database (GdaIBMDB2ConnectionData *conn_data)
 {
 	SQLRETURN   rc = SQL_SUCCESS;
@@ -438,7 +481,8 @@ gda_ibmdb2_provider_update_database (GdaIBMDB2ConnectionData *conn_data)
 }
 
 static GList *
-process_sql_commands (GList *reclist, GdaConnection *cnc, const gchar *sql, GdaCommandOptions options)
+process_sql_commands (GList *reclist, GdaConnection *cnc, const gchar *sql, 
+		      GdaCommandOptions options)
 {
 	GdaIBMDB2ConnectionData *conn_data = NULL;
         gchar **arr;
@@ -451,7 +495,7 @@ process_sql_commands (GList *reclist, GdaConnection *cnc, const gchar *sql, GdaC
 		return NULL;
 	}
 	
-	arr = g_strsplit (sql, ";", 0);
+	arr = gda_delimiter_split_sql (sql);
 	
         if (arr) {
  		gint n = 0;     
@@ -460,9 +504,11 @@ process_sql_commands (GList *reclist, GdaConnection *cnc, const gchar *sql, GdaC
 		
 			g_assert(conn_data->hstmt == SQL_NULL_HANDLE);
 			
-			conn_data->rc = SQLAllocHandle (SQL_HANDLE_STMT, conn_data->hdbc, &conn_data->hstmt);
+			conn_data->rc = SQLAllocHandle (SQL_HANDLE_STMT, 
+							conn_data->hdbc, &conn_data->hstmt);
 			if (conn_data->rc != SQL_SUCCESS) {
-				gda_ibmdb2_emit_error(cnc, conn_data->henv, conn_data->hdbc, SQL_NULL_HANDLE);
+				gda_ibmdb2_emit_error(cnc, conn_data->henv, 
+						      conn_data->hdbc, SQL_NULL_HANDLE);
 				conn_data->hstmt = SQL_NULL_HANDLE;
 				
 				g_list_foreach (reclist, (GFunc) g_object_unref, NULL);
@@ -476,11 +522,13 @@ process_sql_commands (GList *reclist, GdaConnection *cnc, const gchar *sql, GdaC
 					
 			conn_data->rc = SQLExecDirect(conn_data->hstmt,(SQLCHAR*)arr[n], SQL_NTS);
 			if (conn_data->rc != SQL_SUCCESS) {
-				gda_ibmdb2_emit_error(cnc, conn_data->henv, conn_data->hdbc, conn_data->hstmt);
+				gda_ibmdb2_emit_error (cnc, conn_data->henv, 
+						       conn_data->hdbc, conn_data->hstmt);
 				
 				conn_data->rc = SQLFreeHandle(SQL_HANDLE_STMT, conn_data->hstmt);
 				if (conn_data->rc != SQL_SUCCESS) {
-					gda_ibmdb2_emit_error(cnc, conn_data->henv, conn_data->hdbc, SQL_NULL_HANDLE);
+					gda_ibmdb2_emit_error (cnc, conn_data->henv, 
+							       conn_data->hdbc, SQL_NULL_HANDLE);
 				}
 				conn_data->hstmt = SQL_NULL_HANDLE;
 				
@@ -488,7 +536,7 @@ process_sql_commands (GList *reclist, GdaConnection *cnc, const gchar *sql, GdaC
 				g_list_free (reclist);
 
 				if (options == GDA_COMMAND_OPTION_STOP_ON_ERRORS) {
-					g_strfreev(arr);
+					g_strfreev (arr);
 					return NULL;
 				}
 			}
@@ -496,8 +544,9 @@ process_sql_commands (GList *reclist, GdaConnection *cnc, const gchar *sql, GdaC
 			recset = gda_ibmdb2_recordset_new (cnc, conn_data->hstmt);
 
                     	if (GDA_IS_IBMDB2_RECORDSET (recset)) {
-				gda_data_model_set_command_text (recset, arr[n]);
-				gda_data_model_set_command_type (recset, GDA_COMMAND_TYPE_SQL);
+				g_object_set (G_OBJECT (recset), 
+					      "command_text", arr[n],
+					      "command_type", GDA_COMMAND_TYPE_SQL, NULL);
                                 reclist = g_list_append (reclist, recset);
                     	} else {
 				g_list_foreach (reclist, (GFunc) g_object_unref, NULL);
@@ -506,10 +555,11 @@ process_sql_commands (GList *reclist, GdaConnection *cnc, const gchar *sql, GdaC
 			
 			conn_data->rc = SQLFreeHandle(SQL_HANDLE_STMT, conn_data->hstmt);
 			if (conn_data->rc != SQL_SUCCESS) {
-				gda_ibmdb2_emit_error(cnc, conn_data->henv, conn_data->hdbc, SQL_NULL_HANDLE);
+				gda_ibmdb2_emit_error (cnc, conn_data->henv, 
+						       conn_data->hdbc, SQL_NULL_HANDLE);
 				
 				if (options == GDA_COMMAND_OPTION_STOP_ON_ERRORS) {
-					g_strfreev(arr);
+					g_strfreev (arr);
 					return NULL;
 				}
 			}
@@ -523,11 +573,11 @@ process_sql_commands (GList *reclist, GdaConnection *cnc, const gchar *sql, GdaC
 	return reclist;
 }
 				
-static GList
-*gda_ibmdb2_provider_execute_command (GdaServerProvider *provider,
-                                      GdaConnection *cnc,
-                                      GdaCommand *cmd,
-                                      GdaParameterList *params)
+static GList *
+gda_ibmdb2_provider_execute_command (GdaServerProvider *provider,
+				     GdaConnection *cnc,
+				     GdaCommand *cmd,
+				     GdaParameterList *params)
 {
 	GdaIBMDB2Provider *db2_prov = (GdaIBMDB2Provider *) provider;
 	GdaCommandOptions options;
@@ -546,12 +596,11 @@ static GList
 	    case GDA_COMMAND_TYPE_TABLE:
 		str = g_strdup_printf ("SELECT * FROM %s", gda_command_get_text (cmd));
                 reclist = process_sql_commands (reclist, cnc, str, options);
-                if (reclist && GDA_IS_DATA_MODEL (reclist->data)) {
-			gda_data_model_set_command_text (GDA_DATA_MODEL (reclist->data),
-                                                         gda_command_get_text (cmd));
-                        gda_data_model_set_command_type (GDA_DATA_MODEL (reclist->data),
-                                                         GDA_COMMAND_TYPE_TABLE);
-		}
+                if (reclist && GDA_IS_DATA_MODEL (reclist->data))
+			g_object_set (G_OBJECT (reclist->data), 
+				      "command_text", gda_command_get_text (cmd),
+				      "command_type", GDA_COMMAND_TYPE_TABLE, NULL);
+
 		g_free(str);
 		break;
             case GDA_COMMAND_TYPE_XML:
@@ -567,7 +616,8 @@ static GList
 static gboolean
 gda_ibmdb2_provider_begin_transaction (GdaServerProvider *provider,
                                        GdaConnection *cnc,
-                                       GdaTransaction *xaction)
+                                       const gchar *name, GdaTransactionIsolation level,
+				       GError **error)
 {
 	GdaIBMDB2Provider *db2_prov = (GdaIBMDB2Provider *) provider;
 	GdaIBMDB2ConnectionData *conn_data = NULL;
@@ -597,7 +647,7 @@ gda_ibmdb2_provider_begin_transaction (GdaServerProvider *provider,
 static gboolean
 gda_ibmdb2_provider_commit_transaction (GdaServerProvider *provider,
                                         GdaConnection *cnc,
-                                        GdaTransaction *xaction)
+                                        const gchar *name, GError **error)
 {
 	GdaIBMDB2Provider *db2_prov = (GdaIBMDB2Provider *) provider;
 	GdaIBMDB2ConnectionData *conn_data = NULL;
@@ -631,7 +681,7 @@ gda_ibmdb2_provider_commit_transaction (GdaServerProvider *provider,
 static gboolean
 gda_ibmdb2_provider_rollback_transaction (GdaServerProvider *provider,
                                           GdaConnection *cnc,
-                                          GdaTransaction *xaction)
+                                          const gchar *name, GError **error)
 {
 	GdaIBMDB2Provider *db2_prov = (GdaIBMDB2Provider *) provider;
 	GdaIBMDB2ConnectionData *conn_data = NULL;
@@ -719,8 +769,8 @@ gda_ibmdb2_provider_supports (GdaServerProvider *provider,
 }
 
 					
-static GdaDataModel
-*get_db2_aggregates(GdaConnection *cnc, GdaParameterList *params)
+static GdaDataModel *
+get_db2_aggregates(GdaConnection *cnc, GdaParameterList *params)
 {
 	GdaDataModel *recset;
 	GList *reclist;
@@ -744,8 +794,8 @@ static GdaDataModel
 	return recset;
 }
 
-static GdaDataModel
-*get_db2_databases(GdaConnection *cnc, GdaParameterList *params)
+static GdaDataModel *
+get_db2_databases(GdaConnection *cnc, GdaParameterList *params)
 {
 	GdaIBMDB2ConnectionData *conn_data = NULL;
 	SQLCHAR alias[SQL_MAX_DSN_LENGTH + 1];
@@ -776,13 +826,13 @@ static GdaDataModel
 	recset = GDA_DATA_MODEL_ARRAY (gda_data_model_array_new (1));
 	gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 0, _("Database"));
 
-	for (i = 0; conn_data->rc != SQL_NO_DATA_FOUND; i++)
-	{
-	    
+	for (i = 0; conn_data->rc != SQL_NO_DATA_FOUND; i++) {
+		GValue *tmpval;
 		str = g_strndup(alias, alias_len);
 	    
-	    	value_list = g_list_append (NULL, gda_value_new_string (str));
-	    	gda_data_model_append_values (GDA_DATA_MODEL (recset), value_list);
+		g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), str);
+	    	value_list = g_list_append (NULL, tmpval);
+	    	gda_data_model_append_values (GDA_DATA_MODEL (recset), value_list, NULL);
 	   
 	    	g_free(str);
 		g_list_foreach (value_list, (GFunc) gda_value_free, NULL);
@@ -802,14 +852,14 @@ static GdaDataModel
 	return GDA_DATA_MODEL (recset);
 }
 
-static GdaDataModel
-*get_db2_fields(GdaConnection *cnc, GdaParameterList *params)
+static GdaDataModel *
+get_db2_fields(GdaConnection *cnc, GdaParameterList *params)
 {
 	return NULL;
 }
 
-static GdaDataModel
-*get_db2_indexes(GdaConnection *cnc, GdaParameterList *params)
+static GdaDataModel *
+get_db2_indexes(GdaConnection *cnc, GdaParameterList *params)
 {
 	GdaDataModel *recset;
 	GList *reclist;
@@ -835,8 +885,8 @@ static GdaDataModel
 	return recset;
 }
 
-static GdaDataModel
-*get_db2_procedures(GdaConnection *cnc, GdaParameterList *params)
+static GdaDataModel *
+get_db2_procedures(GdaConnection *cnc, GdaParameterList *params)
 {
 	GdaDataModel *recset;
 	GList *reclist;
@@ -866,8 +916,8 @@ static GdaDataModel
 }
 
 
-static GdaDataModel
-*get_db2_tables(GdaConnection *cnc, GdaParameterList *params)
+static GdaDataModel *
+get_db2_tables(GdaConnection *cnc, GdaParameterList *params)
 {
 	GdaDataModel *recset;
 	GList *reclist;
@@ -895,8 +945,8 @@ static GdaDataModel
 	return recset;
 }
 
-static GdaDataModel
-*get_db2_triggers(GdaConnection *cnc, GdaParameterList *params)
+static GdaDataModel *
+get_db2_triggers(GdaConnection *cnc, GdaParameterList *params)
 {
 	GdaDataModel *recset;
 	GList *reclist;
@@ -923,8 +973,8 @@ static GdaDataModel
 	return recset;
 }
 
-static GdaDataModel
-*get_db2_types(GdaConnection *cnc, GdaParameterList *params)
+static GdaDataModel *
+get_db2_types(GdaConnection *cnc, GdaParameterList *params)
 {
         GdaDataModel *recset;
 	GList *reclist;
@@ -976,8 +1026,8 @@ get_db2_users (GdaConnection *cnc, GdaParameterList *params)
         return recset;
 }
 
-static GdaDataModel
-*get_db2_views(GdaConnection *cnc, GdaParameterList *params)
+static GdaDataModel *
+get_db2_views(GdaConnection *cnc, GdaParameterList *params)
 {
 	GdaDataModel *recset;
 	GList *reclist;
@@ -1005,11 +1055,11 @@ static GdaDataModel
 }
 
 
-static GdaDataModel
-*gda_ibmdb2_provider_get_schema (GdaServerProvider *provider,
-				 GdaConnection *cnc,
-                                 GdaConnectionSchema schema,
-                                 GdaParameterList *params)
+static GdaDataModel *
+gda_ibmdb2_provider_get_schema (GdaServerProvider *provider,
+				GdaConnection *cnc,
+				GdaConnectionSchema schema,
+				GdaParameterList *params)
 {
 	GdaIBMDB2Provider *db2_prov = (GdaIBMDB2Provider *) provider;
 	
@@ -1055,20 +1105,40 @@ gda_ibmdb2_provider_class_init (GdaIBMDB2ProviderClass *klass)
 	parent_class = g_type_class_peek_parent (klass);
 
 	object_class->finalize = gda_ibmdb2_provider_finalize;
+
 	provider_class->get_version = gda_ibmdb2_provider_get_version;
+	provider_class->get_server_version = gda_ibmdb2_provider_get_server_version;
+	provider_class->get_info = NULL;
+	provider_class->supports_feature = gda_ibmdb2_provider_supports;
+	provider_class->get_schema = gda_ibmdb2_provider_get_schema;
+
+	provider_class->get_data_handler = NULL;
+	provider_class->string_to_value = NULL;
+	provider_class->get_def_dbms_type = NULL;
+
 	provider_class->open_connection = gda_ibmdb2_provider_open_connection;
 	provider_class->close_connection = gda_ibmdb2_provider_close_connection;
-	provider_class->get_server_version = gda_ibmdb2_provider_get_server_version;
 	provider_class->get_database = gda_ibmdb2_provider_get_database;
 	provider_class->change_database = gda_ibmdb2_provider_change_database;
-	provider_class->create_database_cnc = gda_ibmdb2_provider_create_database_cnc;
-	provider_class->drop_database_cnc = gda_ibmdb2_provider_drop_database_cnc;
+
+	provider_class->supports_operation = NULL;
+        provider_class->create_operation = NULL;
+        provider_class->render_operation = NULL;
+        provider_class->perform_operation = NULL;
+
 	provider_class->execute_command = gda_ibmdb2_provider_execute_command;
+	provider_class->get_last_insert_id = NULL;
+
 	provider_class->begin_transaction = gda_ibmdb2_provider_begin_transaction;
 	provider_class->commit_transaction = gda_ibmdb2_provider_commit_transaction;
 	provider_class->rollback_transaction = gda_ibmdb2_provider_rollback_transaction;
-	provider_class->supports = gda_ibmdb2_provider_supports;
-	provider_class->get_schema = gda_ibmdb2_provider_get_schema;
+	provider_class->add_savepoint = NULL;
+	provider_class->rollback_savepoint = NULL;
+	provider_class->delete_savepoint = NULL;
+	
+	provider_class->create_blob = NULL;
+	provider_class->fetch_blob = NULL;
+
 }
 
 static void
