@@ -46,6 +46,7 @@
 #include <libgda/handlers/gda-handler-type.h>
 
 #include <libgda/sql-delimiter/gda-sql-delimiter.h>
+#include <libgda/gda-connection-private.h>
 
 #define PARENT_TYPE GDA_TYPE_SERVER_PROVIDER
 
@@ -97,15 +98,16 @@ static gchar *gda_mysql_provider_get_last_insert_id (GdaServerProvider *provider
 
 static gboolean gda_mysql_provider_begin_transaction (GdaServerProvider *provider,
 						      GdaConnection *cnc,
-						      GdaTransaction *xaction);
+						      const gchar *name, GdaTransactionIsolation level,
+						      GError **error);
 
 static gboolean gda_mysql_provider_commit_transaction (GdaServerProvider *provider,
 						       GdaConnection *cnc,
-						       GdaTransaction *xaction);
+						       const gchar *name, GError **error);
 
 static gboolean gda_mysql_provider_rollback_transaction (GdaServerProvider *provider,
 							 GdaConnection *cnc,
-							 GdaTransaction *xaction);
+							 const gchar *name, GError **error);
 
 static gboolean gda_mysql_provider_supports (GdaServerProvider *provider,
 					     GdaConnection *cnc,
@@ -169,6 +171,9 @@ gda_mysql_provider_class_init (GdaMysqlProviderClass *klass)
 	provider_class->begin_transaction = gda_mysql_provider_begin_transaction;
 	provider_class->commit_transaction = gda_mysql_provider_commit_transaction;
 	provider_class->rollback_transaction = gda_mysql_provider_rollback_transaction;
+	provider_class->add_savepoint = NULL;
+	provider_class->rollback_savepoint = NULL;
+	provider_class->delete_savepoint = NULL;
 	
 	provider_class->create_blob = NULL;
 	provider_class->fetch_blob = NULL;
@@ -291,6 +296,12 @@ real_open_connection (const gchar *host, gint port, const gchar *socket,
 		return NULL;
 	}
 #endif
+
+#if MYSQL_VERSION_ID >= 50007
+	if (mysql_set_character_set (mysql, "utf8"))
+		g_warning ("Could not set client character set to UTF8 (using %s), expect problems with non UTF-8 characters\n", 
+			   mysql_character_set_name (mysql));
+#endif
 	
 	return mysql;
 }
@@ -428,12 +439,14 @@ process_sql_commands (GList *reclist, GdaConnection *cnc, const gchar *sql)
 	arr = gda_delimiter_split_sql (sql);
 	if (arr) {
 		gint n = 0;
+		gboolean allok = TRUE;
 
-		while (arr[n]) {
+		while (arr[n] && allok) {
 			gint rc;
 			MYSQL_RES *mysql_res;
 			GdaMysqlRecordset *recset;
 			gchar *tststr;
+			GdaConnectionEvent *error = NULL;
 
 			/* if the connection is in read-only mode, just allow SELECT,
 			   SHOW commands */
@@ -447,6 +460,7 @@ process_sql_commands (GList *reclist, GdaConnection *cnc, const gchar *sql)
 					gda_connection_add_event_string (
 						cnc, "Command '%s' cannot be executed in read-only mode", arr[n]);
 					reclist = g_list_append (reclist, NULL);
+					allok = FALSE;
 					break;
 				}
 
@@ -456,63 +470,66 @@ process_sql_commands (GList *reclist, GdaConnection *cnc, const gchar *sql)
 			/* execute the command on the server */
 			rc = mysql_real_query (mysql, arr[n], strlen (arr[n]));
 			if (rc != 0) {
-				gda_connection_add_event (cnc, gda_mysql_make_error (mysql));
+				error = gda_mysql_make_error (mysql);
+				gda_connection_add_event (cnc, error);
 				reclist = g_list_append (reclist, NULL);
-				break;
-			}
-
-			/* command was executed OK */
-			g_strchug (arr[n]);
-			tststr = arr[n];
-			if (! g_ascii_strncasecmp (tststr, "SELECT", 6) ||
-			    ! g_ascii_strncasecmp (tststr, "SHOW", 4) ||
-			    ! g_ascii_strncasecmp (tststr, "DESCRIBE", 6) ||
-			    ! g_ascii_strncasecmp (tststr, "EXPLAIN", 7)) {
-				mysql_res = mysql_store_result (mysql);
-				recset = gda_mysql_recordset_new (cnc, mysql_res, mysql);
-				if (GDA_IS_MYSQL_RECORDSET (recset)) {
-					g_object_set (G_OBJECT (recset), 
-						      "command_text", arr[n],
-						      "command_type", GDA_COMMAND_TYPE_SQL, NULL);
-					reclist = g_list_append (reclist, recset);
-				}
-				else
-					reclist = g_list_append (reclist, NULL);
+				allok = FALSE;
 			}
 			else {
-				int changes;
-				GdaConnectionEvent *event;
-				gchar *str, *tmp, *ptr;
-								
-				changes = mysql_affected_rows (mysql);
-				reclist = g_list_append (reclist, 
-					  gda_parameter_list_new_inline (NULL, 
-									 "IMPACTED_ROWS", G_TYPE_INT, changes,
-									 NULL));
-
-				/* generate a notice about changes */
-				event = gda_connection_event_new (GDA_CONNECTION_EVENT_NOTICE);
-				ptr = tststr;
-				while (*ptr && (*ptr != ' ') && (*ptr != '\t') &&
-				       (*ptr != '\n'))
-					ptr++;
-				*ptr = 0;
-				tmp = g_ascii_strup (tststr, -1);
-				if (!strcmp (tmp, "INSERT")) {
-					if (mysql_insert_id (mysql) != 0)
-						str = g_strdup_printf ("%s %lld %d", tmp, 
-								       mysql_insert_id (mysql),
-								       changes);
+				/* command was executed OK */
+				g_strchug (arr[n]);
+				tststr = arr[n];
+				if (! g_ascii_strncasecmp (tststr, "SELECT", 6) ||
+				    ! g_ascii_strncasecmp (tststr, "SHOW", 4) ||
+				    ! g_ascii_strncasecmp (tststr, "DESCRIBE", 6) ||
+				    ! g_ascii_strncasecmp (tststr, "EXPLAIN", 7)) {
+					mysql_res = mysql_store_result (mysql);
+					recset = gda_mysql_recordset_new (cnc, mysql_res, mysql);
+					if (GDA_IS_MYSQL_RECORDSET (recset)) {
+						g_object_set (G_OBJECT (recset), 
+							      "command_text", arr[n],
+							      "command_type", GDA_COMMAND_TYPE_SQL, NULL);
+						reclist = g_list_append (reclist, recset);
+					}
 					else
-						str = g_strdup_printf ("%s NOID %d", tmp, changes);
+						reclist = g_list_append (reclist, NULL);
+				}
+				else {
+					int changes;
+					GdaConnectionEvent *event;
+					gchar *str, *tmp, *ptr;
+					
+					changes = mysql_affected_rows (mysql);
+					reclist = g_list_append (reclist, 
+								 gda_parameter_list_new_inline (NULL, 
+												"IMPACTED_ROWS", G_TYPE_INT, changes,
+												NULL));
+					
+					/* generate a notice about changes */
+					event = gda_connection_event_new (GDA_CONNECTION_EVENT_NOTICE);
+					ptr = tststr;
+					while (*ptr && (*ptr != ' ') && (*ptr != '\t') &&
+					       (*ptr != '\n'))
+						ptr++;
+					*ptr = 0;
+					tmp = g_ascii_strup (tststr, -1);
+					if (!strcmp (tmp, "INSERT")) {
+						if (mysql_insert_id (mysql) != 0)
+							str = g_strdup_printf ("%s %lld %d", tmp, 
+									       mysql_insert_id (mysql),
+									       changes);
+						else
+							str = g_strdup_printf ("%s NOID %d", tmp, changes);
+					}
+					else
+						str = g_strdup_printf ("%s %d", tmp, changes);
+					gda_connection_event_set_description (event, str);
+					g_free (str);
+					gda_connection_add_event (cnc, event);
 				}
-				else
-					str = g_strdup_printf ("%s %d", tmp, changes);
-				gda_connection_event_set_description (event, str);
-				g_free (str);
-				gda_connection_add_event (cnc, event);
 			}
 
+			gda_connection_internal_treat_sql (cnc, arr[n], error);
 			n++;
 		}
 
@@ -829,15 +846,16 @@ gda_mysql_provider_get_last_insert_id (GdaServerProvider *provider,
 static gboolean
 gda_mysql_provider_begin_transaction (GdaServerProvider *provider,
 				      GdaConnection *cnc,
-				      GdaTransaction *xaction)
+				      const gchar *name, GdaTransactionIsolation level,
+				      GError **error)
 {
 	MYSQL *mysql;
 	gint rc;
 	GdaMysqlProvider *myprv = (GdaMysqlProvider *) provider;
+	GdaConnectionEvent *event = NULL;
 
 	g_return_val_if_fail (GDA_IS_MYSQL_PROVIDER (myprv), FALSE);
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	g_return_val_if_fail (GDA_IS_TRANSACTION (xaction), FALSE);
 
 	mysql = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_MYSQL_HANDLE);
 	if (!mysql) {
@@ -851,7 +869,7 @@ gda_mysql_provider_begin_transaction (GdaServerProvider *provider,
 	}
 
 	/* set isolation level */
-	switch (gda_transaction_get_isolation_level (xaction)) {
+	switch (level) {
 	case GDA_TRANSACTION_ISOLATION_READ_COMMITTED :
 		rc = mysql_real_query (mysql, "SET TRANSACTION ISOLATION LEVEL READ COMMITTED", 46);
 		break;
@@ -869,29 +887,32 @@ gda_mysql_provider_begin_transaction (GdaServerProvider *provider,
 	}
 
 	if (rc != 0) {
-		gda_connection_add_event (cnc, gda_mysql_make_error (mysql));
-		return FALSE;
+		event = gda_mysql_make_error (mysql);
+		gda_connection_add_event (cnc, event);
 	}
-
-	/* start the transaction */
-	rc = mysql_real_query (mysql, "BEGIN", strlen ("BEGIN"));
-	if (rc != 0) {
-		gda_connection_add_event (cnc, gda_mysql_make_error (mysql));
-		return FALSE;
+	else {
+		/* start the transaction */
+		rc = mysql_real_query (mysql, "BEGIN", strlen ("BEGIN"));
+		if (rc != 0) {
+			event = gda_mysql_make_error (mysql);
+			gda_connection_add_event (cnc, event);
+		}
 	}
-
-	return TRUE;
+	
+	gda_connection_internal_treat_sql (cnc, "BEGIN", event);
+	return event ? FALSE : TRUE;
 }
 
 /* commit_transaction handler for the GdaMysqlProvider class */
 static gboolean
 gda_mysql_provider_commit_transaction (GdaServerProvider *provider,
 				       GdaConnection *cnc,
-				       GdaTransaction *xaction)
+				       const gchar *name, GError **error)
 {
 	MYSQL *mysql;
 	gint rc;
 	GdaMysqlProvider *myprv = (GdaMysqlProvider *) provider;
+	GdaConnectionEvent *event = NULL;
 
 	g_return_val_if_fail (GDA_IS_MYSQL_PROVIDER (myprv), FALSE);
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
@@ -909,22 +930,24 @@ gda_mysql_provider_commit_transaction (GdaServerProvider *provider,
 
 	rc = mysql_real_query (mysql, "COMMIT", strlen ("COMMIT"));
 	if (rc != 0) {
-		gda_connection_add_event (cnc, gda_mysql_make_error (mysql));
-		return FALSE;
+		event = gda_mysql_make_error (mysql);
+		gda_connection_add_event (cnc, event);
 	}
 
-	return TRUE;
+	gda_connection_internal_treat_sql (cnc, "COMMIT", event);
+	return event ? FALSE : TRUE;
 }
 
 /* rollback_transaction handler for the GdaMysqlProvider class */
 static gboolean
 gda_mysql_provider_rollback_transaction (GdaServerProvider *provider,
 					 GdaConnection *cnc,
-					 GdaTransaction *xaction)
+					 const gchar *name, GError **error)
 {
 	MYSQL *mysql;
 	gint rc;
 	GdaMysqlProvider *myprv = (GdaMysqlProvider *) provider;
+	GdaConnectionEvent *event = NULL;
 
 	g_return_val_if_fail (GDA_IS_MYSQL_PROVIDER (myprv), FALSE);
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
@@ -942,11 +965,12 @@ gda_mysql_provider_rollback_transaction (GdaServerProvider *provider,
 
 	rc = mysql_real_query (mysql, "ROLLBACK", strlen ("ROLLBACK"));
 	if (rc != 0) {
-		gda_connection_add_event (cnc, gda_mysql_make_error (mysql));
-		return FALSE;
+		event = gda_mysql_make_error (mysql);
+		gda_connection_add_event (cnc, event);
 	}
 
-	return TRUE;
+	gda_connection_internal_treat_sql (cnc, "ROLLBACK", event);
+	return event ? FALSE : TRUE;
 }
 
 /* supports handler for the GdaMysqlProvider class */
@@ -979,7 +1003,8 @@ gda_mysql_provider_get_info (GdaServerProvider *provider, GdaConnection *cnc)
 		TRUE,
 		TRUE,
 		TRUE,
-		TRUE
+		TRUE,
+		FALSE
 	};
 	
 	return &info;
@@ -1363,7 +1388,7 @@ get_mysql_types (GdaConnection *cnc, GdaParameterList *params)
 		const gchar *synonyms;
 	} types[] = {
 		{ "bool", "", "Boolean type", G_TYPE_BOOLEAN, "boolean" },
-		{ "blob", "", "Binary blob (up to 65535 bytes)", GDA_TYPE_BLOB, NULL },
+		{ "blob", "", "Binary blob (up to 65535 bytes)", GDA_TYPE_BINARY, NULL },
 		{ "bigint", "", "Big integer, range is -9223372036854775808 to 9223372036854775807", G_TYPE_INT64, NULL  },
 		{ "bigint unsigned", "", "Big unsigned integer, range is 0 to 18446744073709551615", G_TYPE_UINT64, NULL  },
 		{ "char", "", "Char", G_TYPE_STRING, "binary"  },
@@ -1379,20 +1404,20 @@ get_mysql_types (GdaConnection *cnc, GdaParameterList *params)
 		{ "int unsigned", "", "Unsigned integer, range is 0 to 4294967295", G_TYPE_UINT, "integer unsigned"  },
 		{ "long", "", "Long integer", G_TYPE_LONG, NULL  },
 		{ "long unsigned", "", "Long unsigned integer", G_TYPE_ULONG, NULL  },
-		{ "longblob", "", "Long blob (up to 4Gb)", GDA_TYPE_BLOB, NULL  },
+		{ "longblob", "", "Long blob (up to 4Gb)", GDA_TYPE_BINARY, NULL  },
 		{ "longtext", "", "Long text (up to 4Gb characters)", GDA_TYPE_BINARY, NULL  },
 		{ "mediumint", "", "Medium integer, range is -8388608 to 8388607", G_TYPE_INT, NULL  },
 		{ "mediumint unsigned", "", "Medium unsigned integer, range is 0 to 16777215", G_TYPE_INT, NULL  },
-		{ "mediumblob", "", "Medium blob (up to 16777215 bytes)", GDA_TYPE_BLOB, NULL  },
+		{ "mediumblob", "", "Medium blob (up to 16777215 bytes)", GDA_TYPE_BINARY, NULL  },
 		{ "mediumtext", "", "Medium text (up to 16777215 characters)", GDA_TYPE_BINARY, NULL  },				
 		{ "set", "", "Set: a string object that can have zero or more values, each of which must be chosen from the list of values 'value1', 'value2', ... A SET column can have a maximum of 64 members", G_TYPE_STRING, NULL  },
 		{ "smallint", "", "Small integer, range is -32768 to 32767", GDA_TYPE_SHORT, NULL  },
 		{ "smallint unsigned", "", "Small unsigned integer, range is 0 to 65535", GDA_TYPE_USHORT, NULL  },
-		{ "text", "", "Text (up to 65535 characters)", G_TYPE_STRING, NULL  },
+		{ "text", "", "Text (up to 65535 characters)", GDA_TYPE_BINARY, NULL  },
 		{ "tinyint", "", "Tiny integer, range is -128 to 127", G_TYPE_CHAR, NULL  },
 		{ "tinyint unsigned", "", "Tiny unsigned integer, range is 0 to 255", G_TYPE_UCHAR, NULL  },
 		{ "tinyblob", "", "Tiny blob (up to 255 bytes)", GDA_TYPE_BINARY, NULL  },
-		{ "tinytext", "", "Tiny text (up to 255 characters)", GDA_TYPE_BLOB, NULL  },		
+		{ "tinytext", "", "Tiny text (up to 255 characters)", GDA_TYPE_BINARY, NULL  },		
 		{ "time", "", "Time", GDA_TYPE_TIME, NULL  },
 		{ "timestamp", "", "Time stamp", GDA_TYPE_TIMESTAMP, NULL  },
 		{ "varchar", "", "Variable Length Char", G_TYPE_STRING, "varbinary"  },
@@ -1448,7 +1473,7 @@ field_row_to_value_list (MYSQL *mysql, MYSQL_ROW mysql_row)
 	value_list = g_list_append (value_list, tmpval);
 
 	/* type and size */
-	arr = g_strsplit ((const gchar *) mysql_row[1], "(", 0);
+	arr = g_strsplit_set ((const gchar *) mysql_row[1], "() ", 0);
 	if (!arr) {
 		g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), "");
 		value_list = g_list_append (value_list, tmpval);
@@ -1467,7 +1492,17 @@ field_row_to_value_list (MYSQL *mysql, MYSQL_ROW mysql_row)
 		}
 		else {
 			if (arr[0] && arr[1]) {
-				g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), arr[0]);
+				gint i, len;
+				len = g_strv_length (arr);
+				for (i = 2; (i < len) && strcmp (arr[i], "unsigned"); i++);
+				if (i < len) {
+					gchar *type;
+
+					type = g_strdup_printf ("%s unsigned", arr[0]);
+					g_value_take_string (tmpval = gda_value_new (G_TYPE_STRING), type);
+				}
+				else
+					g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), arr[0]);
 				value_list = g_list_append (value_list, tmpval);
 
 				/* if arr[0] is an int type, hard code its size instead of
@@ -1650,10 +1685,200 @@ get_table_fields (GdaConnection *cnc, GdaParameterList *params)
 	}
 
 	mysql_free_result (mysql_res);
-
+	
 	return GDA_DATA_MODEL (recset);
 }
 
+static void
+mysql_get_constraints_form_create_table_line (GdaDataModelArray *recset, gchar *pos)
+{
+	/* try to parse a line like this:
+	 * "  CONSTRAINT `name` FOREIGN KEY (`fid1`,`fid2`) REFERENCES othertable(`id1`,`id2`)"
+	 */
+	gchar *cname = 0, *start, *stop;
+	GString *kname;
+	gchar *tname = 0;
+	GString *fname;
+	GList *value_list;
+	gchar *temp;
+	GValue *tmpval;
+
+	/* remove prefix spaces */
+	while ( *pos==' ')
+		pos++;
+
+	/* "CONSTRAINT "? */
+	if ( strncmp (pos, "CONSTRAINT ", 11) )
+		return;
+
+	pos += 11; /* strlen("CONSTRAINT ") */ 
+
+	/* find the start of the constraint name.. it should be a ` char */
+	if ( ! ( start = strchr (pos, '`') ) )
+		return;
+	/* we want the first character from the name */
+	start++;
+
+	/* find the ending constraint name.. it should be a ` char */
+	if ( ! ( stop = strchr (start, '`') ) )
+		return;
+
+	/* allocate cname, and store the constraint name */
+	cname = g_malloc ( stop - start  + 1);
+	g_strlcpy (cname, start, stop - start + 1);
+
+	pos = stop + 1;
+
+	/* remove spaces again */
+	while ( *pos == ' ' )
+		pos++;
+
+	/* "FOREIGN KEY "? */
+	if ( strncmp (pos, "FOREIGN KEY ", 12) )
+		goto cname_out;
+	/* this is a foreign key constraint */
+	pos += 12; /* strlen("FOREIGN KEY ") */
+
+	/* we should be able to locate a ( inside our string */
+	if ( ! ( pos = strchr (pos, '(') ) )
+		goto cname_out;
+	pos++;
+
+	/* we should now be at the character behind the ( */
+
+	kname = g_string_new (NULL);
+
+	while (1) {
+		/* find all the entries.. they are in this format "`a`, `b`, `c`".
+		 * The list is terminated by a ) character */
+		if ( ! ( start = strchr (pos, '`') ) )
+			goto kname_out;
+		start++;
+		if ( ! ( stop = strchr (start, '`') ) )
+			goto kname_out;
+		if ( kname->len )
+			g_string_append_c (kname, ',');
+		g_string_append_len (kname, start, stop - start);
+
+		pos = stop + 1;
+
+		while ( *pos == ' ')
+			pos++;
+		if ( *pos == ',' )
+			pos++;
+		else /* if (*pos==')')*/
+			break;
+	}
+	/* pass by the ) character.. but if we had an unexpected \0 character, don't crash */
+	if (*pos)
+		pos++;
+
+	/* remove spaces again */
+	while ( *pos == ' ' )
+		pos++;
+
+	/* we now expected the REFERENCES text */
+	if ( strncmp (pos, "REFERENCES ", 11) )
+		goto kname_out;
+
+	pos += 11;
+
+	/* and yeah... remove spaces */
+	while ( *pos == ' ' )
+		pos++;
+
+	/* we should now hit the name of the foreign table. It is embedded inside ` characters */
+	if ( ! ( start = strchr (pos, '`') ) )
+		goto kname_out;
+	start++;
+
+	if ( ! ( stop = strchr (start, '`') ) )
+		goto kname_out;
+	/* store it in tname */
+	tname = g_malloc ( stop - start + 1);
+	g_strlcpy (tname, start, stop - start + 1);
+
+	pos = stop + 1;
+
+	/* we now should find a ( character */
+	if ( ! ( start = strchr (pos, '(') ) )
+		goto tname_out;
+	pos = start + 1;
+
+	fname = g_string_new (NULL);
+
+	while (1) {
+		/* and after this character we find a list like above with keys */
+		if ( ! ( start = strchr (pos, '`') ) )
+			goto kname_out;
+		start++;
+		if ( ! ( stop = strchr (start, '`') ) )
+			goto kname_out;
+		if (fname->len)
+			g_string_append_c (fname, ',');
+		g_string_append_len (fname, start, stop - start);
+
+		pos = stop + 1;
+		while ( *pos == ' ')
+			pos++;
+		if ( *pos == ',' )
+			pos++;
+		else /* if (*pos==')')*/
+			break;
+	}
+	/* fill in the result into the result table */
+	g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), cname);
+	value_list = g_list_append (NULL, tmpval);
+
+	g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), "FOREIGN_KEY");
+	value_list = g_list_append (value_list, tmpval);
+
+	g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), kname->str);
+	value_list = g_list_append (value_list, tmpval);
+	
+	temp = g_strdup_printf ("%s(%s)", tname, fname->str);
+	g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), temp);
+	value_list = g_list_append (value_list, tmpval);
+	
+	value_list = g_list_append (value_list, gda_value_new_null ());
+
+	gda_data_model_append_values (GDA_DATA_MODEL (recset),
+	                              (const GList *) value_list, NULL);
+		
+	g_list_foreach (value_list, (GFunc) gda_value_free, NULL);
+	g_list_free (value_list);
+
+fname_out:
+	g_string_free (fname, TRUE);
+tname_out:
+	g_free (tname);
+kname_out:
+	g_string_free (kname, TRUE);
+cname_out:
+	g_free (cname);
+}
+
+static void
+mysql_get_constraints_from_create_table (GdaDataModelArray *recset, gchar *pos)
+{
+	/* skip the first line. nothing of interrest there */
+	if ((pos=strchr(pos, '\n')))
+		pos++;
+	else
+		return;
+	while (*pos)
+	{
+		gchar *next;
+		if ((next = strchr (pos, '\n'))) {
+			*next=0;
+			next++;
+		} else
+			next = pos + strlen (pos);
+		mysql_get_constraints_form_create_table_line (recset, pos);
+		pos=next;
+	}
+}
+
 static GdaDataModel *
 get_mysql_constraints (GdaConnection *cnc, GdaParameterList *params)
 {
@@ -1700,7 +1925,7 @@ get_mysql_constraints (GdaConnection *cnc, GdaParameterList *params)
 
 	/* 
 	 * Obtaining list of columns 
-	 */	
+	 */
 	cmd_str = g_strdup_printf ("SHOW COLUMNS FROM %s", table_name);
 	rc = mysql_real_query (mysql, cmd_str, strlen (cmd_str));
 	g_free (cmd_str);
@@ -1815,6 +2040,7 @@ get_mysql_constraints (GdaConnection *cnc, GdaParameterList *params)
 	if (atoi (mysql->server_version) >= 5) {
 		gchar *current_cname = NULL;
 		GString *ref_string = NULL;
+		GString *src_string = NULL;
 		GList *value_list = NULL;
 		
 		cmd_str = g_strdup_printf ("SELECT CONSTRAINT_NAME, COLUMN_NAME, ORDINAL_POSITION, REFERENCED_TABLE_SCHEMA, "
@@ -1845,12 +2071,16 @@ get_mysql_constraints (GdaConnection *cnc, GdaParameterList *params)
 
 				return NULL;
 			}
-		
-	
+
+
 			if ((! current_cname) || strcmp (current_cname, mysql_row[0])) {
 				/* new constraint */
 				if (value_list) {
 					/* complete and store current row */
+					g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), src_string->str);
+					value_list = g_list_append (value_list, tmpval);
+					g_string_free (src_string, TRUE);
+
 					g_string_append_c (ref_string, ')');
 					g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), ref_string->str);
 					value_list = g_list_append (value_list, tmpval);
@@ -1860,13 +2090,13 @@ get_mysql_constraints (GdaConnection *cnc, GdaParameterList *params)
 
 					gda_data_model_append_values (GDA_DATA_MODEL (recset),
 								      (const GList *) value_list, NULL);
-				
+
 					g_list_foreach (value_list, (GFunc) gda_value_free, NULL);
 					g_list_free (value_list);
 				}
 				if (current_cname) 
 					g_free (current_cname);
-			
+
 				/* prepare new constraint */
 				g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), mysql_row[0]);
 				value_list = g_list_append (NULL, tmpval);
@@ -1874,9 +2104,8 @@ get_mysql_constraints (GdaConnection *cnc, GdaParameterList *params)
 				g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), "FOREIGN_KEY");
 				value_list = g_list_append (value_list, tmpval);
 
-				g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), mysql_row[1]);
-				value_list = g_list_append (value_list, tmpval);
-			
+				src_string = g_string_new (mysql_row[1]);
+
 				ref_string = g_string_new (mysql_row[4]);
 				g_string_append_c (ref_string, '(');
 				g_string_append (ref_string, mysql_row[5]);
@@ -1885,11 +2114,17 @@ get_mysql_constraints (GdaConnection *cnc, GdaParameterList *params)
 			else {
 				g_string_append_c (ref_string, ',');
 				g_string_append (ref_string, mysql_row[5]);
+				g_string_append_c (src_string, ',');
+				g_string_append (src_string, mysql_row[1]);
 			}
 		}
 
 		if (value_list) {
 			/* complete and store current row */
+			g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), src_string->str);
+			value_list = g_list_append (value_list, tmpval);
+			g_string_free (src_string, TRUE);
+
 			g_string_append_c (ref_string, ')');
 			g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), ref_string->str);
 			value_list = g_list_append (value_list, tmpval);
@@ -1903,6 +2138,33 @@ get_mysql_constraints (GdaConnection *cnc, GdaParameterList *params)
 			g_list_foreach (value_list, (GFunc) gda_value_free, NULL);
 			g_list_free (value_list);
 		}
+		if (current_cname)
+			g_free(current_cname);
+		mysql_free_result (mysql_res);
+	} else {
+		gchar *temp;
+		MYSQL_ROW mysql_row;
+		cmd_str = g_strdup_printf ("SHOW CREATE TABLE %s",
+					   table_name);
+		rc = mysql_real_query (mysql, cmd_str, strlen (cmd_str));
+		g_free (cmd_str);
+		if (rc != 0) {
+			gda_connection_add_event (cnc, gda_mysql_make_error (mysql));
+			return NULL;
+		}
+		mysql_res = mysql_store_result (mysql);
+		
+		mysql_row = mysql_fetch_row (mysql_res);
+		if (!mysql_row) {
+			mysql_free_result (mysql_res);
+			g_object_unref (G_OBJECT (recset));
+
+			return NULL;
+		}
+		temp = g_strdup ( mysql_row[1] );
+		mysql_get_constraints_from_create_table (recset, mysql_row[1]);
+		g_free (temp);
+		mysql_free_result (mysql_res);
 	}
 
 	return GDA_DATA_MODEL (recset);
@@ -2062,6 +2324,9 @@ gda_mysql_provider_get_data_handler (GdaServerProvider *provider,
 		dh = gda_server_provider_handler_find (provider, NULL, type, NULL);
 		if (!dh) {
 			dh = gda_handler_time_new ();
+			gda_handler_time_set_sql_spec (GDA_HANDLER_TIME (dh), 
+						       G_DATE_YEAR, G_DATE_MONTH, G_DATE_DAY,
+						       '-', FALSE);
 			gda_server_provider_handler_declare (provider, dh, NULL, G_TYPE_DATE, NULL);
 			gda_server_provider_handler_declare (provider, dh, NULL, GDA_TYPE_TIME, NULL);
 			gda_server_provider_handler_declare (provider, dh, NULL, GDA_TYPE_TIMESTAMP, NULL);
