@@ -55,13 +55,14 @@ static gboolean gda_sybase_provider_close_connection (GdaServerProvider *provide
 
 static gboolean gda_sybase_provider_begin_transaction (GdaServerProvider *provider,
                                                        GdaConnection *cnc,
-                                                       GdaTransaction *xaction);
+                                                       const gchar *name, GdaTransactionIsolation level,
+						       GError **error);
 static gboolean gda_sybase_provider_change_database (GdaServerProvider *provider,
                                                      GdaConnection *cnc,
                                                      const gchar *name);
 static gboolean gda_sybase_provider_commit_transaction (GdaServerProvider *provider,
                                                         GdaConnection *cnc,
-                                                        GdaTransaction *xaction);
+                                                        const gchar *name, GError **error);
 static GList *gda_sybase_provider_execute_command (GdaServerProvider *provider,
                                                    GdaConnection *cnc,
                                                    GdaCommand *cmd,
@@ -77,7 +78,7 @@ static const gchar *gda_sybase_provider_get_server_version (GdaServerProvider *p
 static const gchar *gda_sybase_provider_get_version (GdaServerProvider *provider);
 static gboolean gda_sybase_provider_rollback_transaction (GdaServerProvider *provider,
                                                           GdaConnection *cnc,
-                                                          GdaTransaction *xaction);
+                                                          const gchar *name, GError **error);
 static gboolean gda_sybase_provider_supports (GdaServerProvider *provider,
                                               GdaConnection *cnc,
                                               GdaConnectionFeature feature);
@@ -130,6 +131,9 @@ gda_sybase_provider_class_init (GdaSybaseProviderClass *klass)
 	provider_class->begin_transaction = gda_sybase_provider_begin_transaction;
 	provider_class->commit_transaction = gda_sybase_provider_commit_transaction;
 	provider_class->rollback_transaction = gda_sybase_provider_rollback_transaction;
+	provider_class->add_savepoint = NULL;
+	provider_class->rollback_savepoint = NULL;
+	provider_class->delete_savepoint = NULL;
 	
 	provider_class->create_blob = NULL;
 	provider_class->fetch_blob = NULL;
@@ -543,9 +547,10 @@ gda_sybase_provider_close_connection (GdaServerProvider *provider,
 static gboolean
 gda_sybase_provider_begin_transaction(GdaServerProvider *provider,
                                       GdaConnection *cnc,
-                                      GdaTransaction *xaction)
+                                      const gchar *name, GdaTransactionIsolation level,
+				      GError **error)
 {
-	return ( gda_sybase_execute_cmd(cnc,"begin transaction") );
+	return (gda_sybase_execute_cmd (cnc, "begin transaction"));
 }
 
 static gboolean
@@ -570,9 +575,9 @@ gda_sybase_provider_change_database (GdaServerProvider *provider,
 static gboolean 
 gda_sybase_provider_commit_transaction (GdaServerProvider *provider,
                                         GdaConnection *cnc,
-                                        GdaTransaction *xaction)
+                                        const gchar *name, GError **error)
 {
-	return ( gda_sybase_execute_cmd(cnc,"commit transaction") );		
+	return (gda_sybase_execute_cmd (cnc, "commit transaction"));		
 }
 
 static GList *
@@ -756,7 +761,7 @@ gda_sybase_provider_process_sql_commands(GList         *reclist,
 						       &msgcnt);
 					
 					if ( ret != CS_SUCCEED ) {
-						error = gda_connection_event_new();
+						error = gda_connection_event_new (GDA_CONNECTION_EVENT_ERROR);
 						g_return_val_if_fail (error != NULL, FALSE);
 						gda_connection_event_set_description (error, _("An error occurred when attempting to test if there is a server message for resultset"));
 						gda_connection_event_set_code (error, -1);
@@ -1117,7 +1122,7 @@ gda_sybase_provider_get_types (GdaConnection *cnc,
 			g_value_set_ulong (tmpval = gda_value_new (G_TYPE_ULONG), gda_sybase_type_list[i].g_type);
 			value_list = g_list_append (value_list, tmpval);
 			
-			gda_data_model_append_values (GDA_DATA_MODEL (recset), value_list);
+			gda_data_model_append_values (GDA_DATA_MODEL (recset), value_list, NULL);
 			g_list_foreach (value_list, (GFunc) gda_value_free, NULL);
 			g_list_free (value_list);
 		}
@@ -1195,7 +1200,9 @@ gda_sybase_provider_get_schema (GdaServerProvider *provider,
 	case GDA_CONNECTION_SCHEMA_PARENT_TABLES:
 	case GDA_CONNECTION_SCHEMA_SEQUENCES:
 	case GDA_CONNECTION_SCHEMA_LANGUAGES:
-	case	GDA_CONNECTION_SCHEMA_NAMESPACES:
+	case GDA_CONNECTION_SCHEMA_NAMESPACES:
+	case GDA_CONNECTION_SCHEMA_CONSTRAINTS:
+	case GDA_CONNECTION_SCHEMA_TABLE_CONTENTS:
 		return NULL;
 		break;
 	}
@@ -1273,9 +1280,9 @@ gda_sybase_provider_get_version (GdaServerProvider *provider)
 static gboolean 
 gda_sybase_provider_rollback_transaction (GdaServerProvider *provider,
                                           GdaConnection *cnc,
-                                          GdaTransaction *xaction)
+                                          const gchar *name, GError **error)
 {
-	return ( gda_sybase_execute_cmd(cnc,"rollback transaction") );		
+	return (gda_sybase_execute_cmd (cnc, "rollback transaction"));		
 }
 
 static gboolean 
@@ -1297,12 +1304,13 @@ gda_sybase_provider_supports (GdaServerProvider *provider,
 	case GDA_CONNECTION_FEATURE_TRIGGERS:
 		return TRUE;
 
-		//FIXME: Implement missing 
 	case GDA_CONNECTION_FEATURE_AGGREGATES:
 	case GDA_CONNECTION_FEATURE_INHERITANCE:
 	case GDA_CONNECTION_FEATURE_SEQUENCES:
 	case GDA_CONNECTION_FEATURE_XML_QUERIES:
 	case GDA_CONNECTION_FEATURE_NAMESPACES:
+	case GDA_CONNECTION_FEATURE_BLOBS:
+	case GDA_CONNECTION_FEATURE_UPDATABLE_CURSOR:
 		return FALSE;
 	}
 	
